@@ -21,6 +21,7 @@
 #include "git-commit-id.h"
 #include "core-builtin.h"
 #include "core-capabilities.h"
+#include "core-cache.h"
 #include "core-hash.h"
 
 #if defined(HAVE_LINUX_FIEMAP_H)
@@ -39,10 +40,17 @@
 #include <sys/loadavg.h>
 #endif
 
-#if defined(HAVE_SYS_MOUNT_H)
+#if (defined(__FreeBSD__) || 	\
+     defined(__OpenBSD__)) &&	\
+     defined(HAVE_SYS_MOUNT_H)
 #include <sys/mount.h>
 #endif
 
+#if defined(__FreeBSD__) &&	\
+    defined(HAVE_SYS_PARAM_H)
+#include <sys/param.h>
+#endif
+
 #if defined(HAVE_SYS_PRCTL_H)
 #include <sys/prctl.h>
 #endif
@@ -51,6 +59,10 @@
 #include <sys/statvfs.h>
 #endif
 
+#if defined(HAVE_SYS_SYSCTL_H)
+#include <sys/sysctl.h>
+#endif
+
 #if defined(HAVE_SYS_UTSNAME_H)
 #include <sys/utsname.h>
 #endif
@@ -67,6 +79,10 @@
 #include <linux/magic.h>
 #endif
 
+#if defined(HAVE_UVM_UVM_EXTERN_H)
+#include <uvm/uvm_extern.h>
+#endif
+
 /* prctl(2) timer slack support */
 #if defined(HAVE_SYS_PRCTL_H) && \
     defined(HAVE_PRCTL) && \
@@ -75,13 +91,6 @@
 #define HAVE_PRCTL_TIMER_SLACK
 #endif
 
-#if !defined(PR_SET_DISABLE)
-#define SUID_DUMP_DISABLE	(0)       /* No setuid dumping */
-#endif
-#if !defined(SUID_DUMP_USER)
-#define SUID_DUMP_USER		(1)       /* Dump as user of process */
-#endif
-
 #if defined(NSIG)
 #define STRESS_NSIG	NSIG
 #elif defined(_NSIG)
@@ -92,6 +101,9 @@
 int __dso_handle;
 #endif
 
+#define PAGE_4K_SHIFT			(12)
+#define PAGE_4K				(1 << PAGE_4K_SHIFT)
+
 #define STRESS_ABS_MIN_STACK_SIZE	(64 * 1024)
 
 static bool stress_stack_check_flag;
@@ -314,6 +326,27 @@ static const stress_fs_name_t stress_fs_names[] = {
 #if defined(UDF_SUPER_MAGIC)
 	{ UDF_SUPER_MAGIC,	"udf" },
 #endif
+#if defined(UBIFS_SUPER_MAGIC)
+	{ UBIFS_SUPER_MAGIC,	"ubifs" },
+#else
+	{ 0x24051905,		"ubifs" },
+#endif
+	{ 0x1badface,		"bfs" },
+#if defined(HFS_SUPER_MAGIC)
+	{ HFS_SUPER_MAGIC,	"hfs" },
+#else
+	{ 0x4244,		"hfs" },
+#endif
+#if defined(HFSPLUS_SUPER_MAGIC)
+	{ HFSPLUS_SUPER_MAGIC,	"hfsplus" },
+#else
+	{ 0x482b,		"hfsplus" },
+#endif
+#if defined(JFS_SUPER_MAGIC)
+	{ JFS_SUPER_MAGIC,	"jfs" },
+#else
+	{ 0x3153464a,		"jfs" },
+#endif
 };
 #endif
 
@@ -534,7 +567,7 @@ size_t stress_get_page_size(void)
 #if defined(_SC_PAGESIZE)
 	{
 		/* Use modern sysconf */
-		long sz = sysconf(_SC_PAGESIZE);
+		const long sz = sysconf(_SC_PAGESIZE);
 		if (sz > 0) {
 			page_size = (size_t)sz;
 			return page_size;
@@ -546,7 +579,7 @@ size_t stress_get_page_size(void)
 #if defined(HAVE_GETPAGESIZE)
 	{
 		/* Use deprecated getpagesize */
-		long sz = getpagesize();
+		const long sz = getpagesize();
 		if (sz > 0) {
 			page_size = (size_t)sz;
 			return page_size;
@@ -622,6 +655,74 @@ int32_t stress_get_ticks_per_second(void)
 #endif
 }
 
+/*
+ *  stress_get_meminfo()
+ *	wrapper for linux sysinfo
+ */
+static int stress_get_meminfo(
+	size_t *freemem,
+	size_t *totalmem,
+	size_t *freeswap,
+	size_t *totalswap)
+{
+#if defined(HAVE_SYS_SYSINFO_H) &&	\
+    defined(HAVE_SYSINFO)
+	struct sysinfo info;
+
+	(void)memset(&info, 0, sizeof(info));
+
+	if (sysinfo(&info) == 0) {
+		*freemem = info.freeram * info.mem_unit;
+		*totalmem = info.totalram * info.mem_unit;
+		*freeswap = info.freeswap * info.mem_unit;
+		*totalswap = info.totalswap * info.mem_unit;
+		return 0;
+	}
+#endif
+#if defined(__FreeBSD__)
+	{
+		const size_t page_size = (size_t)stress_bsd_getsysctl_uint("vm.stats.vm.v_page_size");
+#if 0
+		/*
+		 *  Enable total swap only when we can determine free swap
+		 */
+		const size_t max_size_t = (size_t)-1;
+		const uint64_t vm_swap_total = stress_bsd_getsysctl_uint64("vm.swap_total");
+
+		*totalswap = (vm_swap_total >= max_size_t) ? max_size_t : (size_t)vm_swap_total;
+#endif
+		*freemem = page_size * stress_bsd_getsysctl_uint32("vm.stats.vm.v_free_count");
+		*totalmem = page_size *
+			(stress_bsd_getsysctl_uint32("vm.stats.vm.v_active_count") +
+			 stress_bsd_getsysctl_uint32("vm.stats.vm.v_inactive_count") +
+			 stress_bsd_getsysctl_uint32("vm.stats.vm.v_laundry_count") +
+			 stress_bsd_getsysctl_uint32("vm.stats.vm.v_wire_count") +
+			 stress_bsd_getsysctl_uint32("vm.stats.vm.v_free_count"));
+		return 0;
+	}
+#endif
+#if defined(__NetBSD__) &&	\
+    defined(HAVE_UVM_UVM_EXTERN_H)
+	{
+		struct uvmexp_sysctl u;
+
+		if (stress_bsd_getsysctl("vm.uvmexp2", &u, sizeof(u)) == 0) {
+			*freemem = (size_t)u.free * u.pagesize;
+			*totalmem = (size_t)u.npages * u.pagesize;
+			*totalswap = (size_t)u.swpages * u.pagesize;
+			*freeswap = *totalswap - (size_t)u.swpginuse * u.pagesize;
+			return 0;
+		}
+	}
+#endif
+	*freemem = 0;
+	*totalmem = 0;
+	*freeswap = 0;
+	*totalswap = 0;
+
+	return -1;
+}
+
 /*
  *  stress_get_memlimits()
  *	get SHMALL and memory in system
@@ -631,28 +732,14 @@ void stress_get_memlimits(
 	size_t *shmall,
 	size_t *freemem,
 	size_t *totalmem,
-	size_t *freeswap)
+	size_t *freeswap,
+	size_t *totalswap)
 {
-#if defined(HAVE_SYS_SYSINFO_H) &&	\
-    defined(HAVE_SYSINFO)
-	struct sysinfo info;
 	FILE *fp;
-#endif
+
+	(void)stress_get_meminfo(freemem, totalmem, freeswap, totalswap);
+
 	*shmall = 0;
-	*freemem = 0;
-	*totalmem = 0;
-	*freeswap = 0;
-
-#if defined(HAVE_SYS_SYSINFO_H) &&	\
-    defined(HAVE_SYSINFO)
-	(void)memset(&info, 0, sizeof(info));
-
-	if (sysinfo(&info) == 0) {
-		*freemem = info.freeram * info.mem_unit;
-		*totalmem = info.totalram * info.mem_unit;
-		*freeswap = info.freeswap * info.mem_unit;
-	}
-
 	fp = fopen("/proc/sys/kernel/shmall", "r");
 	if (!fp)
 		return;
@@ -662,9 +749,61 @@ void stress_get_memlimits(
 		return;
 	}
 	(void)fclose(fp);
-#else
-	UNEXPECTED
-#endif
+}
+
+/*
+ *  stress_low_memory()
+ *	return true if running low on memory
+ */
+bool stress_low_memory(const size_t requested)
+{
+	static size_t prev_freemem = 0;
+	static size_t prev_freeswap = 0;
+	size_t freemem, totalmem, freeswap, totalswap;
+	bool low_memory = false;
+
+	if (stress_get_meminfo(&freemem, &totalmem, &freeswap, &totalswap) == 0) {
+		/*
+		 *  Stats from previous call valid, then check for memory
+		 *  changes
+		 */
+		if ((prev_freemem + prev_freeswap) > 0) {
+			ssize_t delta;
+
+			delta = (ssize_t)prev_freemem - (ssize_t)freemem;
+			delta = (delta * 2) + requested;
+			/* memory shrinking quickly? */
+			if (delta  > (ssize_t)freemem) {
+				low_memory = true;
+				goto update;
+			}
+			/* swap shrinking quickly? */
+			delta = (ssize_t)prev_freeswap - (ssize_t)freeswap;
+			if (delta > 0) {
+				low_memory = true;
+				goto update;
+			}
+		}
+		/* Not enough for allocation and slop? */
+		if (freemem < ((2 * MB) + requested)) {
+			low_memory = true;
+			goto update;
+		}
+		/* Less than 1% left? */
+		if (((double)freemem * 100.0 / (double)(totalmem - requested)) < 1.0) {
+			low_memory = true;
+			goto update;
+		}
+		/* Any swap enabled and is it running low? */
+		if ((totalswap > 0) && (freeswap < (2 * MB) + requested)) {
+			low_memory = true;
+			goto update;
+		}
+update:
+		prev_freemem = freemem;
+		prev_freeswap = freeswap;
+	}
+	return low_memory;
 }
 
 #if defined(_SC_AVPHYS_PAGES)
@@ -876,6 +1015,14 @@ int stress_process_dumpable(const bool dumpable)
 #if 0 && defined(HAVE_PRCTL) &&		\
     defined(HAVE_SYS_PRCTL_H) &&	\
     defined(PR_SET_DUMPABLE)
+
+#if !defined(PR_SET_DISABLE)
+#define SUID_DUMP_DISABLE	(0)       /* No setuid dumping */
+#endif
+#if !defined(SUID_DUMP_USER)
+#define SUID_DUMP_USER		(1)       /* Dump as user of process */
+#endif
+
 	(void)prctl(PR_SET_DUMPABLE,
 		dumpable ? SUID_DUMP_USER : SUID_DUMP_DISABLE);
 #endif
@@ -949,21 +1096,23 @@ void stress_set_proc_name_init(int argc, char *argv[], char *envp[])
  */
 void stress_set_proc_name(const char *name)
 {
-	(void)name;
+	char long_name[64];
 
 	if (g_opt_flags & OPT_FLAGS_KEEP_NAME)
 		return;
+	(void)snprintf(long_name, sizeof(long_name), "%s-%s",
+			g_app_name, name);
 
 #if defined(HAVE_BSD_UNISTD_H) &&	\
     defined(HAVE_SETPROCTITLE)
 	/* Sets argv[0] */
-	setproctitle("-%s", name);
+	setproctitle("-%s", long_name);
 #endif
 #if defined(HAVE_PRCTL) &&		\
     defined(HAVE_SYS_PRCTL_H) &&	\
     defined(PR_SET_NAME)
 	/* Sets the comm field */
-	(void)prctl(PR_SET_NAME, name);
+	(void)prctl(PR_SET_NAME, long_name);
 #endif
 }
 
@@ -974,15 +1123,23 @@ void stress_set_proc_name(const char *name)
  */
 void stress_set_proc_state_str(const char *name, const char *str)
 {
+	char long_name[64];
+
+	(void)str;
 	if (g_opt_flags & OPT_FLAGS_KEEP_NAME)
 		return;
+	(void)snprintf(long_name, sizeof(long_name), "%s-%s",
+			g_app_name, name);
 
 #if defined(HAVE_BSD_UNISTD_H) &&	\
     defined(HAVE_SETPROCTITLE)
-	setproctitle("-%s [%s]", name, str);
-#else
-	(void)name;
-	(void)str;
+	setproctitle("-%s [%s]", long_name, str);
+#endif
+#if defined(HAVE_PRCTL) &&		\
+    defined(HAVE_SYS_PRCTL_H) &&	\
+    defined(PR_SET_NAME)
+	/* Sets the comm field */
+	(void)prctl(PR_SET_NAME, long_name);
 #endif
 }
 
@@ -993,7 +1150,7 @@ void stress_set_proc_state_str(const char *name, const char *str)
  */
 void stress_set_proc_state(const char *name, const int state)
 {
-	static const char *stress_states[] = {
+	static const char * const stress_states[] = {
 		"start",
 		"init",
 		"run",
@@ -1029,6 +1186,27 @@ char *stress_munge_underscore(const char *str)
 	return munged;
 }
 
+/*
+ *  stress_chr_munge()
+ *	convert ch _ to -, otherwise don't change it
+ */
+static inline char stress_chr_munge(const char ch)
+{
+	return (ch == '_') ? '-' : ch;
+}
+
+/*
+ *  stress_strcmp_munged()
+ *	compare strings with _ comcompared to -
+ */
+int stress_strcmp_munged(const char *s1, const char *s2)
+{
+	for (; *s1 && (stress_chr_munge(*s1) == stress_chr_munge(*s2)); s1++, s2++)
+		;
+
+	return (unsigned char)stress_chr_munge(*s1) - (unsigned char)stress_chr_munge(*s2);
+}
+
 /*
  *  stress_get_stack_direction_helper()
  *	helper to determine direction of stack
@@ -1120,12 +1298,9 @@ static void stress_temp_hash_truncate(char *filename)
 #endif
 
 	if (strlen(filename) > f_namemax) {
-		uint32_t upper, lower;
-		uint64_t val;
-
-		upper = stress_hash_jenkin((uint8_t *)filename, len);
-		lower = stress_hash_pjw(filename);
-		val = ((uint64_t)upper << 32) | lower;
+		const uint32_t upper = stress_hash_jenkin((uint8_t *)filename, len);
+		const uint32_t lower = stress_hash_pjw(filename);
+		const uint64_t val = ((uint64_t)upper << 32) | lower;
 
 		stress_base36_encode_uint64(filename, val);
 	}
@@ -1147,13 +1322,13 @@ int stress_temp_filename(
 	char filename[PATH_MAX];
 
 	(void)snprintf(directoryname, sizeof(directoryname),
-		"tmp-%s-%d-%" PRIu32,
-		name, (int)pid, instance);
+		"tmp-%s-%s-%d-%" PRIu32,
+		g_app_name, name, (int)pid, instance);
 	stress_temp_hash_truncate(directoryname);
 
 	(void)snprintf(filename, sizeof(filename),
-		"%s-%d-%" PRIu32 "-%" PRIu64,
-		name, (int)pid, instance, magic);
+		"%s-%s-%d-%" PRIu32 "-%" PRIu64,
+		g_app_name, name, (int)pid, instance, magic);
 	stress_temp_hash_truncate(filename);
 
 	return snprintf(path, len, "%s/%s/%s",
@@ -1186,14 +1361,16 @@ int stress_temp_dir(
 	const uint32_t instance)
 {
 	char directoryname[256];
+	int l;
 
 	(void)snprintf(directoryname, sizeof(directoryname),
-		"tmp-%s-%d-%" PRIu32,
-		name, (int)pid, instance);
+		"tmp-%s-%s-%d-%" PRIu32,
+		g_app_name, name, (int)pid, instance);
 	stress_temp_hash_truncate(directoryname);
 
-	return snprintf(path, len, "%s/%s",
+	l = snprintf(path, len, "%s/%s",
 		stress_get_temp_path(), directoryname);
+	return l;
 }
 
 /*
@@ -1351,16 +1528,20 @@ bool stress_little_endian(void)
 	return *y == 0x78;
 }
 
+/*
+ *  stress_swap32()
+ *	swap order of bytes of a uint32_t value
+ */
 static inline uint32_t OPTIMIZE3 stress_swap32(uint32_t val)
 {
-	register uint32_t swap;
-
-	swap = ((val >> 24) & 0x000000ff) |
+#if defined(HAVE_BUILTIN_BSWAP32)
+	return __builtin_bswap32(val);
+#else
+	return ((val >> 24) & 0x000000ff) |
 	       ((val << 8)  & 0x00ff0000) |
 	       ((val >> 8)  & 0x0000ff00) |
 	       ((val << 24) & 0xff000000);
-
-	return swap;
+#endif
 }
 
 /*
@@ -1389,13 +1570,12 @@ void OPTIMIZE3 stress_uint8rnd4(uint8_t *data, const size_t len)
  */
 void pr_runinfo(void)
 {
+	const char *temp_path = stress_get_temp_path();
+	const char *fs_type = stress_fs_type(temp_path);
+	size_t freemem, totalmem, freeswap, totalswap;
 #if defined(HAVE_UNAME) &&	\
     defined(HAVE_SYS_UTSNAME_H)
 	struct utsname uts;
-#endif
-#if defined(HAVE_SYS_SYSINFO_H) &&	\
-    defined(HAVE_SYSINFO)
-	struct sysinfo info;
 #endif
 	if (!(g_opt_flags & PR_DEBUG))
 		return;
@@ -1410,25 +1590,22 @@ void pr_runinfo(void)
 
 #if defined(HAVE_UNAME) &&	\
     defined(HAVE_SYS_UTSNAME_H)
-	if (uname(&uts) == 0) {
+	if (uname(&uts) >= 0) {
 		pr_dbg("system: %s %s %s %s %s\n",
 			uts.sysname, uts.nodename, uts.release,
 			uts.version, uts.machine);
 	}
 #endif
-#if defined(HAVE_SYS_SYSINFO_H) &&	\
-    defined(HAVE_SYSINFO)
-	/* Keep static analyzer happy */
-	(void)memset(&info, 0, sizeof(info));
-	if (sysinfo(&info) == 0) {
+	if (stress_get_meminfo(&freemem, &totalmem, &freeswap, &totalswap) == 0) {
 		char ram_t[32], ram_f[32], ram_s[32];
 
-		stress_uint64_to_str(ram_t, sizeof(ram_t), (uint64_t)info.totalram);
-		stress_uint64_to_str(ram_f, sizeof(ram_f), (uint64_t)info.freeram);
-		stress_uint64_to_str(ram_s, sizeof(ram_s), (uint64_t)info.freeswap);
+		stress_uint64_to_str(ram_t, sizeof(ram_t), (uint64_t)totalmem);
+		stress_uint64_to_str(ram_f, sizeof(ram_f), (uint64_t)freemem);
+		stress_uint64_to_str(ram_s, sizeof(ram_s), (uint64_t)freeswap);
 		pr_dbg("RAM total: %s, RAM free: %s, swap free: %s\n", ram_t, ram_f, ram_s);
 	}
-#endif
+
+	pr_dbg("temporary file path: '%s'%s\n", temp_path, fs_type);
 }
 
 /*
@@ -1448,7 +1625,7 @@ void pr_yaml_runinfo(FILE *yaml)
 	time_t t;
 	struct tm *tm = NULL;
 	const size_t hostname_len = stress_hostname_length();
-	char hostname[hostname_len];
+	char *hostname;
 	const char *user = shim_getlogin();
 
 	pr_yaml(yaml, "system-info:\n");
@@ -1464,11 +1641,18 @@ void pr_yaml_runinfo(FILE *yaml)
 			tm->tm_hour, tm->tm_min, tm->tm_sec);
 		pr_yaml(yaml, "      epoch-secs: %ld\n", (long)t);
 	}
-	if (!gethostname(hostname, sizeof(hostname)))
+
+	hostname = malloc(hostname_len + 1);
+	if (hostname && !gethostname(hostname, hostname_len)) {
 		pr_yaml(yaml, "      hostname: %s\n", hostname);
+	} else {
+		pr_yaml(yaml, "      hostname: %s\n", "unknown");
+	}
+	free(hostname);
+
 #if defined(HAVE_UNAME) &&	\
     defined(HAVE_SYS_UTSNAME_H)
-	if (uname(&uts) == 0) {
+	if (uname(&uts) >= 0) {
 		pr_yaml(yaml, "      sysname: %s\n", uts.sysname);
 		pr_yaml(yaml, "      nodename: %s\n", uts.nodename);
 		pr_yaml(yaml, "      release: %s\n", uts.release);
@@ -1682,7 +1866,7 @@ bool stress_is_prime64(const uint64_t n)
 		return false;
 	max_d = 1.0 + shim_sqrt((double)n);
 	max = (uint64_t)max_d;
-	for (i = 5; i < max; i+= 6)
+	for (i = 5; i < max; i += 6)
 		if ((n % i == 0) || (n % (i + 2) == 0))
 			return false;
 	return true;
@@ -1699,7 +1883,7 @@ bool stress_is_prime64(const uint64_t n)
 uint64_t stress_get_prime64(const uint64_t n)
 {
 	static uint64_t p = 1009;
-	uint64_t odd_n = (n & ~1UL) + 1;
+	const uint64_t odd_n = (n & ~1UL) + 1;
 
 	if (p < odd_n)
 		p = odd_n;
@@ -2060,7 +2244,7 @@ const char *stress_get_uname_info(void)
     defined(HAVE_SYS_UTSNAME_H)
 	struct utsname buf;
 
-	if (!uname(&buf)) {
+	if (uname(&buf) >= 0) {
 		static char str[sizeof(buf.machine) +
 	                        sizeof(buf.sysname) +
 				sizeof(buf.release) + 3];
@@ -2075,11 +2259,11 @@ const char *stress_get_uname_info(void)
 }
 
 /*
- *  stress_not_implemented()
+ *  stress_unimplemented()
  *	report that a stressor is not implemented
  *	on a particular arch or kernel
  */
-int stress_not_implemented(const stress_args_t *args)
+int stress_unimplemented(const stress_args_t *args)
 {
 	static const char msg[] = "this stressor is not implemented on "
 				  "this system";
@@ -2088,15 +2272,28 @@ int stress_not_implemented(const stress_args_t *args)
     defined(HAVE_SYS_UTSNAME_H)
 		struct utsname buf;
 
-		if (!uname(&buf)) {
-			pr_inf_skip("%s: %s: %s %s\n",
-				args->name, msg, stress_get_uname_info(),
-				stress_get_compiler());
+		if (uname(&buf) >= 0) {
+			if (args->info->unimplemented_reason) {
+				pr_inf_skip("%s: %s: %s %s (%s)\n",
+					args->name, msg, stress_get_uname_info(),
+					stress_get_compiler(),
+					args->info->unimplemented_reason);
+			} else {
+				pr_inf_skip("%s: %s: %s %s\n",
+					args->name, msg, stress_get_uname_info(),
+					stress_get_compiler());
+			}
 			return EXIT_NOT_IMPLEMENTED;
 		}
 #endif
-		pr_inf_skip("%s: %s: %s\n",
-			args->name, msg, stress_get_compiler());
+		if (args->info->unimplemented_reason) {
+			pr_inf_skip("%s: %s: %s (%s)\n",
+				args->name, msg, stress_get_compiler(),
+				args->info->unimplemented_reason);
+		} else {
+			pr_inf_skip("%s: %s: %s\n",
+				args->name, msg, stress_get_compiler());
+		}
 	}
 	return EXIT_NOT_IMPLEMENTED;
 }
@@ -2217,8 +2414,8 @@ bool stress_sigalrm_pending(void)
 char *stress_uint64_to_str(char *str, size_t len, const uint64_t val)
 {
 	typedef struct {
-		uint64_t size;
-		char *suffix;
+		const uint64_t size;
+		const char *suffix;
 	} stress_size_info_t;
 
 	static const stress_size_info_t size_info[] = {
@@ -2230,11 +2427,11 @@ char *stress_uint64_to_str(char *str, size_t len, const uint64_t val)
 		{ KB, "K" },
 	};
 	size_t i;
-	char *suffix = "";
+	const char *suffix = "";
 	uint64_t scale = 1;
 
 	for (i = 0; i < SIZEOF_ARRAY(size_info); i++) {
-		uint64_t scaled = val / size_info[i].size;
+		const uint64_t scaled = val / size_info[i].size;
 
 		if ((scaled >= 1) && (scaled < 1024)) {
 			suffix = size_info[i].suffix;
@@ -2299,6 +2496,10 @@ bool stress_check_capability(const int capability)
 }
 #endif
 
+/*
+ *  stress_drop_capabilities()
+ *	drop all capabilities and disable any new privileges
+ */
 #if defined(HAVE_SYS_CAPABILITY_H)
 int stress_drop_capabilities(const char *name)
 {
@@ -2326,8 +2527,8 @@ int stress_drop_capabilities(const char *name)
 	 *  bits to zero to show the intent
 	 */
 	for (i = 0; i <= CAP_LAST_CAP; i++) {
-		uint32_t idx = CAP_TO_INDEX(i);
-		uint32_t mask = CAP_TO_MASK(i);
+		const uint32_t idx = CAP_TO_INDEX(i);
+		const uint32_t mask = CAP_TO_MASK(i);
 
 		ucd[idx].inheritable &= ~mask;
 		ucd[idx].permitted &= ~mask;
@@ -2509,7 +2710,8 @@ bool stress_warn_once_hash(const char *filename, const int line)
 	if (!g_shared)
 		return true;
 
-	stress_lock_acquire(g_shared->warn_once.lock);
+	if (stress_lock_acquire(g_shared->warn_once.lock) < 0)
+		return true;
 	free_slot = STRESS_WARN_HASH_MAX;
 
 	/*
@@ -2562,6 +2764,10 @@ uint16_t HOT OPTIMIZE3 stress_ipv4_checksum(uint16_t *ptr, const size_t sz)
 	return (uint16_t)~sum;
 }
 
+/*
+ *  stress_uid_comp()
+ *	uid comparison for sorting
+ */
 #if defined(HAVE_SETPWENT) &&	\
     defined(HAVE_GETPWENT) &&	\
     defined(HAVE_ENDPWENT) &&	\
@@ -2665,7 +2871,7 @@ int stress_get_unused_uid(uid_t *uid)
 /*
  *  stress_read_buffer()
  *	In addition to read() this function makes sure all bytes have been
- *	written. You're also able to ignore EINTR interrupts which could happen
+ *	read. You're also able to ignore EINTR interrupts which could happen
  *	on alarm() in the parent process.
  */
 ssize_t stress_read_buffer(int fd, void* buffer, ssize_t size, bool ignore_int)
@@ -2860,7 +3066,7 @@ static inline long stress_min_aux_sig_stack_size(void)
 #if defined(HAVE_SYS_AUXV_H) && \
     defined(HAVE_GETAUXVAL) &&	\
     defined(AT_MINSIGSTKSZ)
-	long sz = (long)getauxval(AT_MINSIGSTKSZ);
+	const long sz = (long)getauxval(AT_MINSIGSTKSZ);
 
 	if (sz > 0)
 		return sz;
@@ -3009,6 +3215,10 @@ void stress_set_stack_smash_check_flag(const bool flag)
 	stress_stack_check_flag = flag;
 }
 
+/*
+ *  stress_tty_width()
+ *	get tty column width
+ */
 int stress_tty_width(void)
 {
 	const int max_width = 80;
@@ -3021,7 +3231,7 @@ int stress_tty_width(void)
 	if (ret < 0)
 		return max_width;
 	ret = (int)ws.ws_col;
-	if ((ret < 0) || (ret > 1024))
+	if ((ret <= 0) || (ret > 1024))
 		return max_width;
 	return ret;
 #else
@@ -3175,14 +3385,163 @@ const char *stress_fs_type(const char *filename)
 	struct statfs buf;
 	static char tmp[64];
 
-	if (statfs(filename, &buf) != 0) {
+	if (statfs(filename, &buf) != 0)
 		return "";
-	}
 	(void)snprintf(tmp, sizeof(tmp), ", filesystem type: %s", stress_fs_magic_to_name((unsigned long)buf.f_type));
 	return tmp;
+#elif (defined(__FreeBSD__) &&		\
+       defined(HAVE_SYS_MOUNT_H) &&	\
+       defined(HAVE_SYS_PARAM_H)) ||	\
+      (defined(__OpenBSD__) &&		\
+       defined(HAVE_SYS_MOUNT_H))
+	struct statfs buf;
+	static char tmp[64];
+
+	if (statfs(filename, &buf) != 0)
+		return "";
+	(void)snprintf(tmp, sizeof(tmp), ", filesystem type: %s", buf.f_fstypename);
+	return tmp;
 #else
 	(void)filename;
 
 	return "";
 #endif
 }
+
+/*
+ *  Indicate a stress test failed because of limited resources
+ *  rather than a failure of the tests during execution.
+ *  err is the errno of the failure.
+ */
+int stress_exit_status(const int err)
+{
+	switch (err) {
+	case ENOMEM:
+	case ENOSPC:
+		return EXIT_NO_RESOURCE;
+	case ENOSYS:
+		return EXIT_NOT_IMPLEMENTED;
+	}
+	return EXIT_FAILURE;	/* cppcheck-suppress ConfigurationNotChecked */
+}
+
+/*
+ *  stress_proc_self_exe_path()
+ *	get proceess' executable path via readlink
+ */
+char *stress_proc_self_exe_path(const char *proc_path)
+{
+	static char path[PATH_MAX];
+	ssize_t len;
+
+	len = shim_readlink(proc_path, path, sizeof(path));
+	if ((len < 0) || (len >= PATH_MAX))
+		return NULL;
+	path[len] = '\0';
+	return path;
+}
+
+/*
+ *  stress_proc_self_exe()
+ *  	determine the path to the executable, return NULL if not possible/failed
+ */
+char *stress_proc_self_exe(void)
+{
+#if defined(__linux__)
+	return stress_proc_self_exe_path("/proc/self/exe");
+#elif defined(__NetBSD__)
+	return stress_proc_self_exe_path("/proc/curproc/exe");
+#elif defined(__DragonFly__)
+	return stress_proc_self_exe_path("/proc/curproc/file");
+#elif defined(__FreeBSD__)
+	return stress_proc_self_exe_path("/proc/curproc/file");
+#elif defined(__sun__) && 	\
+      defined(HAVE_GETEXECNAME)
+	static char path[PATH_MAX];
+	const char *execname = getexecname();
+
+	if (!execname)
+		return NULL;
+	/* Need to perform a string copy to deconstify execname */
+	(void)shim_strlcpy(path, execname, sizeof(path));
+	return  path;
+#else
+	return NULL;
+#endif
+}
+
+#if defined(__FreeBSD__) ||	\
+    defined(__NetBSD__)
+/*
+ *  stress_bsd_getsysctl()
+ *	get sysctl using name, ptr to obj, size = size of obj
+ */
+int stress_bsd_getsysctl(const char *name, void *ptr, size_t size)
+{
+	int ret;
+	size_t nsize = size;
+	if (!ptr)
+		return -1;
+
+	(void)memset(ptr, 0, size);
+
+	ret = sysctlbyname(name, ptr, &nsize, NULL, 0);
+	if ((ret < 0) || (nsize != size)) {
+		(void)memset(ptr, 0, size);
+		return -1;
+	}
+	return 0;
+}
+
+/*
+ *  stress_bsd_getsysctl_uint64()
+ *	get sysctl by name, return uint64 value
+ */
+uint64_t stress_bsd_getsysctl_uint64(const char *name)
+{
+	uint64_t val;
+
+	if (stress_bsd_getsysctl(name, &val, sizeof(val)) == 0)
+		return val;
+	return 0ULL;
+}
+
+/*
+ *  stress_bsd_getsysctl_uint32()
+ *	get sysctl by name, return uint32 value
+ */
+uint32_t stress_bsd_getsysctl_uint32(const char *name)
+{
+	uint32_t val;
+
+	if (stress_bsd_getsysctl(name, &val, sizeof(val)) == 0)
+		return val;
+	return 0UL;
+}
+
+/*
+ *  stress_bsd_getsysctl_uint()
+ *	get sysctl by name, return unsigned int value
+ */
+unsigned int stress_bsd_getsysctl_uint(const char *name)
+{
+	unsigned int val;
+
+	if (stress_bsd_getsysctl(name, &val, sizeof(val)) == 0)
+		return val;
+	return 0;
+}
+
+/*
+ *  stress_bsd_getsysctl_int()
+ *	get sysctl by name, return int value
+ */
+int stress_bsd_getsysctl_int(const char *name)
+{
+	int val;
+
+	if (stress_bsd_getsysctl(name, &val, sizeof(val)) == 0)
+		return val;
+	return 0;
+}
+#endif
