@@ -251,7 +251,8 @@ static int stress_aio(const stress_args_t *args)
 	}
 
 	if ((io_reqs = calloc(opt_aio_requests, sizeof(*io_reqs))) == NULL) {
-		pr_err("%s: cannot allocate io request structures\n", args->name);
+		pr_inf_skip("%s: cannot allocate %" PRIu32 " io request "
+			    "structures, skipping stressor\n", args->name, opt_aio_requests);
 		return EXIT_NO_RESOURCE;
 	}
 
@@ -385,9 +386,10 @@ stressor_info_t stress_aio_info = {
 };
 #else
 stressor_info_t stress_aio_info = {
-	.stressor = stress_not_implemented,
+	.stressor = stress_unimplemented,
 	.class = CLASS_IO | CLASS_INTERRUPT | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
-	.help = help
+	.help = help,
+	.unimplemented_reason = "built without aio.h"
 };
 #endif
