@@ -126,11 +126,6 @@ static void MLOCKED_TEXT stress_apparmor_alrm_handler(int signum)
 	apparmor_run = false;
 }
 
-static void MLOCKED_TEXT stress_apparmor_usr1_handler(int signum)
-{
-	(void)signum;
-}
-
 /*
  *  stress_apparmor_read()
  *	read a proc file
@@ -243,7 +238,6 @@ again:
 			_exit(EXIT_FAILURE);
 		}
 
-		(void)setpgid(0, g_pgrp);
 		(void)sched_settings_apply(true);
 		stress_parent_died_alarm();
 		if (!stress_apparmor_keep_stressing_inc(args, false))
@@ -254,7 +248,6 @@ abort:
 		(void)kill(args->pid, SIGUSR1);
 		_exit(ret);
 	}
-	(void)setpgid(pid, g_pgrp);
 	return pid;
 }
 
@@ -629,38 +622,39 @@ static const stress_apparmor_func apparmor_funcs[] = {
 	apparmor_stress_corruption,
 };
 
+#define MAX_APPARMOR_FUNCS	(SIZEOF_ARRAY(apparmor_funcs))
+
 /*
  *  stress_apparmor()
  *	stress AppArmor
  */
 static int stress_apparmor(const stress_args_t *args)
 {
-	const size_t n = SIZEOF_ARRAY(apparmor_funcs);
-	pid_t pids[n];
+	pid_t pids[MAX_APPARMOR_FUNCS];
 	size_t i;
 	int rc = EXIT_NO_RESOURCE;
 
-	if (stress_sighandler(args->name, SIGUSR1, stress_apparmor_usr1_handler, NULL) < 0)
+	if (stress_sighandler(args->name, SIGUSR1, stress_sighandler_nop, NULL) < 0)
 		return EXIT_FAILURE;
 
 	data_copy = malloc(g_apparmor_data_len);
 	if (!data_copy) {
-		pr_inf("%s: failed to allocate apparmor data copy buffer, skipping stressor\n", args->name);
+		pr_inf_skip("%s: failed to allocate apparmor data copy buffer, skipping stressor\n", args->name);
 		return rc;
 	}
 	data_prev = malloc(g_apparmor_data_len);
 	if (!data_prev) {
-		pr_inf("%s: failed to allocate apparmor data prev buffer, skipping stressor\n", args->name);
+		pr_inf_skip("%s: failed to allocate apparmor data prev buffer, skipping stressor\n", args->name);
 		goto err_free_data_copy;
 	}
 
 	counter_lock = stress_lock_create();
 	if (!counter_lock) {
-		pr_inf("%s: failed to create counter lock. skipping stressor\n", args->name);
+		pr_inf_skip("%s: failed to create counter lock. skipping stressor\n", args->name);
 		goto err_free_data_prev;
 	}
 
-	for (i = 0; i < n; i++) {
+	for (i = 0; i < MAX_APPARMOR_FUNCS; i++) {
 		pids[i] = apparmor_spawn(args, apparmor_funcs[i]);
 	}
 
@@ -677,12 +671,12 @@ static int stress_apparmor(const stress_args_t *args)
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
 	/* Wakeup, time to die */
-	for (i = 0; i < n; i++) {
+	for (i = 0; i < MAX_APPARMOR_FUNCS; i++) {
 		if (pids[i] >= 0)
 			(void)kill(pids[i], SIGALRM);
 	}
 	/* Now apply death grip */
-	for (i = 0; i < n; i++) {
+	for (i = 0; i < MAX_APPARMOR_FUNCS; i++) {
 		int status;
 
 		if (pids[i] >= 0) {
@@ -722,10 +716,11 @@ static int stress_apparmor_supported(const char *name)
 }
 
 stressor_info_t stress_apparmor_info = {
-	.stressor = stress_not_implemented,
+	.stressor = stress_unimplemented,
 	.supported = stress_apparmor_supported,
 	.class = CLASS_OS | CLASS_SECURITY,
 	.verify = VERIFY_ALWAYS,
-	.help = help
+	.help = help,
+	.unimplemented_reason = "built without sys/apparmor.h"
 };
 #endif
