@@ -1,5 +1,6 @@
 /*
- * Copyright (C) 2013-2017 Canonical, Ltd.
+ * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,84 +16,87 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-sort.h"
+
+#if defined(HAVE_SEARCH_H)
+#include <search.h>
+#endif
+
+#define MIN_BSEARCH_SIZE	(1 * KB)
+#define MAX_BSEARCH_SIZE	(4 * MB)
+#define DEFAULT_BSEARCH_SIZE	(64 * KB)
+
+static const stress_help_t help[] = {
+	{ NULL,	"bsearch N",	  "start N workers that exercise a binary search" },
+	{ NULL,	"bsearch-ops N",  "stop after N binary search bogo operations" },
+	{ NULL,	"bsearch-size N", "number of 32 bit integers to bsearch" },
+	{ NULL,	NULL,		  NULL }
+};
 
 /*
  *  stress_set_bsearch_size()
  *	set bsearch size from given option string
  */
-void stress_set_bsearch_size(const char *opt)
+static int stress_set_bsearch_size(const char *opt)
 {
 	uint64_t bsearch_size;
 
-	bsearch_size = get_uint64(opt);
-	check_range("bsearch-size", bsearch_size,
+	bsearch_size = stress_get_uint64(opt);
+	stress_check_range("bsearch-size", bsearch_size,
 		MIN_BSEARCH_SIZE, MAX_BSEARCH_SIZE);
-	set_setting("bsearch-size", TYPE_ID_UINT64, &bsearch_size);
+	return stress_set_setting("bsearch-size", TYPE_ID_UINT64, &bsearch_size);
 }
 
-/*
- *  cmp()
- *	compare int32 values for bsearch
- */
-static int cmp(const void *p1, const void *p2)
-{
-	const int32_t *i1 = (const int32_t *)p1;
-	const int32_t *i2 = (const int32_t *)p2;
+static const stress_opt_set_func_t opt_set_funcs[] = {
+	{ OPT_bsearch_size,	stress_set_bsearch_size },
+	{ 0,			NULL },
+};
 
-	if (*i1 > *i2)
-		return 1;
-	else if (*i1 < *i2)
-		return -1;
-	else
-		return 0;
-}
+#if defined(HAVE_BSEARCH)
 
 /*
  *  Monotonically increasing values
  */
 #define SETDATA(d, i, v, prev)		\
+do {					\
 	d[i] = 1 + prev + (v & 0x7);	\
 	v >>= 2;			\
 	prev = d[i];			\
 	i++;				\
+} while (0)
 
 /*
  *  stress_bsearch()
  *	stress bsearch
  */
-int stress_bsearch(const args_t *args)
+static int stress_bsearch(const stress_args_t *args)
 {
 	int32_t *data, *ptr, prev = 0;
 	size_t n, n8, i;
 	uint64_t bsearch_size = DEFAULT_BSEARCH_SIZE;
 
-	if (!get_setting("bsearch-size", &bsearch_size)) {
+	if (!stress_get_setting("bsearch-size", &bsearch_size)) {
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
 			bsearch_size = MAX_BSEARCH_SIZE;
 		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
 			bsearch_size = MIN_BSEARCH_SIZE;
 	}
 	n = (size_t)bsearch_size;
-	n8 = (n + 7) & ~7;
+	n8 = (n + 7) & ~7UL;
 
 	/* allocate in multiples of 8 */
-	if ((data = calloc(n8, sizeof(int32_t))) == NULL) {
-		pr_fail_dbg("malloc");
+	if ((data = calloc(n8, sizeof(*data))) == NULL) {
+		pr_inf_skip("%s: malloc of %zu bytes failed, out of memory\n",
+			args->name, n8 * sizeof(*data));
 		return EXIT_NO_RESOURCE;
 	}
 
 	/* Populate with ascending data */
 	prev = 0;
 	for (i = 0; i < n;) {
-		uint64_t v = mwc64();
+		int32_t v = (int32_t)stress_mwc32();
 
 		SETDATA(data, i, v, prev);
 		SETDATA(data, i, v, prev);
@@ -104,14 +108,16 @@ int stress_bsearch(const args_t *args)
 		SETDATA(data, i, v, prev);
 	}
 
+	stress_set_proc_state(args->name, STRESS_STATE_RUN);
+
 	do {
 		for (ptr = data, i = 0; i < n; i++, ptr++) {
 			int32_t *result;
-			result = bsearch(ptr, data, n, sizeof(*ptr), cmp);
+
+			result = bsearch(ptr, data, n, sizeof(*ptr), stress_sort_cmp_int32);
 			if (g_opt_flags & OPT_FLAGS_VERIFY) {
 				if (result == NULL)
-					pr_fail("%s: element %zu could "
-						"not be found\n",
+					pr_fail("%s: element %zu could not be found\n",
 						args->name, i);
 				else if (*result != *ptr)
 					pr_fail("%s: element %zu "
@@ -121,8 +127,31 @@ int stress_bsearch(const args_t *args)
 			}
 		}
 		inc_counter(args);
-	} while (keep_stressing());
+	} while (keep_stressing(args));
+
+	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
 	free(data);
 	return EXIT_SUCCESS;
 }
+
+stressor_info_t stress_bsearch_info = {
+	.stressor = stress_bsearch,
+	.class = CLASS_CPU_CACHE | CLASS_CPU | CLASS_MEMORY,
+	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_OPTIONAL,
+	.help = help
+};
+
+#else
+
+stressor_info_t stress_bsearch_info = {
+	.stressor = stress_unimplemented,
+	.class = CLASS_CPU_CACHE | CLASS_CPU | CLASS_MEMORY,
+	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_OPTIONAL,
+	.help = help,
+	.unimplemented_reason = "built without stdlib bsearch() support"
+};
+
+#endif
