@@ -26,8 +26,8 @@
 
 static const stress_help_t help[] = {
 	{ NULL, "chdir N",	"start N workers thrashing chdir on many paths" },
-	{ NULL, "chdir-ops N",	"stop chdir workers after N bogo chdir operations" },
 	{ NULL,	"chdir-dirs N",	"select number of directories to exercise chdir on" },
+	{ NULL, "chdir-ops N",	"stop chdir workers after N bogo chdir operations" },
 	{ NULL,	NULL,		NULL }
 };
 
@@ -37,10 +37,10 @@ static const stress_help_t help[] = {
  */
 static int stress_set_chdir_dirs(const char *opt)
 {
-	uint64_t chdir_dirs;
+	uint32_t chdir_dirs;
 
 	chdir_dirs = stress_get_uint32(opt);
-	stress_check_range("chdir-dirs", chdir_dirs,
+	stress_check_range("chdir-dirs", (uint64_t)chdir_dirs,
 		MIN_CHDIR_DIRS, MAX_CHDIR_DIRS);
 	return stress_set_setting("chdir-dirs", TYPE_ID_UINT32, &chdir_dirs);
 }
@@ -59,22 +59,26 @@ static int stress_chdir(const stress_args_t *args)
 	struct stat statbuf;
 	const bool is_root = stress_check_capability(SHIM_CAP_IS_ROOT);
 	bool got_statbuf = false;
+	double count = 0.0, duration = 0.0, rate;
 
 	(void)stress_get_setting("chdir-dirs", &chdir_dirs);
 	paths = calloc(chdir_dirs, sizeof(*paths));
 	if (!paths) {
-		pr_err("%s: out of memory allocating paths\n", args->name);
+		pr_inf_skip("%s: out of memory allocating %" PRIu32 " paths, "
+			    "skipping stressor\n", args->name, chdir_dirs);
 		return EXIT_NO_RESOURCE;
 	}
 	fds = calloc(chdir_dirs, sizeof(*fds));
 	if (!fds) {
-		pr_err("%s: out of memory allocating file descriptors\n", args->name);
+		pr_inf_skip("%s: out of memory allocating %" PRIu32 " file descriptors, "
+			    "skipping stressor\n", args->name, chdir_dirs);
 		free(paths);
 		return EXIT_NO_RESOURCE;
 	}
 	mkdir_ok = calloc(chdir_dirs, sizeof(*mkdir_ok));
 	if (!mkdir_ok) {
-		pr_err("%s: out of memory allocating file descriptors\n", args->name);
+		pr_inf_skip("%s: out of memory allocating %" PRIu32 " boolean flags, "
+			    "skipping stressor\n", args->name, chdir_dirs);
 		free(fds);
 		free(paths);
 		return EXIT_NO_RESOURCE;
@@ -88,7 +92,7 @@ static int stress_chdir(const stress_args_t *args)
 
 	rc = stress_temp_dir_mk_args(args);
 	if (rc < 0) {
-		ret = exit_status(-rc);
+		ret = stress_exit_status(-rc);
 		goto err;
 	}
 
@@ -106,8 +110,8 @@ static int stress_chdir(const stress_args_t *args)
 
 	/* Populate */
 	for (i = 0; keep_stressing(args) && (i < chdir_dirs); i++) {
-		uint64_t rnd = (uint64_t)stress_mwc32() << 32;
-		uint32_t gray_code = (i >> 1) ^ i;
+		const uint64_t rnd = (uint64_t)stress_mwc32() << 32;
+		const uint32_t gray_code = (i >> 1) ^ i;
 		int flags = O_RDONLY;
 
 #if defined(O_DIRECTORY)
@@ -129,7 +133,7 @@ static int stress_chdir(const stress_args_t *args)
 				fds[i] = -1;
 				continue;
 			}
-			ret = exit_status(errno);
+			ret = stress_exit_status(errno);
 			if (ret == EXIT_FAILURE)
 				pr_fail("%s: mkdir %s failed, errno=%d (%s)\n",
 					args->name, path, errno, strerror(errno));
@@ -156,16 +160,23 @@ static int stress_chdir(const stress_args_t *args)
 
 	do {
 		for (i = 0; keep_stressing(args) && (i < chdir_dirs); i++) {
-			uint32_t j = stress_mwc32() % chdir_dirs;
+			const uint32_t j = stress_mwc32() % chdir_dirs;
 			const int fd = fds[j] >= 0 ? fds[j] : fds[0];
+			double t;
 
-			if (mkdir_ok[i] && (chdir(paths[i]) < 0)) {
-				if (errno != ENOMEM) {
-					pr_fail("%s: chdir %s failed, errno=%d (%s)%s\n",
-						args->name, paths[i],
-						errno, strerror(errno),
-						stress_fs_type(path));
-					goto abort;
+			if (mkdir_ok[i] && paths[i]) {
+				t = stress_time_now();
+				if (chdir(paths[i]) == 0) {
+					duration += stress_time_now() - t;
+					count += 1.0;
+				} else {
+					if (errno != ENOMEM) {
+						pr_fail("%s: chdir %s failed, errno=%d (%s)%s\n",
+							args->name, paths[i],
+							errno, strerror(errno),
+							stress_fs_type(path));
+						goto abort;
+					}
 				}
 			}
 
@@ -182,7 +193,11 @@ static int stress_chdir(const stress_args_t *args)
 			/*
 			 *  chdir to / should always work, surely?
 			 */
-			if (chdir("/") < 0) {
+			t = stress_time_now();
+			if (chdir("/") == 0) {
+				duration += stress_time_now() - t;
+				count += 1.0;
+			} else {
 				if ((errno != ENOMEM) && (errno != EACCES)) {
 					pr_fail("%s: chdir / failed, errno=%d (%s)%s\n",
 						args->name,
@@ -205,8 +220,12 @@ static int stress_chdir(const stress_args_t *args)
 
 			while (keep_stressing(args)) {
 				/* We need chdir to cwd to always succeed */
-				if (chdir(cwd) == 0)
+				t = stress_time_now();
+				if (chdir(cwd) == 0) {
+					duration += stress_time_now() - t;
+					count += 1.0;
 					break;
+				}
 				/* Maybe low memory, force retry */
 				if (errno != ENOMEM) {
 					pr_fail("%s: chdir %s failed, errno=%d (%s)%s\n",
@@ -266,6 +285,9 @@ tidy:
 		}
 	}
 	(void)stress_temp_dir_rm_args(args);
+
+	rate = (duration > 0.0) ? count / duration : 0.0;
+	stress_misc_stats_set(args->misc_stats, 0, "chdir calls per sec", rate);
 err:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 	free(mkdir_ok);
