@@ -30,7 +30,8 @@ static const stress_help_t help[] = {
 	{ NULL,	NULL,		NULL }
 };
 
-#if defined(__linux__)
+#if defined(__linux__) &&	\
+    !defined(STRESS_ARCH_ALPHA)
 
 typedef struct {
 	uint16_t	varname[512];
@@ -388,8 +389,9 @@ static int stress_efivar(const stress_args_t *args)
 	efi_ignore = mmap(NULL, sz, PROT_READ | PROT_WRITE,
 			MAP_ANONYMOUS | MAP_SHARED, -1, 0);
 	if (efi_ignore == MAP_FAILED) {
-		pr_fail("%s: cannot mmap shared memory: %d (%s))\n",
-			args->name, errno, strerror(errno));
+		pr_inf_skip("%s: cannot mmap %zd bytes of shared memory: "
+			"errno=%d (%s), skipping stressor\n",
+			args->name, sz, errno, strerror(errno));
 		return EXIT_NO_RESOURCE;
 	}
 
@@ -458,10 +460,11 @@ static int stress_efivar_supported(const char *name)
 	return -1;
 }
 stressor_info_t stress_efivar_info = {
-	.stressor = stress_not_implemented,
+	.stressor = stress_unimplemented,
 	.supported = stress_efivar_supported,
 	.class = CLASS_OS,
 	.verify = VERIFY_ALWAYS,
-	.help = help
+	.help = help,
+	.unimplemented_reason = "only supported on Linux with EFI variable filesystem"
 };
 #endif
