@@ -40,11 +40,11 @@
 
 static const stress_help_t help[] = {
 	{ NULL,	"epoll N",	  	"start N workers doing epoll handled socket activity" },
+	{ NULL,	"epoll-domain D", 	"specify socket domain, default is unix" },
 	{ NULL,	"epoll-ops N",	  	"stop after N epoll bogo operations" },
 	{ NULL,	"epoll-port P",	  	"use socket ports P upwards" },
-	{ NULL,	"epoll-domain D", 	"specify socket domain, default is unix" },
 	{ NULL, "epoll-sockets N",	"specify maximum number of open sockets" },
-	{ NULL,	NULL,		  NULL }
+	{ NULL,	NULL,			NULL }
 };
 
 #if defined(HAVE_SYS_EPOLL_H) &&	\
@@ -58,7 +58,7 @@ static const stress_help_t help[] = {
 typedef void (stress_epoll_func_t)(
 	const stress_args_t *args,
 	const int child,
-	const pid_t ppid,
+	const pid_t mypid,
 	const int epoll_port,
 	const int epoll_domain,
 	const int epoll_sockets);
@@ -78,9 +78,7 @@ static int stress_set_epoll_port(const char *opt)
 	int epoll_port;
 
 	stress_set_net_port("epoll-port", opt,
-		MIN_EPOLL_PORT,
-		MAX_EPOLL_PORT - (STRESS_PROCS_MAX * MAX_SERVERS),
-		&epoll_port);
+		MIN_EPOLL_PORT, MAX_EPOLL_PORT, &epoll_port);
 	return stress_set_setting("epoll-port", TYPE_ID_INT, &epoll_port);
 }
 
@@ -207,7 +205,7 @@ static pid_t epoll_spawn(
 	const stress_args_t *args,
 	stress_epoll_func_t func,
 	const int child,
-	const pid_t ppid,
+	const pid_t mypid,
 	const int epoll_port,
 	const int epoll_domain,
 	const int epoll_sockets)
@@ -222,13 +220,11 @@ again:
 		return -1;
 	}
 	if (pid == 0) {
-		(void)setpgid(0, g_pgrp);
 		stress_parent_died_alarm();
 		(void)sched_settings_apply(true);
-		func(args, child, ppid, epoll_port, epoll_domain, epoll_sockets);
+		func(args, child, mypid, epoll_port, epoll_domain, epoll_sockets);
 		_exit(EXIT_SUCCESS);
 	}
-	(void)setpgid(pid, g_pgrp);
 	return pid;
 }
 
@@ -549,7 +545,7 @@ err:
  */
 static int epoll_client(
 	const stress_args_t *args,
-	const pid_t ppid,
+	const pid_t mypid,
 	const int epoll_port,
 	const int epoll_domain)
 {
@@ -567,7 +563,7 @@ static int epoll_client(
 		int fd, saved_errno;
 		int retries = 0;
 		int ret;
-		int port = epoll_port + port_counter +
+		const int port = epoll_port + port_counter +
 				(max_servers * (int)args->instance);
 		socklen_t addr_len = 0;
 
@@ -620,8 +616,11 @@ retry:
 			return EXIT_FAILURE;
 		}
 
-		stress_set_sockaddr(args->name, args->instance, ppid,
-			epoll_domain, port, &addr, &addr_len, NET_ADDR_ANY);
+		if (stress_set_sockaddr(args->name, args->instance, mypid,
+			epoll_domain, port, &addr, &addr_len, NET_ADDR_ANY) < 0) {
+			(void)close(fd);
+			return EXIT_FAILURE;
+		}
 
 		errno = 0;
 		ret = connect(fd, addr, addr_len);
@@ -698,7 +697,7 @@ retry:
 static void NORETURN epoll_server(
 	const stress_args_t *args,
 	const int child,
-	const pid_t ppid,
+	const pid_t mypid,
 	const int epoll_port,
 	const int epoll_domain,
 	const int epoll_sockets)
@@ -730,8 +729,11 @@ static void NORETURN epoll_server(
 		goto die_close;
 	}
 
-	stress_set_sockaddr(args->name, args->instance, ppid,
-		epoll_domain, port, &addr, &addr_len, NET_ADDR_ANY);
+	if (stress_set_sockaddr(args->name, args->instance, mypid,
+		epoll_domain, port, &addr, &addr_len, NET_ADDR_ANY) < 0) {
+		rc = EXIT_FAILURE;
+		goto die_close;
+	}
 
 	if (bind(sfd, addr, addr_len) < 0) {
 		pr_fail("%s: bind failed, errno=%d (%s)\n",
@@ -893,7 +895,7 @@ static void NORETURN epoll_server(
 			saved_errno = errno;
 
 			/* Invalid epoll_wait syscall having invalid maxevents argument */
-			(void)epoll_wait(efd, events, INT_MIN, 100);
+			(void)epoll_wait(efd, events, 0, 100);
 
 		} else {
 			if (!wait_segv) {
@@ -996,11 +998,12 @@ die:
  */
 static int stress_epoll(const stress_args_t *args)
 {
-	pid_t pids[MAX_SERVERS], ppid = getppid();
+	pid_t pids[MAX_SERVERS], mypid = getpid();
 	int i, rc = EXIT_SUCCESS;
 	int epoll_domain = AF_UNIX;
 	int epoll_port = DEFAULT_EPOLL_PORT;
 	int epoll_sockets = DEFAULT_EPOLL_SOCKETS;
+	int start_port, end_port, reserved_port;
 
 	(void)stress_get_setting("epoll-domain", &epoll_domain);
 	(void)stress_get_setting("epoll-port", &epoll_port);
@@ -1010,14 +1013,38 @@ static int stress_epoll(const stress_args_t *args)
 		return EXIT_NO_RESOURCE;
 
 	if (max_servers == 1) {
+		start_port = epoll_port + (int)args->instance;
+		reserved_port = stress_net_reserve_ports(start_port, start_port);
+		if (reserved_port < 0) {
+			pr_inf_skip("%s: cannot reserve port %d, skipping stressor\n",
+				args->name, start_port);
+			return EXIT_NO_RESOURCE;
+		}
+		/* adjust for reserved port range */
+		start_port = reserved_port;
+		end_port = reserved_port;
+		epoll_port = start_port - (int)args->instance;
+
 		pr_dbg("%s: process [%" PRIdMAX "] using socket port %d\n",
 			args->name, (intmax_t)args->pid,
 			epoll_port + (int)args->instance);
 	} else {
+		start_port = epoll_port + (max_servers * (int)args->instance);
+		end_port = start_port + max_servers - 1;
+
+		reserved_port = stress_net_reserve_ports(start_port, end_port);
+		if (reserved_port < 0) {
+			pr_inf_skip("%s: cannot reserve ports %d..%d, skipping stressor\n",
+				args->name, start_port, end_port);
+			return EXIT_NO_RESOURCE;
+		}
+		/* adjust for reserved port range */
+		start_port = reserved_port;
+		end_port = reserved_port + max_servers - 1;
+		epoll_port = start_port - (max_servers * (int)args->instance);
+
 		pr_dbg("%s: process [%" PRIdMAX "] using socket ports %d..%d\n",
-			args->name, (intmax_t)args->pid,
-			epoll_port + (max_servers * (int)args->instance),
-			epoll_port + (max_servers * (int)(args->instance + 1)) - 1);
+			args->name, (intmax_t)args->pid, start_port, end_port);
 	}
 
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
@@ -1038,7 +1065,7 @@ static int stress_epoll(const stress_args_t *args)
 	 */
 	(void)memset(pids, 0, sizeof(pids));
 	for (i = 0; i < max_servers; i++) {
-		pids[i] = epoll_spawn(args, epoll_server, i, ppid, epoll_port, epoll_domain, epoll_sockets);
+		pids[i] = epoll_spawn(args, epoll_server, i, mypid, epoll_port, epoll_domain, epoll_sockets);
 		if (pids[i] < 0) {
 			pr_fail("%s: fork failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
@@ -1046,9 +1073,10 @@ static int stress_epoll(const stress_args_t *args)
 		}
 	}
 
-	epoll_client(args, ppid, epoll_port, epoll_domain);
+	epoll_client(args, mypid, epoll_port, epoll_domain);
 reap:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
+	stress_net_release_ports(start_port, end_port);
 
 	for (i = 0; i < max_servers; i++) {
 		int status;
@@ -1073,9 +1101,10 @@ stressor_info_t stress_epoll_info = {
 };
 #else
 stressor_info_t stress_epoll_info = {
-	.stressor = stress_not_implemented,
+	.stressor = stress_unimplemented,
 	.class = CLASS_NETWORK | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
-	.help = help
+	.help = help,
+	.unimplemented_reason = "built without sys/epoll.h or librt or timer support"
 };
 #endif
