@@ -36,8 +36,8 @@
 
 static const stress_help_t help[] = {
 	{ NULL,	"fiemap N",	  "start N workers exercising the FIEMAP ioctl" },
-	{ NULL,	"fiemap-ops N",	  "stop after N FIEMAP ioctl bogo operations" },
 	{ NULL,	"fiemap-bytes N", "specify size of file to fiemap" },
+	{ NULL,	"fiemap-ops N",	  "stop after N FIEMAP ioctl bogo operations" },
 	{ NULL,	NULL,		   NULL }
 };
 
@@ -138,7 +138,9 @@ static void stress_fiemap_ioctl(
 	const stress_args_t *args,
 	const int fd)
 {
+#if !defined(O_SYNC)
 	int c = stress_mwc32() % COUNT_MAX;
+#endif
 
 	do {
 		struct fiemap *fiemap, *tmp;
@@ -155,7 +157,7 @@ static void stress_fiemap_ioctl(
 		if (ioctl(fd, FS_IOC_FIEMAP, fiemap) < 0) {
 			if (errno == EOPNOTSUPP) {
 				if (args->instance == 0)
-					pr_inf_skip("%s: FS_IOC_FIEMAP not supported on the file system, skipping stressor\n",
+					pr_inf_skip("%s: ioctl FS_IOC_FIEMAP not supported on the file system, skipping stressor\n",
 						args->name);
 				free(fiemap);
 				break;
@@ -196,10 +198,14 @@ static void stress_fiemap_ioctl(
 			break;
 		}
 		free(fiemap);
+		if (!keep_stressing(args))
+			break;
+#if !defined(O_SYNC)
 		if (c++ > COUNT_MAX) {
 			c = 0;
 			fdatasync(fd);
 		}
+#endif
 	} while (inc_counter_lock(args, counter_lock, true));
 }
 
@@ -215,14 +221,12 @@ static inline pid_t stress_fiemap_spawn(const stress_args_t *args, const int fd)
 	if (pid < 0)
 		return -1;
 	if (pid == 0) {
-		(void)setpgid(0, g_pgrp);
 		stress_parent_died_alarm();
 		(void)sched_settings_apply(true);
 		stress_mwc_reseed();
 		stress_fiemap_ioctl(args, fd);
 		_exit(EXIT_SUCCESS);
 	}
-	(void)setpgid(pid, g_pgrp);
 	return pid;
 }
 
@@ -239,6 +243,11 @@ static int stress_fiemap(const stress_args_t *args)
 	uint64_t fiemap_bytes = DEFAULT_FIEMAP_SIZE;
 	struct fiemap fiemap;
 	const char *fs_type;
+#if defined(O_SYNC)
+	const int flags = O_CREAT | O_RDWR | O_SYNC;
+#else
+	const int flags = O_CREAT | O_RDWR;
+#endif
 
 	counter_lock = stress_lock_create();
 	if (!counter_lock) {
@@ -258,14 +267,14 @@ static int stress_fiemap(const stress_args_t *args)
 
 	ret = stress_temp_dir_mk_args(args);
 	if (ret < 0) {
-		rc = exit_status(-ret);
+		rc = stress_exit_status(-ret);
 		goto clean;
 	}
 
 	(void)stress_temp_filename_args(args,
 		filename, sizeof(filename), stress_mwc32());
-	if ((fd = open(filename, O_CREAT | O_RDWR, S_IRUSR | S_IWUSR)) < 0) {
-		rc = exit_status(errno);
+	if ((fd = open(filename, flags, S_IRUSR | S_IWUSR)) < 0) {
+		rc = stress_exit_status(errno);
 		pr_fail("%s: open %s failed, errno=%d (%s)\n",
 			args->name, filename, errno, strerror(errno));
 		goto dir_clean;
@@ -331,9 +340,10 @@ stressor_info_t stress_fiemap_info = {
 };
 #else
 stressor_info_t stress_fiemap_info = {
-	.stressor = stress_not_implemented,
+	.stressor = stress_unimplemented,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
-	.help = help
+	.help = help,
+	.unimplemented_reason = "built without linux/fiemap.h, linux/fs.h or ioctl() FS_IOC_FIEMAP support"
 };
 #endif
