@@ -239,7 +239,7 @@ static int stress_get(const stress_args_t *args)
 		{
 			char name[128];
 
-			VOID_RET(int, getdomainname(name, sizeof(name)));
+			VOID_RET(int, shim_getdomainname(name, sizeof(name)));
 			if (!keep_stressing_flag())
 				break;
 		}
@@ -320,7 +320,7 @@ static int stress_get(const stress_args_t *args)
 		 *  valgrind happy.
 		 */
 		(void)memset(gids, 0, sizeof(gids));
-		VOID_RET(int, syscall(__NR_getgroups, -1, gids));
+		VOID_RET(long, syscall(__NR_getgroups, -1, gids));
 #endif
 		if (!keep_stressing_flag())
 			break;
@@ -633,7 +633,7 @@ static int stress_get(const stress_args_t *args)
 					uname_segv = true;
 				} else {
 					ret = uname(args->mapped->page_none);
-					if (ret == 0) {
+					if (ret >= 0) {
 						pr_fail("%s: uname unexpectedly succeeded with read only utsbuf, "
 							"expected -EFAULT, instead got errno=%d (%s)\n",
 							args->name, errno, strerror(errno));
