@@ -1,5 +1,6 @@
 /*
- * Copyright (C) 2016-2017 Canonical, Ltd.
+ * Copyright (C) 2016-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,45 +16,54 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-sort.h"
+
+#define MIN_HEAPSORT_SIZE	(1 * KB)
+#define MAX_HEAPSORT_SIZE	(4 * MB)
+#define DEFAULT_HEAPSORT_SIZE	(256 * KB)
 
 #if defined(HAVE_LIB_BSD)
-#include <bsd/stdlib.h>
-
 static volatile bool do_jmp = true;
 static sigjmp_buf jmp_env;
 #endif
 
+static const stress_help_t help[] = {
+	{ NULL,	"heapsort N",	   "start N workers heap sorting 32 bit random integers" },
+	{ NULL,	"heapsort-ops N",  "stop after N heap sort bogo operations" },
+	{ NULL,	"heapsort-size N", "number of 32 bit integers to sort" },
+	{ NULL,	NULL,		   NULL }
+};
+
 /*
  *  stress_set_heapsort_size()
  *	set heapsort size
  */
-void stress_set_heapsort_size(const void *opt)
+static int stress_set_heapsort_size(const char *opt)
 {
 	uint64_t heapsort_size;
 
-	heapsort_size = get_uint64(opt);
-	check_range("heapsort-size", heapsort_size,
+	heapsort_size = stress_get_uint64(opt);
+	stress_check_range("heapsort-size", heapsort_size,
 		MIN_HEAPSORT_SIZE, MAX_HEAPSORT_SIZE);
-	set_setting("heapsort-size", TYPE_ID_UINT64, &heapsort_size);
+	return stress_set_setting("heapsort-size", TYPE_ID_UINT64, &heapsort_size);
 }
 
+static const stress_opt_set_func_t opt_set_funcs[] = {
+	{ OPT_heapsort_integers,	stress_set_heapsort_size },
+	{ 0,				NULL }
+};
+
 #if defined(HAVE_LIB_BSD)
 
 /*
  *  stress_heapsort_handler()
  *	SIGALRM generic handler
  */
-static void MLOCKED stress_heapsort_handler(int dummy)
+static void MLOCKED_TEXT stress_heapsort_handler(int signum)
 {
-	(void)dummy;
+	(void)signum;
 
 	if (do_jmp) {
 		do_jmp = false;
@@ -61,50 +71,11 @@ static void MLOCKED stress_heapsort_handler(int dummy)
 	}
 }
 
-/*
- *  stress_heapsort_cmp_1()
- *	heapsort comparison - sort on int32 values
- */
-static int stress_heapsort_cmp_1(const void *p1, const void *p2)
-{
-	const int32_t *i1 = (const int32_t *)p1;
-	const int32_t *i2 = (const int32_t *)p2;
-
-	if (*i1 > *i2)
-		return 1;
-	else if (*i1 < *i2)
-		return -1;
-	else
-		return 0;
-}
-
-/*
- *  stress_heapsort_cmp_1()
- *	heapsort comparison - reverse sort on int32 values
- */
-static int stress_heapsort_cmp_2(const void *p1, const void *p2)
-{
-	return stress_heapsort_cmp_1(p2, p1);
-}
-
-/*
- *  stress_heapsort_cmp_1()
- *	heapsort comparison - sort on int8 values
- */
-static int stress_heapsort_cmp_3(const void *p1, const void *p2)
-{
-	const int8_t *i1 = (const int8_t *)p1;
-	const int8_t *i2 = (const int8_t *)p2;
-
-	/* Force re-ordering on 8 bit value */
-	return *i1 - *i2;
-}
-
 /*
  *  stress_heapsort()
  *	stress heapsort
  */
-int stress_heapsort(const args_t *args)
+static int stress_heapsort(const stress_args_t *args)
 {
 	uint64_t heapsort_size = DEFAULT_HEAPSORT_SIZE;
 	int32_t *data, *ptr;
@@ -112,7 +83,7 @@ int stress_heapsort(const args_t *args)
 	struct sigaction old_action;
 	int ret;
 
-	if (!get_setting("heapsort-size", &heapsort_size)) {
+	if (!stress_get_setting("heapsort-size", &heapsort_size)) {
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
 			heapsort_size = MAX_HEAPSORT_SIZE;
 		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
@@ -120,8 +91,9 @@ int stress_heapsort(const args_t *args)
 	}
 	n = (size_t)heapsort_size;
 
-	if ((data = calloc(n, sizeof(int32_t))) == NULL) {
-		pr_fail_dbg("malloc");
+	if ((data = calloc(n, sizeof(*data))) == NULL) {
+		pr_inf_skip("%s: failed to allocate %zu integers, skipping stressor\n",
+			args->name, n);
 		return EXIT_NO_RESOURCE;
 	}
 
@@ -141,17 +113,19 @@ int stress_heapsort(const args_t *args)
 
 	/* This is expensive, do it once */
 	for (ptr = data, i = 0; i < n; i++)
-		*ptr++ = mwc32();
+		*ptr++ = (int32_t)stress_mwc32();
+
+	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
 		/* Sort "random" data */
-		if (heapsort(data, n, sizeof(uint32_t), stress_heapsort_cmp_1) < 0) {
+		if (heapsort(data, n, sizeof(*data), stress_sort_cmp_int32) < 0) {
 			pr_fail("%s: heapsort of random data failed: %d (%s)\n",
 				args->name, errno, strerror(errno));
 		} else {
 			if (g_opt_flags & OPT_FLAGS_VERIFY) {
 				for (ptr = data, i = 0; i < n - 1; i++, ptr++) {
-					if (*ptr > *(ptr+1)) {
+					if (*ptr > *(ptr + 1)) {
 						pr_fail("%s: sort error "
 							"detected, incorrect ordering "
 							"found\n", args->name);
@@ -160,17 +134,17 @@ int stress_heapsort(const args_t *args)
 				}
 			}
 		}
-		if (!g_keep_stressing_flag)
+		if (!keep_stressing_flag())
 			break;
 
 		/* Reverse sort */
-		if (heapsort(data, n, sizeof(uint32_t), stress_heapsort_cmp_2) < 0) {
+		if (heapsort(data, n, sizeof(*data), stress_sort_cmp_rev_int32) < 0) {
 			pr_fail("%s: reversed heapsort of random data failed: %d (%s)\n",
 				args->name, errno, strerror(errno));
 		} else {
 			if (g_opt_flags & OPT_FLAGS_VERIFY) {
 				for (ptr = data, i = 0; i < n - 1; i++, ptr++) {
-					if (*ptr < *(ptr+1)) {
+					if (*ptr < *(ptr + 1)) {
 						pr_fail("%s: reverse sort "
 							"error detected, incorrect "
 							"ordering found\n", args->name);
@@ -179,22 +153,22 @@ int stress_heapsort(const args_t *args)
 				}
 			}
 		}
-		if (!g_keep_stressing_flag)
+		if (!keep_stressing_flag())
 			break;
 		/* And re-order by byte compare */
-		if (heapsort(data, n * 4, sizeof(uint8_t), stress_heapsort_cmp_3) < 0) {
+		if (heapsort(data, n * 4, sizeof(uint8_t), stress_sort_cmp_int8) < 0) {
 			pr_fail("%s: heapsort failed: %d (%s)\n",
 				args->name, errno, strerror(errno));
 		}
 
 		/* Reverse sort this again */
-		if (heapsort(data, n, sizeof(uint32_t), stress_heapsort_cmp_2) < 0) {
+		if (heapsort(data, n, sizeof(*data), stress_sort_cmp_rev_int32) < 0) {
 			pr_fail("%s: reversed heapsort of random data failed: %d (%s)\n",
 				args->name, errno, strerror(errno));
 		} else {
 			if (g_opt_flags & OPT_FLAGS_VERIFY) {
 				for (ptr = data, i = 0; i < n - 1; i++, ptr++) {
-					if (*ptr < *(ptr+1)) {
+					if (*ptr < *(ptr + 1)) {
 						pr_fail("%s: reverse sort "
 							"error detected, incorrect "
 							"ordering found\n", args->name);
@@ -203,22 +177,35 @@ int stress_heapsort(const args_t *args)
 				}
 			}
 		}
-		if (!g_keep_stressing_flag)
+		if (!keep_stressing_flag())
 			break;
 
 		inc_counter(args);
-	} while (keep_stressing());
+	} while (keep_stressing(args));
 
 	do_jmp = false;
 	(void)stress_sigrestore(args->name, SIGALRM, &old_action);
 tidy:
+	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
+
 	free(data);
 
 	return EXIT_SUCCESS;
 }
+
+stressor_info_t stress_heapsort_info = {
+	.stressor = stress_heapsort,
+	.class = CLASS_CPU_CACHE | CLASS_CPU | CLASS_MEMORY,
+	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_OPTIONAL,
+	.help = help
+};
 #else
-int stress_heapsort(const args_t *args)
-{
-	return stress_not_implemented(args);
-}
+stressor_info_t stress_heapsort_info = {
+	.stressor = stress_unimplemented,
+	.class = CLASS_CPU_CACHE | CLASS_CPU | CLASS_MEMORY,
+	.opt_set_funcs = opt_set_funcs,
+	.help = help,
+	.unimplemented_reason = "built without BSD library"
+};
 #endif
