@@ -32,8 +32,8 @@
 
 static const stress_help_t help[] = {
 	{ NULL,	"memrate N",		"start N workers exercised memory read/writes" },
-	{ NULL,	"memrate-ops N",	"stop after N memrate bogo operations" },
 	{ NULL,	"memrate-bytes N",	"size of memory buffer being exercised" },
+	{ NULL,	"memrate-ops N",	"stop after N memrate bogo operations" },
 	{ NULL,	"memrate-rd-mbs N",	"read rate from buffer in megabytes per second" },
 	{ NULL,	"memrate-wr-mbs N",	"write rate to buffer in megabytes per second" },
 	{ NULL,	NULL,			NULL }
@@ -606,10 +606,8 @@ static inline void *stress_memrate_mmap(const stress_args_t *args, uint64_t sz)
 	} else {
 #if defined(HAVE_MADVISE) &&	\
     defined(MADV_HUGEPAGE)
-		int ret, advice = MADV_NORMAL;
 
-		ret = madvise(ptr, sz, advice);
-		(void)ret;
+		VOID_RET(int, madvise(ptr, sz, MADV_HUGEPAGE));
 #endif
 	}
 	return ptr;
@@ -677,7 +675,6 @@ static int stress_memrate(const stress_args_t *args)
 {
 	int rc;
 	size_t i, stats_size;
-	bool lock = false;
 	stress_memrate_context_t context;
 
 	context.memrate_bytes = DEFAULT_MEMRATE_BYTES;
@@ -693,8 +690,11 @@ static int stress_memrate(const stress_args_t *args)
 
 	context.stats = (stress_memrate_stats_t *)mmap(NULL, stats_size,
 		PROT_READ | PROT_WRITE, MAP_SHARED | MAP_ANONYMOUS, -1, 0);
-	if (context.stats == MAP_FAILED)
+	if (context.stats == MAP_FAILED) {
+		pr_inf_skip("%s: failed to mmap %zd byte statistics buffer, skipping stressor\n",
+			args->name, stats_size);
 		return EXIT_NO_RESOURCE;
+	}
 	for (i = 0; i < memrate_items; i++) {
 		context.stats[i].duration = 0.0;
 		context.stats[i].kbytes = 0.0;
@@ -709,7 +709,7 @@ static int stress_memrate(const stress_args_t *args)
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
-	pr_lock(&lock);
+	pr_lock();
 	for (i = 0; i < memrate_items; i++) {
 		if (!context.stats[i].valid)
 			continue;
@@ -717,17 +717,17 @@ static int stress_memrate(const stress_args_t *args)
 			char tmp[32];
 			const double rate = context.stats[i].kbytes / (context.stats[i].duration * KB);
 
-			pr_inf_lock(&lock, "%s: %10.10s: %12.2f MB/sec\n",
+			pr_inf("%s: %10.10s: %12.2f MB per sec\n",
 				args->name, memrate_info[i].name, rate);
 
-			(void)snprintf(tmp, sizeof(tmp), "%s MB/sec", memrate_info[i].name);
-			stress_misc_stats_set(args->misc_stats, (int)i, tmp, rate);
+			(void)snprintf(tmp, sizeof(tmp), "%s MB per sec", memrate_info[i].name);
+			stress_misc_stats_set(args->misc_stats, i, tmp, rate);
 		} else {
-			pr_inf_lock(&lock, "%s: %10.10s: interrupted early\n",
+			pr_inf("%s: %10.10s: interrupted early\n",
 				args->name, memrate_info[i].name);
 		}
 	}
-	pr_unlock(&lock);
+	pr_unlock();
 
 	(void)munmap((void *)context.stats, stats_size);
 
