@@ -24,10 +24,16 @@
 #include "core-pthread.h"
 #include "core-target-clones.h"
 
+#if defined(HAVE_LINUX_MEMPOLICY_H) &&	\
+    defined(__NR_mbind)
+#include <linux/mempolicy.h>
+#define HAVE_MEMTHRASH_NUMA	(1)
+#endif
+
 static const stress_help_t help[] = {
 	{ NULL,	"memthrash N",		"start N workers thrashing a 16MB memory buffer" },
-	{ NULL,	"memthrash-ops N",	"stop after N memthrash bogo operations" },
 	{ NULL,	"memthrash-method M",	"specify memthrash method M, default is all" },
+	{ NULL,	"memthrash-ops N",	"stop after N memthrash bogo operations" },
 	{ NULL,	NULL,			NULL }
 };
 
@@ -41,19 +47,27 @@ static const stress_help_t help[] = {
 #define STRESS_CACHE_LINE_SHIFT	(6)	/* Typical 64 byte size */
 #define STRESS_CACHE_LINE_SIZE	(1 << STRESS_CACHE_LINE_SHIFT)
 
-typedef void (*stress_memthrash_func_t)(const stress_args_t *args, size_t mem_size);
 
 typedef struct {
+	const stress_args_t *args;
+	const struct stress_memthrash_method_info *memthrash_method;
+	uint32_t total_cpus;
+	uint32_t max_threads;
+#if defined(HAVE_MEMTHRASH_NUMA)
+	unsigned long numa_nodes;
+	unsigned long max_numa_nodes;
+	unsigned long *numa_node_mask;
+	size_t numa_node_mask_size;
+#endif
+} stress_memthrash_context_t;
+
+typedef void (*stress_memthrash_func_t)(const stress_memthrash_context_t *context, size_t mem_size);
+
+typedef struct stress_memthrash_method_info {
 	const char		*name;	/* human readable form of stressor */
 	stress_memthrash_func_t	func;	/* the method function */
 } stress_memthrash_method_info_t;
 
-typedef struct {
-	uint32_t total_cpus;
-	uint32_t max_threads;
-	const stress_memthrash_method_info_t *memthrash_method;
-} stress_memthrash_context_t;
-
 /* Per-pthread information */
 typedef struct {
 	pthread_t pthread;	/* pthread handle */
@@ -65,6 +79,7 @@ typedef struct {
 	size_t  prime_stride;	/* prime cache sized stride */
 } stress_memthrash_primes_t;
 
+
 static const stress_memthrash_method_info_t memthrash_methods[];
 static void *mem;
 static volatile bool thread_terminate;
@@ -118,53 +133,53 @@ static inline HOT OPTIMIZE3 void stress_memthrash_random_chunk(
 }
 
 static void HOT OPTIMIZE3 stress_memthrash_random_chunkpage(
-	const stress_args_t *args,
+	const stress_memthrash_context_t *context,
 	const size_t mem_size)
 {
-	stress_memthrash_random_chunk(args->page_size, mem_size);
+	stress_memthrash_random_chunk(context->args->page_size, mem_size);
 }
 
 static void HOT OPTIMIZE3 stress_memthrash_random_chunk256(
-	const stress_args_t *args,
+	const stress_memthrash_context_t *context,
 	const size_t mem_size)
 {
-	(void)args;
+	(void)context;
 
 	stress_memthrash_random_chunk(256, mem_size);
 }
 
 static void HOT OPTIMIZE3 stress_memthrash_random_chunk64(
-	const stress_args_t *args,
+	const stress_memthrash_context_t *context,
 	const size_t mem_size)
 {
-	(void)args;
+	(void)context;
 
 	stress_memthrash_random_chunk(64, mem_size);
 }
 
 static void HOT OPTIMIZE3 stress_memthrash_random_chunk8(
-	const stress_args_t *args,
+	const stress_memthrash_context_t *context,
 	const size_t mem_size)
 {
-	(void)args;
+	(void)context;
 
 	stress_memthrash_random_chunk(8, mem_size);
 }
 
 static void HOT OPTIMIZE3 stress_memthrash_random_chunk1(
-	const stress_args_t *args,
+	const stress_memthrash_context_t *context,
 	const size_t mem_size)
 {
-	(void)args;
+	(void)context;
 
 	stress_memthrash_random_chunk(1, mem_size);
 }
 
 static void stress_memthrash_memset(
-	const stress_args_t *args,
+	const stress_memthrash_context_t *context,
 	const size_t mem_size)
 {
-	(void)args;
+	(void)context;
 
 #if defined(__GNUC__)
 	(void)__builtin_memset((void *)mem, stress_mwc8(), mem_size);
@@ -174,12 +189,12 @@ static void stress_memthrash_memset(
 }
 
 static void stress_memthrash_memmove(
-	const stress_args_t *args,
+	const stress_memthrash_context_t *context,
 	const size_t mem_size)
 {
 	char *dst = ((char *)mem) + 1;
 
-	(void)args;
+	(void)context;
 #if defined(__GNUC__)
 	(void)shim_builtin_memmove((void *)dst, mem, mem_size - 1);
 #else
@@ -188,14 +203,14 @@ static void stress_memthrash_memmove(
 }
 
 static void HOT OPTIMIZE3 stress_memthrash_memset64(
-	const stress_args_t *args,
+	const stress_memthrash_context_t *context,
 	const size_t mem_size)
 {
 	register uint64_t *ptr = (uint64_t *)mem;
 	register const uint64_t *end = (uint64_t *)(((uintptr_t)mem) + mem_size);
 	register uint64_t val = stress_mwc64();
 
-	(void)args;
+	(void)context;
 
 #if defined(HAVE_NT_STORE64)
 	if (stress_cpu_x86_has_sse2()) {
@@ -228,13 +243,13 @@ static void HOT OPTIMIZE3 stress_memthrash_memset64(
 }
 
 static void OPTIMIZE3 TARGET_CLONES stress_memthrash_swap64(
-	const stress_args_t *args,
+	const stress_memthrash_context_t *context,
 	const size_t mem_size)
 {
 	uint64_t *ptr = (uint64_t *)mem;
 	register const uint64_t *end = (uint64_t *)(((uintptr_t)mem) + mem_size);
 
-	(void)args;
+	(void)context;
 
 	while (LIKELY(ptr < end)) {
 		register uint64_t r0, r1, r2, r3, r4, r5, r6, r7;
@@ -285,13 +300,13 @@ static void OPTIMIZE3 TARGET_CLONES stress_memthrash_swap64(
 
 #if defined(HAVE_INT128_T)
 static void OPTIMIZE3 TARGET_CLONES stress_memthrash_copy128(
-	const stress_args_t *args,
+	const stress_memthrash_context_t *context,
 	const size_t mem_size)
 {
 	__uint128_t *ptr = (__uint128_t *)mem;
 	register const __uint128_t *end = (__uint128_t *)(((uintptr_t)mem) + mem_size - 128);
 
-	(void)args;
+	(void)context;
 
 	while (LIKELY(ptr < end)) {
 		register __uint128_t r0, r1, r2, r3, r4, r5, r6, r7;
@@ -338,13 +353,13 @@ static void OPTIMIZE3 TARGET_CLONES stress_memthrash_copy128(
 #endif
 
 static void HOT OPTIMIZE3 stress_memthrash_flip_mem(
-	const stress_args_t *args,
+	const stress_memthrash_context_t *context,
 	const size_t mem_size)
 {
 	volatile uint64_t *ptr = (volatile uint64_t *)mem;
 	const uint64_t *end = (uint64_t *)(((uintptr_t)mem) + mem_size);
 
-	(void)args;
+	(void)context;
 
 	while (LIKELY(ptr < end)) {
 		*ptr = *ptr ^ ~0ULL;
@@ -353,7 +368,7 @@ static void HOT OPTIMIZE3 stress_memthrash_flip_mem(
 }
 
 static void HOT OPTIMIZE3 stress_memthrash_swap(
-	const stress_args_t *args,
+	const stress_memthrash_context_t *context,
 	const size_t mem_size)
 {
 	size_t i;
@@ -361,7 +376,7 @@ static void HOT OPTIMIZE3 stress_memthrash_swap(
 	register size_t offset2 = stress_mwc32() % mem_size;
 	uint8_t *mem_u8 = (uint8_t *)mem;
 
-	(void)args;
+	(void)context;
 
 	for (i = 0; !thread_terminate && (i < 65536); i++) {
 		register uint8_t tmp;
@@ -380,17 +395,17 @@ static void HOT OPTIMIZE3 stress_memthrash_swap(
 }
 
 static void HOT OPTIMIZE3 stress_memthrash_matrix(
-	const stress_args_t *args,
+	const stress_memthrash_context_t *context,
 	const size_t mem_size)
 {
 	size_t i, j;
 	volatile uint8_t *vmem = mem;
 
-	(void)args;
+	(void)context;
 	(void)mem_size;
 
-	for (i = 0; !thread_terminate && (i < MATRIX_SIZE); i+= ((stress_mwc8() & 0xf) + 1)) {
-		for (j = 0; j < MATRIX_SIZE; j+= 16) {
+	for (i = 0; !thread_terminate && (i < MATRIX_SIZE); i += ((stress_mwc8() & 0xf) + 1)) {
+		for (j = 0; j < MATRIX_SIZE; j += 16) {
 			size_t i1 = (i * MATRIX_SIZE) + j;
 			size_t i2 = (j * MATRIX_SIZE) + i;
 			uint8_t tmp;
@@ -403,34 +418,34 @@ static void HOT OPTIMIZE3 stress_memthrash_matrix(
 }
 
 static void HOT OPTIMIZE3 stress_memthrash_prefetch(
-	const stress_args_t *args,
+	const stress_memthrash_context_t *context,
 	const size_t mem_size)
 {
 	uint32_t i;
 	const uint32_t max = stress_mwc16();
 
-	(void)args;
+	(void)context;
 
 	for (i = 0; !thread_terminate && (i < max); i++) {
 		size_t offset = stress_mwc32() % mem_size;
 		uint8_t *const ptr = ((uint8_t *)mem) + offset;
 		volatile uint8_t *const vptr = ptr;
 
+		/* Force prefetch and then modify to thrash cache */
 		shim_builtin_prefetch(ptr, 1, 1);
-		//(void)*vptr;
 		*vptr = i & 0xff;
 	}
 }
 
 #if defined(HAVE_ASM_X86_CLFLUSH)
 static void HOT OPTIMIZE3 stress_memthrash_flush(
-	const stress_args_t *args,
+	const stress_memthrash_context_t *context,
 	const size_t mem_size)
 {
 	uint32_t i;
 	const uint32_t max = stress_mwc16();
 
-	(void)args;
+	(void)context;
 
 	for (i = 0; !thread_terminate && (i < max); i++) {
 		size_t offset = stress_mwc32() % mem_size;
@@ -444,13 +459,13 @@ static void HOT OPTIMIZE3 stress_memthrash_flush(
 #endif
 
 static void HOT OPTIMIZE3 stress_memthrash_mfence(
-	const stress_args_t *args,
+	const stress_memthrash_context_t *context,
 	const size_t mem_size)
 {
 	uint32_t i;
 	const uint32_t max = stress_mwc16();
 
-	(void)args;
+	(void)context;
 
 	for (i = 0; !thread_terminate && (i < max); i++) {
 		size_t offset = stress_mwc32() % mem_size;
@@ -463,12 +478,12 @@ static void HOT OPTIMIZE3 stress_memthrash_mfence(
 
 #if defined(MEM_LOCK)
 static void HOT OPTIMIZE3 stress_memthrash_lock(
-	const stress_args_t *args,
+	const stress_memthrash_context_t *context,
 	const size_t mem_size)
 {
 	uint32_t i;
 
-	(void)args;
+	(void)context;
 
 	for (i = 0; !thread_terminate && (i < 64); i++) {
 		size_t offset = stress_mwc32() % mem_size;
@@ -480,7 +495,7 @@ static void HOT OPTIMIZE3 stress_memthrash_lock(
 #endif
 
 static void HOT OPTIMIZE3 stress_memthrash_spinread(
-	const stress_args_t *args,
+	const stress_memthrash_context_t *context,
 	const size_t mem_size)
 {
 	uint32_t i;
@@ -488,8 +503,8 @@ static void HOT OPTIMIZE3 stress_memthrash_spinread(
 	const size_t size = mem_size - (8 * sizeof(*ptr));
 	const size_t offset = (stress_mwc32() % size) & ~(size_t)3;
 
+	(void)context;
 	ptr = (uint32_t *)(((uintptr_t)mem) + offset);
-	(void)args;
 
 	for (i = 0; !thread_terminate && (i < 65536); i++) {
 		(void)*ptr;
@@ -505,7 +520,7 @@ static void HOT OPTIMIZE3 stress_memthrash_spinread(
 }
 
 static void HOT OPTIMIZE3 stress_memthrash_spinwrite(
-	const stress_args_t *args,
+	const stress_memthrash_context_t *context,
 	const size_t mem_size)
 {
 	uint32_t i;
@@ -513,8 +528,8 @@ static void HOT OPTIMIZE3 stress_memthrash_spinwrite(
 	const size_t size = mem_size - (8 * sizeof(*ptr));
 	const size_t offset = (stress_mwc32() % size) & ~(size_t)3;
 
+	(void)context;
 	ptr = (uint32_t *)(((uintptr_t)mem) + offset);
-	(void)args;
 
 	for (i = 0; !thread_terminate && (i < 65536); i++) {
 		*ptr = i;
@@ -530,7 +545,7 @@ static void HOT OPTIMIZE3 stress_memthrash_spinwrite(
 }
 
 static void HOT OPTIMIZE3 stress_memthrash_tlb(
-	const stress_args_t *args,
+	const stress_memthrash_context_t *context,
 	const size_t mem_size)
 {
 	const size_t cache_lines = mem_size >> STRESS_CACHE_LINE_SHIFT;
@@ -541,7 +556,7 @@ static void HOT OPTIMIZE3 stress_memthrash_tlb(
 	volatile uint8_t *ptr;
 	register size_t j, k;
 
-	(void)args;
+	(void)context;
 
 	/* Find size of stride for the given memory size */
 	for (i = 0; i < MEM_SIZE_PRIMES; i++) {
@@ -566,12 +581,12 @@ static void HOT OPTIMIZE3 stress_memthrash_tlb(
 }
 
 static void OPTIMIZE3 TARGET_CLONES stress_memthrash_swapfwdrev(
-	const stress_args_t *args,
+	const stress_memthrash_context_t *context,
 	const size_t mem_size)
 {
 	uint64_t *fwd, *rev, *end = (uint64_t *)((uintptr_t)mem + mem_size);
 
-	(void)args;
+	(void)context;
 	for (fwd = (uint64_t *)mem, rev = end - 1; fwd < end; rev--, fwd++) {
 		register uint64_t tmp;
 
@@ -588,9 +603,36 @@ static void OPTIMIZE3 TARGET_CLONES stress_memthrash_swapfwdrev(
 	}
 }
 
+#if defined(HAVE_MEMTHRASH_NUMA)
+static void OPTIMIZE3 TARGET_CLONES stress_memthrash_numa(
+	const stress_memthrash_context_t *context,
+	const size_t mem_size)
+{
+	uint8_t *ptr;
+	const uint8_t *end = (uint8_t *)((uintptr_t)mem + mem_size);
+	const size_t page_size = context->args->page_size;
+	unsigned long node;
 
-static void stress_memthrash_all(const stress_args_t *args, size_t mem_size);
-static void stress_memthrash_random(const stress_args_t *args, size_t mem_size);
+	if (context->numa_nodes < 1)
+		return;
+
+	node = (unsigned long)stress_mwc32() % context->numa_nodes;
+	(void)memset(context->numa_node_mask, 0, context->numa_node_mask_size);
+
+	for (ptr = (uint8_t *)mem; ptr < end; ptr += page_size) {
+		STRESS_SETBIT(context->numa_node_mask, node);
+
+		(void)shim_mbind((void *)ptr, page_size, MPOL_PREFERRED, context->numa_node_mask, context->max_numa_nodes, 0);
+		STRESS_CLRBIT(context->numa_node_mask, node);
+		node++;
+		if (node >= context->numa_nodes)
+			node = 0;
+	}
+}
+#endif
+
+static void stress_memthrash_all(const stress_memthrash_context_t *context, size_t mem_size);
+static void stress_memthrash_random(const stress_memthrash_context_t *context, size_t mem_size);
 
 static const stress_memthrash_method_info_t memthrash_methods[] = {
 	{ "all",	stress_memthrash_all },		/* MUST always be first! */
@@ -615,6 +657,9 @@ static const stress_memthrash_method_info_t memthrash_methods[] = {
 	{ "memset",	stress_memthrash_memset },
 	{ "memset64",	stress_memthrash_memset64 },
 	{ "mfence",	stress_memthrash_mfence },
+#if defined(HAVE_MEMTHRASH_NUMA)
+	{ "numa",	stress_memthrash_numa },
+#endif
 	{ "prefetch",	stress_memthrash_prefetch },
 	{ "random",	stress_memthrash_random },
 	{ "spinread",	stress_memthrash_spinread },
@@ -625,13 +670,13 @@ static const stress_memthrash_method_info_t memthrash_methods[] = {
 	{ "tlb",	stress_memthrash_tlb },
 };
 
-static void stress_memthrash_all(const stress_args_t *args, size_t mem_size)
+static void stress_memthrash_all(const stress_memthrash_context_t *context, size_t mem_size)
 {
 	static size_t i = 1;
 	const double t = stress_time_now();
 
 	do {
-		memthrash_methods[i].func(args, mem_size);
+		memthrash_methods[i].func(context, mem_size);
 	} while (!thread_terminate && (stress_time_now() - t < 0.01));
 
 	i++;
@@ -639,7 +684,7 @@ static void stress_memthrash_all(const stress_args_t *args, size_t mem_size)
 		i = 1;
 }
 
-static void stress_memthrash_random(const stress_args_t *args, size_t mem_size)
+static void stress_memthrash_random(const stress_memthrash_context_t *context, size_t mem_size)
 {
 	/* loop until we find a good candidate */
 	for (;;) {
@@ -649,7 +694,7 @@ static void stress_memthrash_random(const stress_args_t *args, size_t mem_size)
 		/* Don't run stress_memthrash_random/all to avoid recursion */
 		if ((func != stress_memthrash_random) &&
 		    (func != stress_memthrash_all)) {
-			func(args, mem_size);
+			func(context, mem_size);
 			return;
 		}
 	}
@@ -696,14 +741,13 @@ static void stress_memthrash_find_primes(void)
 
 /*
  *  stress_memthrash_func()
- *	pthread that exits immediately
  */
-static void *stress_memthrash_func(void *arg)
+static void *stress_memthrash_func(void *ctxt)
 {
 	static void *nowt = NULL;
-	const stress_pthread_args_t *parg = (stress_pthread_args_t *)arg;
-	const stress_args_t *args = parg->args;
-	const stress_memthrash_func_t func = (stress_memthrash_func_t)parg->data;
+	const stress_memthrash_context_t *context = (stress_memthrash_context_t *)ctxt;
+	const stress_memthrash_func_t func = context->memthrash_method->func;
+	const stress_args_t *args = context->args;
 
 	/*
 	 *  Block all signals, let controlling thread
@@ -717,12 +761,12 @@ static void *stress_memthrash_func(void *arg)
 		for (j = MATRIX_SIZE_MIN_SHIFT; j <= MATRIX_SIZE_MAX_SHIFT &&
 		     !thread_terminate && keep_stressing(args); j++) {
 			size_t mem_size = 1 << (2 * j);
-
 			size_t i;
+
 			for (i = 0; i < SIZEOF_ARRAY(memthrash_methods); i++)
 				if (func == memthrash_methods[i].func)
 					break;
-			func(args, mem_size);
+			func(context, mem_size);
 			inc_counter(args);
 			shim_sched_yield();
 		}
@@ -778,7 +822,6 @@ static int stress_memthrash_child(const stress_args_t *args, void *ctxt)
 	const uint32_t max_threads = context->max_threads;
 	uint32_t i;
 	int ret;
-	stress_pthread_args_t pargs;
 	stress_pthread_info_t *pthread_info;
 
 	int flags = MAP_PRIVATE | MAP_ANONYMOUS;
@@ -788,16 +831,13 @@ static int stress_memthrash_child(const stress_args_t *args, void *ctxt)
 
 	pthread_info = calloc(max_threads, sizeof(*pthread_info));
 	if (!pthread_info) {
-		pr_inf("%s: failed to allocate pthread information array\n",
+		pr_inf_skip("%s: failed to allocate pthread information array, skipping stressor\n",
 			args->name);
 		return EXIT_NO_RESOURCE;
 	}
 
-	ret = stress_sighandler(args->name, SIGALRM, stress_memthrash_sigalrm_handler, NULL);
-	(void)ret;
+	VOID_RET(int, stress_sighandler(args->name, SIGALRM, stress_memthrash_sigalrm_handler, NULL));
 
-	pargs.args = args;
-	pargs.data = (void *)context->memthrash_method->func;
 
 mmap_retry:
 	mem = mmap(NULL, MEM_SIZE, PROT_READ | PROT_WRITE, flags, -1, 0);
@@ -820,7 +860,7 @@ mmap_retry:
 	for (i = 0; i < max_threads; i++) {
 		pthread_info[i].ret = pthread_create(&pthread_info[i].pthread,
 						NULL, stress_memthrash_func,
-						(void *)&pargs);
+						(void *)context);
 		if (pthread_info[i].ret) {
 			ret = pthread_info[i].ret;
 
@@ -868,9 +908,28 @@ static int stress_memthrash(const stress_args_t *args)
 
 	stress_memthrash_find_primes();
 
+	context.args = args;
 	context.total_cpus = (uint32_t)stress_get_processors_online();
-	context.max_threads = stress_memthrash_max(args->num_instances, context.total_cpus);
 	context.memthrash_method = &memthrash_methods[0];
+	context.max_threads = stress_memthrash_max(args->num_instances, context.total_cpus);
+#if defined(HAVE_MEMTHRASH_NUMA)
+	{
+		const size_t numa_bits = sizeof(unsigned long) * 8;
+		size_t numa_elements;
+
+		context.numa_nodes = stress_numa_count_mem_nodes(&context.max_numa_nodes);
+
+		numa_elements = (context.max_numa_nodes + numa_bits - 1) / numa_bits;
+		numa_elements = numa_elements ? numa_elements: 1;
+		context.numa_node_mask = calloc(context.max_numa_nodes, numa_elements);
+		context.numa_node_mask_size = (size_t)context.max_numa_nodes * numa_elements;
+		if (!context.numa_node_mask) {
+			pr_inf_skip("%s: could not allocate %zd numa elements in numa mask, skipping stressor\n",
+				args->name, numa_elements);
+			return EXIT_NO_RESOURCE;
+		}
+	}
+#endif
 
 	(void)stress_get_setting("memthrash-method", &context.memthrash_method);
 
@@ -895,6 +954,9 @@ static int stress_memthrash(const stress_args_t *args)
 	rc = stress_oomable_child(args, &context, stress_memthrash_child, STRESS_OOMABLE_NORMAL);
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
+#if defined(HAVE_MEMTHRASH_NUMA)
+	free(context.numa_node_mask);
+#endif
 
 	return rc;
 }
@@ -926,9 +988,10 @@ static const stress_opt_set_func_t opt_set_funcs[] = {
 };
 
 stressor_info_t stress_memthrash_info = {
-	.stressor = stress_not_implemented,
+	.stressor = stress_unimplemented,
 	.class = CLASS_MEMORY,
 	.opt_set_funcs = opt_set_funcs,
-	.help = help
+	.help = help,
+	.unimplemented_reason = "built without pthread support"
 };
 #endif
