@@ -47,7 +47,8 @@ static int do_mlock(const void *addr, size_t len)
 	static bool use_mlock2 = true;
 
 	if (use_mlock2) {
-		uint32_t rnd = stress_mwc32() >> 5;
+		const uint32_t rnd = stress_mwc32() >> 5;
+
 		/* Randomly use mlock2 or mlock */
 		if (rnd & 1) {
 			const int flags = (rnd & 2) ?
@@ -185,9 +186,9 @@ static int stress_mlock_child(const stress_args_t *args, void *context)
 	const size_t page_size = args->page_size;
 	const size_t max = stress_mlock_max_lockable();
 	const size_t mappings_len = max * sizeof(*mappings);
-	size_t shmall, freemem, totalmem, freeswap;
+	size_t shmall, freemem, totalmem, freeswap, totalswap;
 
-	stress_get_memlimits(&shmall, &freemem, &totalmem, &freeswap);
+	stress_get_memlimits(&shmall, &freemem, &totalmem, &freeswap, &totalswap);
 
 	(void)context;
 
@@ -216,6 +217,10 @@ static int stress_mlock_child(const stress_args_t *args, void *context)
 			if (!keep_stressing(args))
 				break;
 
+			/* Low memory avoidance, re-start */
+			if ((g_opt_flags & OPT_FLAGS_OOM_AVOID) && stress_low_memory(page_size * 3))
+				break;
+
 			mappings[n] = (uint8_t *)mmap(NULL, page_size * 3,
 				PROT_READ | PROT_WRITE,
 				MAP_SHARED | MAP_ANONYMOUS, -1, 0);
@@ -323,8 +328,9 @@ stressor_info_t stress_mlock_info = {
 };
 #else
 stressor_info_t stress_mlock_info = {
-	.stressor = stress_not_implemented,
+	.stressor = stress_unimplemented,
 	.class = CLASS_VM | CLASS_OS,
-	.help = help
+	.help = help,
+	.unimplemented_reason = "built without mlock() support or _POSIX_MEMLOCK_RANGE defined"
 };
 #endif
