@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,15 +16,17 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
+#if defined(HAVE_SYS_IPC_H)
+#include <sys/ipc.h>
+#endif
+
+#if defined(HAVE_SYS_MSG_H)
+#include <sys/msg.h>
+#endif
+
 #define STRESS_MAX_IDS		(1024)
 
 static const stress_help_t help[] = {
@@ -157,24 +160,20 @@ static void stress_msgget(void)
  */
 static void stress_msgsnd(const int msgq_id)
 {
-	int ret;
 	stress_msg_t msg;
 
 	/* Invalid msgq_id */
 	msg.mtype = 1;
 	msg.value = 0;
-	ret = msgsnd(-1, &msg, sizeof(msg), 0);
-	(void)ret;
+	VOID_RET(int, msgsnd(-1, &msg, sizeof(msg.value), 0));
 
 	/* Zero msg length + 0 msg.type */
 	msg.mtype = 0;
-	ret = msgsnd(msgq_id, &msg, 0, 0);
-	(void)ret;
+	VOID_RET(int, msgsnd(msgq_id, &msg, 0, 0));
 
 	/* Illegal flags, may or may not succeed */
 	msg.mtype = 1;
-	ret = msgsnd(msgq_id, &msg, sizeof(msg), ~0);
-	(void)ret;
+	VOID_RET(int, msgsnd(msgq_id, &msg, sizeof(msg.value), ~0));
 }
 
 #if defined(__linux__)
@@ -240,13 +239,13 @@ static int stress_msg(const stress_args_t *args)
 
 	msgq_ids = calloc(max_ids, sizeof(*msgq_ids));
 	if (!msgq_ids) {
-		pr_inf("%s: failed to allocate msgq id array\n", args->name);
+		pr_inf_skip("%s: failed to allocate msgq id array, skipping stressor\n", args->name);
 		return EXIT_NO_RESOURCE;
 	}
 
 	msgq_id = msgget(IPC_PRIVATE, S_IRUSR | S_IWUSR | IPC_CREAT | IPC_EXCL);
 	if (msgq_id < 0) {
-		const int ret = exit_status(errno);
+		const int ret = stress_exit_status(errno);
 
 		pr_fail("%s: msgget failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
@@ -282,7 +281,6 @@ again:
 		rc = EXIT_FAILURE;
 		goto cleanup;
 	} else if (pid == 0) {
-		(void)setpgid(0, g_pgrp);
 		stress_parent_died_alarm();
 		(void)sched_settings_apply(true);
 
@@ -300,11 +298,8 @@ again:
 				 *  and we don't care if it succeeds or not
 				 */
 				if ((i & 0xfff) == 0) {
-					ssize_t ret;
-
-					ret = msgrcv(msgq_id, &msg, sizeof(msg.value), mtype,
-						MSG_COPY | IPC_NOWAIT);
-					(void)ret;
+					VOID_RET(ssize_t, msgrcv(msgq_id, &msg, sizeof(msg.value), mtype,
+						MSG_COPY | IPC_NOWAIT));
 				}
 #endif
 
@@ -313,7 +308,7 @@ again:
 					(void)msgrcv(-1, &msg, sizeof(msg.value), mtype, 0);
 
 					/* Exercise invalid msgrcv message size */
-					(void)msgrcv(msgq_id, &msg, -1, mtype, 0);
+					(void)msgrcv(msgq_id, &msg, (size_t)-1, mtype, 0);
 					(void)msgrcv(msgq_id, &msg, 0, mtype, 0);
 
 					/* Exercise invalid msgrcv message flag */
@@ -352,8 +347,6 @@ again:
 		int status;
 
 		/* Parent */
-		(void)setpgid(pid, g_pgrp);
-
 		msg.value = 0;
 
 		do {
@@ -419,13 +412,15 @@ stressor_info_t stress_msg_info = {
 	.stressor = stress_msg,
 	.class = CLASS_SCHEDULER | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
 stressor_info_t stress_msg_info = {
-	.stressor = stress_not_implemented,
+	.stressor = stress_unimplemented,
 	.class = CLASS_SCHEDULER | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
-	.help = help
+	.help = help,
+	.unimplemented_reason = "built without sys/ipc.h, sys/msg.h or System V message queues support"
 };
 #endif
