@@ -35,8 +35,9 @@
 #define _LARGEFILE64_SOURCE
 #endif
 #if !defined(__PCC__) && 	\
-    !defined(__TINYC__)
-#define _FORTIFY_SOURCE 2
+    !defined(__TINYC__) &&	\
+    !defined(_FORTIFY_SOURCE)
+#define _FORTIFY_SOURCE 	(2)
 #endif
 
 #if !defined(_FILE_OFFSET_BITS)
@@ -197,15 +198,6 @@ typedef unsigned long int __kernel_ulong_t;
 #define shim_itimer_which_t	int
 #endif
 
-#if defined(__sun__)
-#if defined(HAVE_GETDOMAINNAME)
-extern int getdomainname(char *name, size_t len);
-#endif
-#if defined(HAVE_SETDOMAINNAME)
-extern int setdomainname(const char *name, size_t len);
-#endif
-#endif
-
 #define STRESS_BIT_U(shift)	(1U << shift)
 #define STRESS_BIT_UL(shift)	(1UL << shift)
 #define STRESS_BIT_ULL(shift)	(1ULL << shift)
@@ -327,6 +319,9 @@ extern int setdomainname(const char *name, size_t len);
 #define OPT_FLAGS_STDOUT	 STRESS_BIT_ULL(43)	/* --stdout */
 #define OPT_FLAGS_KLOG_CHECK	 STRESS_BIT_ULL(44)	/* --klog-check */
 #define OPT_FLAGS_DRY_RUN	 STRESS_BIT_ULL(45)	/* Don't actually run */
+#define OPT_FLAGS_OOM_AVOID	 STRESS_BIT_ULL(46)	/* --oom-avoid */
+#define OPT_FLAGS_TZ_INFO	 STRESS_BIT_ULL(47)	/* Require thermal zone info */
+#define OPT_FLAGS_LOG_LOCKLESS	 STRESS_BIT_ULL(48)	/* --log-lockless */
 
 #define OPT_FLAGS_MINMAX_MASK		\
 	(OPT_FLAGS_MINIMIZE | OPT_FLAGS_MAXIMIZE)
@@ -389,8 +384,6 @@ typedef enum {
 	TYPE_ID_UINTPTR_T
 } stress_type_id_t;
 
-typedef struct stress_stressor_info *stress_pstressor_info_t;
-
 /*
  *  Per ELISA request, we have a duplicated counter
  *  and run_ok flag in a different shared memory region
@@ -409,7 +402,7 @@ typedef struct {
 /* settings for storing opt arg parsed data */
 typedef struct stress_setting {
 	struct stress_setting *next;	/* next setting in list */
-	stress_pstressor_info_t	proc;
+	struct stress_stressor_info *proc;
 	char *name;			/* name of setting */
 	stress_type_id_t type_id;	/* setting type */
 	bool		global;		/* true if global */
@@ -449,9 +442,6 @@ typedef union {
 	volatile void 		*void_ptr_val;
 } stress_put_val_t;
 
-/* Large prime to stride around large VM regions */
-#define PRIME_64		(0x8f0000000017116dULL)
-
 typedef uint32_t stress_class_t;
 
 typedef struct {
@@ -476,10 +466,10 @@ typedef struct {
 	const uint32_t instance;	/* stressor instance # */
 	const uint32_t num_instances;	/* number of instances */
 	pid_t pid;			/* stressor pid */
-	pid_t ppid;			/* stressor ppid */
 	size_t page_size;		/* page size */
 	stress_mapped_t *mapped;	/* mmap'd pages, addr of g_shared mapped */
 	stress_misc_stats_t *misc_stats;/* misc per stressor stats */
+	const struct stressor_info *info; /* stressor info */
 } stress_args_t;
 
 typedef struct {
@@ -494,7 +484,7 @@ typedef enum {
 } stress_verify_t;
 
 /* stressor information */
-typedef struct {
+typedef struct stressor_info {
 	int (*stressor)(const stress_args_t *args);	/* stressor function */
 	int (*supported)(const char *name);	/* return 0 = supported, -1, not */
 	void (*init)(void);		/* stressor init, NULL = ignore */
@@ -504,73 +494,15 @@ typedef struct {
 	const stress_opt_set_func_t *opt_set_funcs;	/* option functions */
 	const stress_help_t *help;	/* stressor help options */
 	const stress_class_t class;	/* stressor class */
-	const stress_verify_t verify;	/* true = has verification mode */
+	const stress_verify_t verify;	/* verification mode */
+	const char *unimplemented_reason;	/* unsupported reason message */
 } stressor_info_t;
 
-/* pthread wrapped stress_args_t */
-typedef struct {
-	const stress_args_t *args;	/* Stress test args */
-	void *data;			/* Per thread private data */
-	int pthread_ret;		/* Per thread return value */
-} stress_pthread_args_t;
-
-/* string hash linked list type */
-typedef struct stress_hash {
-	struct stress_hash *next; 	/* next hash item */
-} stress_hash_t;
-
-/* string hash table */
-typedef struct {
-	stress_hash_t	**table;	/* hash table */
-	size_t		n;		/* number of hash items in table */
-} stress_hash_table_t;
-
-/* vmstat information */
-typedef struct {			/* vmstat column */
-	uint64_t	procs_running;	/* r */
-	uint64_t	procs_blocked;	/* b */
-	uint64_t	swap_total;	/* swpd info, total */
-	uint64_t	swap_free;	/* swpd info, free */
-	uint64_t	swap_used;	/* swpd used = total - free */
-	uint64_t	memory_free;	/* free */
-	uint64_t	memory_buff;	/* buff */
-	uint64_t	memory_cache;	/* cache */
-	uint64_t	swap_in;	/* si */
-	uint64_t	swap_out;	/* so */
-	uint64_t	block_in;	/* bi */
-	uint64_t	block_out;	/* bo */
-	uint64_t	interrupt;	/* in */
-	uint64_t	context_switch;	/* cs */
-	uint64_t	user_time;	/* us */
-	uint64_t	system_time;	/* sy */
-	uint64_t	idle_time;	/* id */
-	uint64_t	wait_time;	/* wa */
-	uint64_t	stolen_time;	/* st */
-} stress_vmstat_t;
-
-/* iostat information, from /sys/block/$dev/stat */
-typedef struct {
-	uint64_t	read_io;	/* number of read I/Os processed */
-	uint64_t	read_merges;	/* number of read I/Os merged with in-queue I/O */
-	uint64_t	read_sectors;	/* number of sectors read */
-	uint64_t	read_ticks;	/* total wait time for read requests */
-	uint64_t	write_io;	/* number of write I/Os processed */
-	uint64_t	write_merges;	/* number of write I/Os merged with in-queue I/O */
-	uint64_t	write_sectors;	/* number of sectors written */
-	uint64_t	write_ticks;	/* total wait time for write requests */
-	uint64_t	in_flight;	/* number of I/Os currently in flight */
-	uint64_t	io_ticks;	/* total time this block device has been active */
-	uint64_t	time_in_queue;	/* total wait time for all requests */
-	uint64_t	discard_io;	/* number of discard I/Os processed */
-	uint64_t	discard_merges;	/* number of discard I/Os merged with in-queue I/O */
-	uint64_t	discard_sectors;/* number of sectors discarded */
-	uint64_t	discard_ticks;	/* total wait time for discard requests */
-} stress_iostat_t;
 
 /* gcc 4.7 and later support vector ops */
 #if defined(__GNUC__) &&	\
     NEED_GNUC(4, 7, 0)
-#define STRESS_VECTOR	1
+#define STRESS_VECTOR	(1)
 #endif
 
 /* gcc 7.0 and later support __attribute__((fallthrough)); */
@@ -649,14 +581,23 @@ typedef struct {
 
 /* -O0 attribute support */
 #if defined(__GNUC__) &&	\
-    !defined(__clang__) &&	\
     !defined(__ICC) &&		\
     NEED_GNUC(4, 6, 0)
 #define OPTIMIZE0 	__attribute__((optimize("-O0")))
+#elif (defined(__clang__) && NEED_CLANG(10, 0, 0))
+#define OPTIMIZE0	__attribute__((optnone))
 #else
 #define OPTIMIZE0
 #endif
 
+/* loop unroll hint */
+#if defined(HAVE_UNROLL) &&	\
+    NEED_GNUC(8, 0, 0)
+#define UNROLL 		__attribute__((optimize("unroll-loops")))
+#else
+#define UNROLL
+#endif
+
 /* warn unused attribute */
 #if (defined(__GNUC__) && NEED_GNUC(4, 2, 0)) ||	\
     (defined(__clang__) && NEED_CLANG(3, 0, 0))
@@ -668,7 +609,8 @@ typedef struct {
 #if ((defined(__GNUC__) && NEED_GNUC(3, 3, 0)) ||	\
      (defined(__clang__) && NEED_CLANG(3, 0, 0)) ||	\
      (defined(__ICC) && NEED_ICC(2021, 0, 0))) &&	\
-    !defined(__PCC__)
+    !defined(__PCC__) &&				\
+    !defined(__minix__)
 #define ALIGNED(a)	__attribute__((aligned(a)))
 #else
 #define ALIGNED(a)
@@ -680,9 +622,14 @@ typedef struct {
 /* Force alignment to nearest 64 bytes */
 #define ALIGN64		ALIGNED(64)
 
+
 #if (defined(__GNUC__) && NEED_GNUC(4, 6, 0)) ||	\
     (defined(__clang__) && NEED_CLANG(3, 0, 0))
+#if (defined(__APPLE__) && defined(__MACH__))
+#define SECTION(s)	__attribute__((__section__(# s "," # s)))
+#else
 #define SECTION(s)	__attribute__((__section__(# s)))
+#endif
 #else
 #define SECTION(s)
 #endif
@@ -704,7 +651,7 @@ typedef struct {
     !defined(__APPLE__) &&				\
     !defined(BUILD_STATIC)
 #define MLOCKED_TEXT	__attribute__((__section__("mlocked_text")))
-#define MLOCKED_SECTION 1
+#define MLOCKED_SECTION	(1)
 #else
 #define MLOCKED_TEXT
 #endif
@@ -743,9 +690,7 @@ typedef struct {
 #define shim_vfork()		g_shared->vfork()
 
 /* Logging helpers */
-extern int pr_msg(FILE *fp, const uint64_t flag,
-	const char *const fmt, va_list va) FORMAT(printf, 3, 0);
-extern int pr_yaml(FILE *fp, const char *const fmt, ...) FORMAT(printf, 2, 3);
+extern int  pr_yaml(FILE *fp, const char *const fmt, ...) FORMAT(printf, 2, 3);
 extern void pr_yaml_runinfo(FILE *fp);
 extern void pr_runinfo(void);
 extern void pr_openlog(const char *filename);
@@ -762,22 +707,10 @@ extern void pr_fail(const char *fmt, ...) FORMAT(printf, 1, 2);
 extern void pr_tidy(const char *fmt, ...) FORMAT(printf, 1, 2);
 extern void pr_warn(const char *fmt, ...) FORMAT(printf, 1, 2);
 
-extern void pr_lock(bool *locked);
-extern void pr_unlock(bool *locked);
-extern void pr_inf_lock(bool *locked, const char *fmt, ...)  FORMAT(printf, 2, 3);
-extern void pr_dbg_lock(bool *locked, const char *fmt, ...)  FORMAT(printf, 2, 3);
-
-#if defined(HAVE_SYSLOG_H)
-#define shim_syslog(priority, format, ...)	\
-		syslog(priority, format, __VA_ARGS__)
-#define shim_openlog(ident, option, facility) \
-		openlog(ident, option, facility)
-#define shim_closelog()		closelog()
-#else
-#define shim_syslog(priority, format, ...)
-#define shim_openlog(ident, option, facility)
-#define shim_closelog()
-#endif
+extern void pr_lock_init(void);
+extern void pr_lock(void);
+extern void pr_unlock(void);
+extern void pr_lock_exited(const pid_t pid);
 
 /* Memory size constants */
 #define KB			(1ULL << 10)
@@ -791,9 +724,6 @@ extern void pr_dbg_lock(bool *locked, const char *fmt, ...)  FORMAT(printf, 2, 3
 #define ONE_MILLIONTH		(1.0E-6)
 #define ONE_THOUSANDTH		(1.0E-3)
 
-#define PAGE_4K_SHIFT		(12)
-#define PAGE_4K			(1 << PAGE_4K_SHIFT)
-
 #define STACK_ALIGNMENT		(64)	/* thread stacks align to 64 bytes */
 
 #define MIN_OPS			(1ULL)
@@ -832,7 +762,7 @@ extern void pr_dbg_lock(bool *locked, const char *fmt, ...)  FORMAT(printf, 2, 3
 
 #define STR_SHARED_SIZE		(65536 * 32)
 #define MEM_CACHE_SIZE		(65536 * 32)
-#define DEFAULT_CACHE_LEVEL     3
+#define DEFAULT_CACHE_LEVEL     (3)
 #define UNDEFINED		(-1)
 
 #define PAGE_MAPPED		(0x01)
@@ -870,33 +800,6 @@ extern int stress_lock_destroy(void *lock_handle);
 extern int stress_lock_acquire(void *lock_handle);
 extern int stress_lock_release(void *lock_handle);
 
-/* pthread porting shims, spinlock or fallback to mutex */
-#if defined(HAVE_LIB_PTHREAD)
-#if defined(HAVE_LIB_PTHREAD_SPINLOCK) &&	\
-    !defined(__DragonFly__) &&			\
-    !defined(__OpenBSD__)
-typedef pthread_spinlock_t 	shim_pthread_spinlock_t;
-
-#define SHIM_PTHREAD_PROCESS_SHARED		PTHREAD_PROCESS_SHARED
-#define SHIM_PTHREAD_PROCESS_PRIVATE		PTHREAD_PROCESS_PRIVATE
-
-#define shim_pthread_spin_lock(lock)		pthread_spin_lock(lock)
-#define shim_pthread_spin_unlock(lock)		pthread_spin_unlock(lock)
-#define shim_pthread_spin_init(lock, shared)	pthread_spin_init(lock, shared)
-#define shim_pthread_spin_destroy(lock)		pthread_spin_destroy(lock)
-#else
-typedef pthread_mutex_t		shim_pthread_spinlock_t;
-
-#define SHIM_PTHREAD_PROCESS_SHARED		NULL
-#define SHIM_PTHREAD_PROCESS_PRIVATE		NULL
-
-#define shim_pthread_spin_lock(lock)		pthread_mutex_lock(lock)
-#define shim_pthread_spin_unlock(lock)		pthread_mutex_unlock(lock)
-#define shim_pthread_spin_init(lock, shared)	pthread_mutex_init(lock, shared)
-#define shim_pthread_spin_destroy(lock)		pthread_mutex_destroy(lock)
-#endif
-#endif
-
 /* stress process prototype */
 typedef int (*stress_func_t)(const stress_args_t *args);
 
@@ -951,6 +854,8 @@ typedef struct {
 	uint64_t counter;		/* number of bogo ops */
 	double start;			/* wall clock start time */
 	double finish;			/* wall clock stop time */
+	pid_t pid;			/* stressor pid */
+	bool signalled;			/* set true if signalled with a kill */
 #if defined(STRESS_PERF_STATS)
 	stress_perf_t sp;		/* perf counters */
 #endif
@@ -962,6 +867,7 @@ typedef struct {
 #if defined(HAVE_GETRUSAGE)
 	double rusage_utime;		/* rusage user time */
 	double rusage_stime;		/* rusage system time */
+	long int rusage_maxrss;		/* rusage max RSS, 0 = unused */
 #else
 	struct tms tms;			/* run time stats of process */
 #endif
@@ -984,6 +890,13 @@ typedef struct {
 	uint32_t mem_cache_ways;			/* cache ways size */
 	uint64_t zero;					/* zero'd data */
 	void *nullptr;					/* Null pointer */
+#if defined(HAVE_ATOMIC_COMPARE_EXCHANGE) &&	\
+    defined(HAVE_ATOMIC_STORE)
+	double pr_whence;				/* pr_* lock time */
+	pid_t pr_atomic_lock;				/* pr_* atomic spinlock */
+	int pr_lock_count;				/* pr_* lock count, release when zero */
+	pid_t pr_pid;					/* pid owning the lock */
+#endif
 	bool     klog_error;				/* True if error detected in klog */
 	pid_t (*vfork)(void);				/* vfork syscall */
 	stress_mapped_t mapped;				/* mmap'd pages to help testing */
@@ -1000,7 +913,8 @@ typedef struct {
 		uint8_t	 padding2;			/* more padding */
 	} atomic;					/* Shared atomic temp vars */
 	struct {
-		uint32_t futex[STRESS_PROCS_MAX];	/* Shared futexes */
+		/* futexes must be aligned to avoid -EINVAL */
+		uint32_t futex[STRESS_PROCS_MAX] ALIGNED(4);/* Shared futexes */
 		uint64_t timeout[STRESS_PROCS_MAX];	/* Shared futex timeouts */
 	} futex;
 #if defined(HAVE_SEM_SYSV) && 	\
@@ -1032,326 +946,16 @@ typedef struct {
 	uint8_t  str_shared[STR_SHARED_SIZE];		/* str copying buffer */
 	stress_checksum_t *checksums;			/* per stressor counter checksum */
 	size_t	checksums_length;			/* size of checksums mapping */
+	struct {
+		uint8_t allocated[65536 / 8];
+		void *lock;
+	} net_port_map;
 	struct {
 		uint32_t ready;				/* incremented when rawsock stressor is ready */
 	} rawsock;
 	stress_stats_t stats[];				/* Shared statistics */
 } stress_shared_t;
 
-/* Stress test classes */
-typedef struct {
-	stress_class_t class;		/* Class type bit mask */
-	const char *name;		/* Name of class */
-} stress_class_info_t;
-
-#define STRESSORS(MACRO)	\
-	MACRO(access) 		\
-	MACRO(af_alg) 		\
-	MACRO(affinity) 	\
-	MACRO(aio) 		\
-	MACRO(aiol) 		\
-	MACRO(apparmor) 	\
-	MACRO(alarm)		\
-	MACRO(atomic)		\
-	MACRO(bad_altstack) 	\
-	MACRO(bad_ioctl) 	\
-	MACRO(bigheap)		\
-	MACRO(bind_mount)	\
-	MACRO(binderfs)		\
-	MACRO(branch)		\
-	MACRO(brk)		\
-	MACRO(bsearch)		\
-	MACRO(cache)		\
-	MACRO(cacheline)	\
-	MACRO(cap)		\
-	MACRO(chattr)		\
-	MACRO(chdir)		\
-	MACRO(chmod)		\
-	MACRO(chown)		\
-	MACRO(chroot)		\
-	MACRO(clock)		\
-	MACRO(clone)		\
-	MACRO(close)		\
-	MACRO(context)		\
-	MACRO(copy_file)	\
-	MACRO(cpu)		\
-	MACRO(cpu_online)	\
-	MACRO(crypt)		\
-	MACRO(cyclic)		\
-	MACRO(daemon)		\
-	MACRO(dccp)		\
-	MACRO(dekker)		\
-	MACRO(dentry)		\
-	MACRO(dev)		\
-	MACRO(dev_shm)		\
-	MACRO(dir)		\
-	MACRO(dirdeep)		\
-	MACRO(dirmany)		\
-	MACRO(dnotify)		\
-	MACRO(dup)		\
-	MACRO(dynlib)		\
-	MACRO(efivar)		\
-	MACRO(enosys)		\
-	MACRO(env)		\
-	MACRO(epoll)		\
-	MACRO(eventfd) 		\
-	MACRO(exec)		\
-	MACRO(exit_group)	\
-	MACRO(fallocate)	\
-	MACRO(fanotify)		\
-	MACRO(fault)		\
-	MACRO(fcntl)		\
-	MACRO(fiemap)		\
-	MACRO(fifo)		\
-	MACRO(file_ioctl)	\
-	MACRO(filename)		\
-	MACRO(flock)		\
-	MACRO(fork)		\
-	MACRO(fp_error)		\
-	MACRO(fpunch)		\
-	MACRO(fstat)		\
-	MACRO(full)		\
-	MACRO(funccall)		\
-	MACRO(funcret)		\
-	MACRO(futex)		\
-	MACRO(get)		\
-	MACRO(getdent)		\
-	MACRO(getrandom)	\
-	MACRO(goto)		\
-	MACRO(gpu)		\
-	MACRO(handle)		\
-	MACRO(hash)		\
-	MACRO(hdd)		\
-	MACRO(heapsort)		\
-	MACRO(hrtimers)		\
-	MACRO(hsearch)		\
-	MACRO(icache)		\
-	MACRO(icmp_flood)	\
-	MACRO(idle_page)	\
-	MACRO(inode_flags)	\
-	MACRO(inotify)		\
-	MACRO(io)		\
-	MACRO(iomix)		\
-	MACRO(ioport)		\
-	MACRO(ioprio)		\
-	MACRO(io_uring)		\
-	MACRO(ipsec_mb)		\
-	MACRO(itimer)		\
-	MACRO(jpeg)		\
-	MACRO(judy)		\
-	MACRO(kcmp)		\
-	MACRO(key)		\
-	MACRO(kill)		\
-	MACRO(klog)		\
-	MACRO(kvm)		\
-	MACRO(l1cache)		\
-	MACRO(landlock)		\
-	MACRO(lease)		\
-	MACRO(link)		\
-	MACRO(list)		\
-	MACRO(loadavg)		\
-	MACRO(locka)		\
-	MACRO(lockbus)		\
-	MACRO(lockf)		\
-	MACRO(lockofd)		\
-	MACRO(longjmp)		\
-	MACRO(loop)		\
-	MACRO(lsearch)		\
-	MACRO(madvise)		\
-	MACRO(malloc)		\
-	MACRO(matrix)		\
-	MACRO(matrix_3d)	\
-	MACRO(mcontend)		\
-	MACRO(membarrier)	\
-	MACRO(memcpy)		\
-	MACRO(memfd)		\
-	MACRO(memhotplug)	\
-	MACRO(memrate)		\
-	MACRO(memthrash)	\
-	MACRO(mergesort)	\
-	MACRO(mincore)		\
-	MACRO(misaligned)	\
-	MACRO(mknod)		\
-	MACRO(mlock)		\
-	MACRO(mlockmany)	\
-	MACRO(mmap)		\
-	MACRO(mmapaddr)		\
-	MACRO(mmapfixed)	\
-	MACRO(mmapfork)		\
-	MACRO(mmaphuge)		\
-	MACRO(mmapmany)		\
-	MACRO(mprotect)		\
-	MACRO(mq)		\
-	MACRO(mremap)		\
-	MACRO(msg)		\
-	MACRO(msync)		\
-	MACRO(msyncmany)	\
-	MACRO(munmap)		\
-	MACRO(mutex)		\
-	MACRO(nanosleep)	\
-	MACRO(netdev)		\
-	MACRO(netlink_proc)	\
-	MACRO(netlink_task)	\
-	MACRO(nice)		\
-	MACRO(nop)		\
-	MACRO(null)		\
-	MACRO(numa)		\
-	MACRO(oom_pipe)		\
-	MACRO(opcode)		\
-	MACRO(open)		\
-	MACRO(pageswap)		\
-	MACRO(pci)		\
-	MACRO(personality)	\
-	MACRO(peterson)		\
-	MACRO(physpage)		\
-	MACRO(pidfd)		\
-	MACRO(ping_sock)	\
-	MACRO(pipe)		\
-	MACRO(pipeherd)		\
-	MACRO(pkey)		\
-	MACRO(poll)		\
-	MACRO(prctl)		\
-	MACRO(prefetch)		\
-	MACRO(procfs)		\
-	MACRO(pthread)		\
-	MACRO(ptrace)		\
-	MACRO(pty)		\
-	MACRO(qsort)		\
-	MACRO(quota)		\
-	MACRO(radixsort)	\
-	MACRO(randlist)		\
-	MACRO(ramfs)		\
-	MACRO(rawdev)		\
-	MACRO(rawpkt)		\
-	MACRO(rawsock)		\
-	MACRO(rawudp)		\
-	MACRO(rdrand)		\
-	MACRO(readahead)	\
-	MACRO(reboot)		\
-	MACRO(remap)		\
-	MACRO(rename)		\
-	MACRO(resched)		\
-	MACRO(resources)	\
-	MACRO(revio)		\
-	MACRO(rlimit)		\
-	MACRO(rmap)		\
-	MACRO(rseq)		\
-	MACRO(rtc)		\
-	MACRO(schedpolicy)	\
-	MACRO(sctp)		\
-	MACRO(seal)		\
-	MACRO(seccomp)		\
-	MACRO(secretmem)	\
-	MACRO(seek)		\
-	MACRO(sem)		\
-	MACRO(sem_sysv)		\
-	MACRO(sendfile)		\
-	MACRO(session)		\
-	MACRO(set)		\
-	MACRO(shellsort)	\
-	MACRO(shm)		\
-	MACRO(shm_sysv)		\
-	MACRO(sigabrt)		\
-	MACRO(sigchld)		\
-	MACRO(sigfd)		\
-	MACRO(sigfpe)		\
-	MACRO(sigio)		\
-	MACRO(signal)		\
-	MACRO(signest)		\
-	MACRO(sigpending)	\
-	MACRO(sigpipe)		\
-	MACRO(sigq)		\
-	MACRO(sigrt)		\
-	MACRO(sigsegv)		\
-	MACRO(sigsuspend)	\
-	MACRO(sigtrap)		\
-	MACRO(skiplist)		\
-	MACRO(sleep)		\
-	MACRO(smi)		\
-	MACRO(sock)		\
-	MACRO(sockabuse)	\
-	MACRO(sockdiag)		\
-	MACRO(sockfd)		\
-	MACRO(sockpair)		\
-	MACRO(sockmany)		\
-	MACRO(softlockup)	\
-	MACRO(spawn)		\
-	MACRO(sparsematrix)	\
-	MACRO(splice)		\
-	MACRO(stack)		\
-	MACRO(stackmmap)	\
-	MACRO(str)		\
-	MACRO(stream)		\
-	MACRO(swap)		\
-	MACRO(switch)		\
-	MACRO(symlink)		\
-	MACRO(sync_file)	\
-	MACRO(syncload)		\
-	MACRO(sysbadaddr)	\
-	MACRO(sysinfo)		\
-	MACRO(sysinval)		\
-	MACRO(sysfs)		\
-	MACRO(tee)		\
-	MACRO(timer)		\
-	MACRO(timerfd)		\
-	MACRO(tlb_shootdown)	\
-	MACRO(tmpfs)		\
-	MACRO(touch)		\
-	MACRO(tree)		\
-	MACRO(tsc)		\
-	MACRO(tsearch)		\
-	MACRO(tun)		\
-	MACRO(udp)		\
-	MACRO(udp_flood)	\
-	MACRO(unshare)		\
-	MACRO(uprobe)		\
-	MACRO(urandom)		\
-	MACRO(userfaultfd)	\
-	MACRO(usersyscall)	\
-	MACRO(utime)		\
-	MACRO(vdso)		\
-	MACRO(vecmath)		\
-	MACRO(vecwide)		\
-	MACRO(verity)		\
-	MACRO(vfork)		\
-	MACRO(vforkmany)	\
-	MACRO(vm)		\
-	MACRO(vm_addr)		\
-	MACRO(vm_rw)		\
-	MACRO(vm_segv)		\
-	MACRO(vm_splice)	\
-	MACRO(wait)		\
-	MACRO(watchdog)		\
-	MACRO(wcs)		\
-	MACRO(x86syscall)	\
-	MACRO(xattr)		\
-	MACRO(yield)		\
-	MACRO(zero)		\
-	MACRO(zlib)		\
-	MACRO(zombie)
-
-/*
- *  Declaration of stress_*_info object
- */
-#define STRESSOR_ENUM(name)	\
-	STRESS_ ## name,
-
-/*
- *  Declaration of stress_*_info object
- */
-#define STRESSOR_DECL(name)     \
-	extern stressor_info_t stress_ ## name ## _info;
-
-STRESSORS(STRESSOR_DECL)
-
-/* Stress tests */
-typedef enum {
-	STRESS_START = -1,
-	STRESSORS(STRESSOR_ENUM)
-	/* STRESS_MAX must be last one */
-	STRESS_MAX
-} stress_id_t;
-
 /* Command line long options */
 typedef enum {
 	OPT_undefined = 0,
@@ -1527,11 +1131,12 @@ typedef enum {
 
 	OPT_cyclic,
 	OPT_cyclic_ops,
+	OPT_cyclic_dist,
 	OPT_cyclic_method,
 	OPT_cyclic_policy,
 	OPT_cyclic_prio,
+	OPT_cyclic_samples,
 	OPT_cyclic_sleep,
-	OPT_cyclic_dist,
 
 	OPT_daemon,
 	OPT_daemon_ops,
@@ -1603,6 +1208,9 @@ typedef enum {
 	OPT_exec,
 	OPT_exec_ops,
 	OPT_exec_max,
+	OPT_exec_method,
+	OPT_exec_fork_method,
+	OPT_exec_no_pthread,
 
 	OPT_exit_group,
 	OPT_exit_group_ops,
@@ -1613,6 +1221,9 @@ typedef enum {
 	OPT_fanotify,
 	OPT_fanotify_ops,
 
+	OPT_far_branch,
+	OPT_far_branch_ops,
+
 	OPT_fault,
 	OPT_fault_ops,
 
@@ -1624,6 +1235,7 @@ typedef enum {
 	OPT_fiemap_bytes,
 
 	OPT_fifo,
+	OPT_fifo_data_size,
 	OPT_fifo_ops,
 	OPT_fifo_readers,
 
@@ -1637,6 +1249,9 @@ typedef enum {
 	OPT_flock,
 	OPT_flock_ops,
 
+	OPT_flushcache,
+	OPT_flushcache_ops,
+
 	OPT_fork_ops,
 	OPT_fork_max,
 	OPT_fork_vm,
@@ -1647,6 +1262,9 @@ typedef enum {
 	OPT_fpunch,
 	OPT_fpunch_ops,
 
+	OPT_fsize,
+	OPT_fsize_ops,
+
 	OPT_fstat,
 	OPT_fstat_ops,
 	OPT_fstat_dir,
@@ -1754,6 +1372,7 @@ typedef enum {
 	OPT_ipsec_mb,
 	OPT_ipsec_mb_ops,
 	OPT_ipsec_mb_feature,
+	OPT_ipsec_mb_jobs,
 
 	OPT_itimer,
 	OPT_itimer_ops,
@@ -1806,17 +1425,23 @@ typedef enum {
 
 	OPT_link,
 	OPT_link_ops,
+	OPT_link_sync,
 
 	OPT_list,
 	OPT_list_ops,
 	OPT_list_method,
 	OPT_list_size,
 
+	OPT_llc_affinity,
+	OPT_llc_affinity_ops,
+
 	OPT_loadavg,
 	OPT_loadavg_ops,
+	OPT_loadavg_max,
 
 	OPT_lockbus,
 	OPT_lockbus_ops,
+	OPT_lockbus_nosplit,
 
 	OPT_locka,
 	OPT_locka_ops,
@@ -1830,6 +1455,7 @@ typedef enum {
 
 	OPT_log_brief,
 	OPT_log_file,
+	OPT_log_lockless,
 
 	OPT_longjmp,
 	OPT_longjmp_ops,
@@ -1853,6 +1479,7 @@ typedef enum {
 	OPT_malloc_pthreads,
 	OPT_malloc_threshold,
 	OPT_malloc_touch,
+	OPT_malloc_zerofree,
 
 	OPT_matrix,
 	OPT_matrix_ops,
@@ -1982,6 +1609,7 @@ typedef enum {
 
 	OPT_nanosleep,
 	OPT_nanosleep_ops,
+	OPT_nanosleep_threads,
 
 	OPT_netdev,
 	OPT_netdev_ops,
@@ -2010,6 +1638,7 @@ typedef enum {
 	OPT_numa_ops,
 
 	OPT_oomable,
+	OPT_oom_avoid,
 
 	OPT_oom_pipe,
 	OPT_oom_pipe_ops,
@@ -2020,10 +1649,15 @@ typedef enum {
 
 	OPT_open_ops,
 	OPT_open_fd,
+	OPT_open_max,
 
 	OPT_page_in,
 	OPT_pathological,
 
+	OPT_pagemove,
+	OPT_pagemove_ops,
+	OPT_pagemove_bytes,
+
 	OPT_pageswap,
 	OPT_pageswap_ops,
 
@@ -2058,6 +1692,11 @@ typedef enum {
 	OPT_pkey,
 	OPT_pkey_ops,
 
+	OPT_plugin,
+	OPT_plugin_ops,
+	OPT_plugin_method,
+	OPT_plugin_so,
+
 	OPT_poll_ops,
 	OPT_poll_fds,
 
@@ -2068,6 +1707,9 @@ typedef enum {
 	OPT_prctl,
 	OPT_prctl_ops,
 
+	OPT_priv_instr,
+	OPT_priv_instr_ops,
+
 	OPT_procfs,
 	OPT_procfs_ops,
 
@@ -2089,6 +1731,10 @@ typedef enum {
 	OPT_quota,
 	OPT_quota_ops,
 
+	OPT_race_sched,
+	OPT_race_sched_ops,
+	OPT_race_sched_method,
+
 	OPT_radixsort,
 	OPT_radixsort_ops,
 	OPT_radixsort_size,
@@ -2101,6 +1747,7 @@ typedef enum {
 
 	OPT_ramfs,
 	OPT_ramfs_ops,
+	OPT_ramfs_fill,
 	OPT_ramfs_size,
 
 	OPT_rawdev,
@@ -2113,6 +1760,7 @@ typedef enum {
 
 	OPT_rawsock,
 	OPT_rawsock_ops,
+	OPT_rawsock_port,
 
 	OPT_rawudp,
 	OPT_rawudp_ops,
@@ -2130,6 +1778,9 @@ typedef enum {
 	OPT_reboot,
 	OPT_reboot_ops,
 
+	OPT_regs,
+	OPT_regs_ops,
+
 	OPT_remap,
 	OPT_remap_ops,
 
@@ -2146,6 +1797,12 @@ typedef enum {
 	OPT_revio_opts,
 	OPT_revio_bytes,
 
+	OPT_ring_pipe,
+	OPT_ring_pipe_num,
+	OPT_ring_pipe_ops,
+	OPT_ring_pipe_size,
+	OPT_ring_pipe_splice,
+
 	OPT_rlimit,
 	OPT_rlimit_ops,
 
@@ -2212,7 +1869,7 @@ typedef enum {
 
 	OPT_shellsort,
 	OPT_shellsort_ops,
-	OPT_shellsort_integers,
+	OPT_shellsort_size,
 
 	OPT_shm,
 	OPT_shm_ops,
@@ -2295,6 +1952,7 @@ typedef enum {
 
 	OPT_sockabuse,
 	OPT_sockabuse_ops,
+	OPT_sockabuse_port,
 
 	OPT_sockdiag,
 	OPT_sockdiag_ops,
@@ -2304,8 +1962,9 @@ typedef enum {
 	OPT_sockfd_port,
 
 	OPT_sockmany,
-	OPT_sockmany_ops,
 	OPT_sockmany_if,
+	OPT_sockmany_ops,
+	OPT_sockmany_port,
 
 	OPT_sockpair,
 	OPT_sockpair_ops,
@@ -2337,6 +1996,8 @@ typedef enum {
 	OPT_stack_ops,
 	OPT_stack_fill,
 	OPT_stack_mlock,
+	OPT_stack_pageout,
+	OPT_stack_unmap,
 
 	OPT_stackmmap,
 	OPT_stackmmap_ops,
@@ -2357,6 +2018,7 @@ typedef enum {
 
 	OPT_symlink,
 	OPT_symlink_ops,
+	OPT_symlink_sync,
 
 	OPT_sync_file,
 	OPT_sync_file_ops,
@@ -2370,6 +2032,10 @@ typedef enum {
 	OPT_sysbadaddr,
 	OPT_sysbadaddr_ops,
 
+	OPT_syscall,
+	OPT_syscall_ops,
+	OPT_syscall_method,
+
 	OPT_sysinfo,
 	OPT_sysinfo_ops,
 
@@ -2474,9 +2140,17 @@ typedef enum {
 	OPT_vdso_ops,
 	OPT_vdso_func,
 
+	OPT_vecfp,
+	OPT_vecfp_ops,
+	OPT_vecfp_method,
+
 	OPT_vecmath,
 	OPT_vecmath_ops,
 
+	OPT_vecshuf,
+	OPT_vecshuf_ops,
+	OPT_vecshuf_method,
+
 	OPT_vecwide,
 	OPT_vecwide_ops,
 
@@ -2530,6 +2204,9 @@ typedef enum {
 	OPT_wcs_ops,
 	OPT_wcs_method,
 
+	OPT_x86cpuid,
+	OPT_x86cpuid_ops,
+
 	OPT_x86syscall,
 	OPT_x86syscall_ops,
 	OPT_x86syscall_func,
@@ -2559,7 +2236,7 @@ typedef enum {
 /* stress test metadata */
 typedef struct {
 	const stressor_info_t *info;	/* stress test info */
-	const stress_id_t id;		/* stress test ID */
+	const unsigned int id;		/* stress test ID */
 	const short int short_getopt;	/* getopt short option */
 	const stress_op_t op;		/* ops option */
 	const char *name;		/* name of stress test */
@@ -2570,7 +2247,6 @@ typedef struct stress_stressor_info {
 	struct stress_stressor_info *next;	/* next proc info struct in list */
 	struct stress_stressor_info *prev;	/* prev proc info struct in list */
 	const stress_t *stressor;	/* stressor */
-	pid_t	*pids;			/* stressor process id */
 	stress_stats_t **stats;		/* stressor stats info */
 	int32_t started_instances;	/* count of started instances */
 	int32_t num_instances;		/* number of instances per stressor */
@@ -2586,38 +2262,6 @@ typedef struct {
 	const uint64_t	scale;		/* Amount to scale by */
 } stress_scale_t;
 
-/* Cache types */
-typedef enum stress_cache_type {
-	CACHE_TYPE_UNKNOWN = 0,		/* Unknown type */
-	CACHE_TYPE_DATA,		/* D$ */
-	CACHE_TYPE_INSTRUCTION,		/* I$ */
-	CACHE_TYPE_UNIFIED,		/* D$ + I$ */
-} stress_cache_type_t;
-
-/* CPU cache information */
-typedef struct stress_cpu_cache {
-	uint64_t           size;      	/* cache size in bytes */
-	uint32_t           line_size;	/* cache line size in bytes */
-	uint32_t           ways;	/* cache ways */
-	stress_cache_type_t type;	/* cache type */
-	uint16_t           level;	/* cache level, L1, L2 etc */
-	uint8_t		   padding[2];	/* padding */
-} stress_cpu_cache_t;
-
-typedef struct stress_cpu {
-	stress_cpu_cache_t *caches;	/* CPU cache data */
-	uint32_t	num;		/* CPU # number */
-	uint32_t	cache_count;	/* CPU cache #  */
-	bool		online;		/* CPU online when true */
-	uint8_t		padding[7];	/* padding */
-} stress_cpu_t;
-
-typedef struct stress_cpus {
-	stress_cpu_t *cpus;		/* CPU data */
-	uint32_t	count;		/* CPU count */
-	uint8_t		padding[4];	/* padding */
-} stress_cpus_t;
-
 /* Various global option settings and flags */
 extern const char g_app_name[];		/* Name of application */
 extern stress_shared_t *g_shared;	/* shared memory */
@@ -2626,8 +2270,7 @@ extern uint64_t	g_opt_flags;		/* option flags */
 extern int32_t g_opt_sequential;	/* Number of sequential stressors */
 extern int32_t g_opt_parallel;		/* Number of parallel stressors */
 extern volatile bool g_keep_stressing_flag; /* false to exit stressor */
-extern volatile bool g_caught_sigint;	/* true if stopped by SIGINT */
-extern pid_t g_pgrp;			/* proceess group leader */
+extern volatile bool g_caught_signal;	/* true if stopped by SIGINT/SIGALRM */
 extern jmp_buf g_error_env;		/* parsing error env */
 
 /*
@@ -2718,7 +2361,13 @@ static inline bool inc_counter_lock(const stress_args_t *args, void *lock, const
 {
 	bool ret;
 
-	stress_lock_acquire(lock);
+	/*
+	 *  Failure in lock acquire, don't bump counter
+	 *  and get racy keep_stressing state, that's
+	 *  probably the best we can do in this failure mode
+	 */
+	if (stress_lock_acquire(lock) < 0)
+		return keep_stressing(args);
 	ret = keep_stressing(args);
 	if (inc && ret)
 		inc_counter(args);
@@ -2732,6 +2381,7 @@ static inline bool inc_counter_lock(const stress_args_t *args, void *lock, const
  */
 extern int stress_set_setting(const char *name, const stress_type_id_t type_id,
 	const void *value);
+extern int stress_set_setting_true(const char *name, const char *opt);
 extern int stress_set_setting_global(const char *name, const stress_type_id_t type_id,
 	const void *value);
 extern bool stress_get_setting(const char *name, void *value);
@@ -2778,6 +2428,7 @@ extern void stress_mwc_reseed(void);
 
 /* Time handling */
 extern WARN_UNUSED double stress_timeval_to_double(const struct timeval *tv);
+extern WARN_UNUSED double stress_timespec_to_double(const struct timespec *ts);
 extern WARN_UNUSED double stress_time_now(void);
 extern const char *stress_duration_to_str(const double duration);
 
@@ -2834,6 +2485,7 @@ extern void stress_check_range_bytes(const char *const opt,
 	const uint64_t val, const uint64_t lo, const uint64_t hi);
 extern WARN_UNUSED int stress_set_cpu_affinity(const char *arg);
 extern WARN_UNUSED int stress_set_mbind(const char *arg);
+extern int stress_numa_count_mem_nodes(unsigned long *max_node);
 extern WARN_UNUSED uint32_t stress_get_uint32(const char *const str);
 extern WARN_UNUSED int32_t  stress_get_int32(const char *const str);
 extern WARN_UNUSED int32_t  stress_get_opt_sched(const char *const str);
@@ -2850,6 +2502,7 @@ extern WARN_UNUSED size_t stress_min_pthread_stack_size(void);
 extern void stress_shared_unmap(void);
 extern void stress_log_system_mem_info(void);
 extern WARN_UNUSED char *stress_munge_underscore(const char *str);
+extern WARN_UNUSED int stress_strcmp_munged(const char *s1, const char *s2);
 extern size_t stress_get_page_size(void);
 extern WARN_UNUSED int32_t stress_get_processors_online(void);
 extern WARN_UNUSED int32_t stress_get_processors_configured(void);
@@ -2857,7 +2510,7 @@ extern WARN_UNUSED int32_t stress_get_ticks_per_second(void);
 extern WARN_UNUSED ssize_t stress_get_stack_direction(void);
 extern WARN_UNUSED void *stress_get_stack_top(void *start, size_t size);
 extern void stress_get_memlimits(size_t *shmall, size_t *freemem,
-	size_t *totalmem, size_t *freeswap);
+	size_t *totalmem, size_t *freeswap, size_t *totalswap);
 extern WARN_UNUSED int stress_get_load_avg(double *min1, double *min5,
 	double *min15);
 extern void stress_set_max_limits(void);
@@ -2872,6 +2525,7 @@ extern void stress_temp_path_free(void);
 extern void stress_strnrnd(char *str, const size_t len);
 extern void stress_uint8rnd4(uint8_t *data, const size_t len);
 extern void stress_get_cache_size(uint64_t *l2, uint64_t *l3);
+extern void stress_get_llc_size(size_t *llc_size, size_t *cache_line_size);
 extern WARN_UNUSED unsigned int stress_get_cpu(void);
 extern WARN_UNUSED const char *stress_get_compiler(void);
 extern WARN_UNUSED const char *stress_get_uname_info(void);
@@ -2903,7 +2557,7 @@ extern WARN_UNUSED int stress_sig_stop_stressing(const char *name,
 	const int sig);
 extern int stress_sigrestore(const char *name, const int signum,
 	struct sigaction *orig_action);
-extern WARN_UNUSED int stress_not_implemented(const stress_args_t *args);
+extern WARN_UNUSED int stress_unimplemented(const stress_args_t *args);
 extern WARN_UNUSED size_t stress_probe_max_pipe_size(void);
 extern WARN_UNUSED void *stress_align_address(const void *addr,
 	const size_t alignment);
@@ -2928,6 +2582,16 @@ extern WARN_UNUSED bool stress_check_capability(const int capability);
 extern WARN_UNUSED bool stress_sigalrm_pending(void);
 extern WARN_UNUSED bool stress_is_dev_tty(const int fd);
 extern WARN_UNUSED bool stress_little_endian(void);
+extern WARN_UNUSED char *stress_proc_self_exe_path(const char *proc_path);
+extern WARN_UNUSED char *stress_proc_self_exe(void);
+#if defined(__FreeBSD__) ||	\
+    defined(__NetBSD__)
+extern WARN_UNUSED int stress_bsd_getsysctl(const char *name, void *ptr, size_t size);
+extern WARN_UNUSED uint64_t stress_bsd_getsysctl_uint64(const char *name);
+extern WARN_UNUSED uint32_t stress_bsd_getsysctl_uint32(const char *name);
+extern WARN_UNUSED unsigned int stress_bsd_getsysctl_uint(const char *name);
+extern WARN_UNUSED int stress_bsd_getsysctl_int(const char *name);
+#endif
 
 extern WARN_UNUSED int stress_try_open(const stress_args_t *args,
 	const char *path, const int flags, const unsigned long timeout_ns);
@@ -2943,12 +2607,13 @@ extern WARN_UNUSED int32_t stress_set_vmstat(const char *const str);
 extern WARN_UNUSED int32_t stress_set_thermalstat(const char *const str);
 extern WARN_UNUSED int32_t stress_set_iostat(const char *const str);
 extern void stress_misc_stats_set(stress_misc_stats_t *misc_stats,
-	const int idx, const char *description, const double value);
+	const size_t idx, const char *description, const double value);
 extern WARN_UNUSED int stress_tty_width(void);
 extern WARN_UNUSED size_t stress_get_extents(const int fd);
 extern WARN_UNUSED bool stress_redo_fork(const int err);
 extern void stress_sighandler_nop(int sig);
 extern int stress_killpid(const pid_t pid);
+extern WARN_UNUSED bool stress_low_memory(const size_t requested);
 
 /* kernel module helpers */
 extern int stress_module_load(const char *name, const char *alias,
@@ -2956,22 +2621,7 @@ extern int stress_module_load(const char *name, const char *alias,
 extern int stress_module_unload(const char *name, const char *alias,
 	const bool already_loaded);
 
-/*
- *  Indicate a stress test failed because of limited resources
- *  rather than a failure of the tests during execution.
- *  err is the errno of the failure.
- */
-static inline WARN_UNUSED ALWAYS_INLINE int exit_status(const int err)
-{
-	switch (err) {
-	case ENOMEM:
-	case ENOSPC:
-		return EXIT_NO_RESOURCE;
-	case ENOSYS:
-		return EXIT_NOT_IMPLEMENTED;
-	}
-	return EXIT_FAILURE;	/* cppcheck-suppress ConfigurationNotChecked */
-}
+extern WARN_UNUSED int stress_exit_status(const int err);
 
 /*
  *  Stack aligning for clone() system calls
@@ -3009,13 +2659,6 @@ extern int stress_pagein_self(const char *name);
 extern void stress_mount_free(char *mnts[], const int n);
 extern WARN_UNUSED int stress_mount_get(char *mnts[], const int max);
 
-/* CPU caches */
-extern stress_cpus_t *stress_get_all_cpu_cache_details(void);
-extern uint16_t stress_get_max_cache_level(const stress_cpus_t *cpus);
-extern stress_cpu_cache_t *stress_get_cpu_cache(const stress_cpus_t *cpus,
-	const uint16_t cache_level);
-extern void stress_free_cpu_caches(stress_cpus_t *cpus);
-
 /* Used to set options for specific stressors */
 extern void stress_adjust_pthread_max(const uint64_t max);
 extern void stress_adjust_sleep_max(const uint64_t max);
@@ -3100,25 +2743,6 @@ struct shim_sched_attr {
 	uint32_t sched_util_max;	/* utilization hint, max */
 };
 
-#if defined(HAVE_TERMIOS_H)
-
-#define HAVE_SHIM_TERMIOS2
-/* shim_speed_t */
-typedef unsigned int shim_speed_t;
-
-/* shim termios2 */
-struct shim_termios2 {
-	tcflag_t c_iflag;		/* input mode flags */
-	tcflag_t c_oflag;		/* output mode flags */
-	tcflag_t c_cflag;		/* control mode flags */
-	tcflag_t c_lflag;		/* local mode flags */
-	cc_t c_line;			/* line discipline */
-	cc_t c_cc[NCCS];		/* control characters */
-	shim_speed_t c_ispeed;		/* input speed */
-	shim_speed_t c_ospeed;		/* output speed */
-};
-#endif
-
 /* shim'd STATX flags */
 #define SHIM_STATX_TYPE			(0x00000001U)
 #define SHIM_STATX_MODE			(0x00000002U)
@@ -3160,6 +2784,18 @@ struct shim_ustat {
 #define P_PIDFD		(3)
 #endif
 
+/* shim'd timex struct */
+#if defined(HAVE_SYS_TIMEX_H) &&	\
+    defined(HAVE_TIMEX)
+typedef struct timex	shim_timex_t;
+#else
+typedef struct shim_timex {
+	int modes;
+	/* other fields we don't care about */
+	uint8_t padding[256 - sizeof(int)];
+} shim_timex_t;
+#endif
+
 /*
  *  shim_unconstify_ptr()
  *      some older system calls require non-const void *
@@ -3181,10 +2817,11 @@ extern int shim_arch_prctl(int code, unsigned long addr);
 extern int shim_brk(void *addr);
 extern int shim_cacheflush(char *addr, int nbytes, int cache);
 extern void shim_flush_icache(void *begin, void *end);
+extern int shim_clock_adjtime(clockid_t clk_id, shim_timex_t *buf);
 extern int shim_clock_getres(clockid_t clk_id, struct timespec *res);
 extern int shim_clock_gettime(clockid_t clk_id, struct timespec *tp);
 extern int shim_clock_settime(clockid_t clk_id, struct timespec *tp);
-extern int sys_clone3(struct shim_clone_args *cl_args, size_t size);
+extern int shim_clone3(struct shim_clone_args *cl_args, size_t size);
 extern int shim_close_range(unsigned int fd, unsigned int max_fd, unsigned int flags);
 extern ssize_t shim_copy_file_range(int fd_in, shim_loff_t *off_in,
 	int fd_out, shim_loff_t *off_out, size_t len, unsigned int flags);
@@ -3211,6 +2848,7 @@ extern int shim_getdents(unsigned int fd, struct shim_linux_dirent *dirp,
 	unsigned int count);
 extern int shim_getdents64(unsigned int fd, struct shim_linux_dirent64 *dirp,
 	unsigned int count);
+extern int shim_getdomainname(char *name, size_t len);
 extern char *shim_getlogin(void);
 extern int shim_get_mempolicy(int *mode, unsigned long *nodemask,
 	unsigned long maxnode, void *addr, unsigned long flags);
@@ -3253,7 +2891,7 @@ extern int shim_msync(void *addr, size_t length, int flags);
 extern int shim_munlock(const void *addr, size_t len);
 extern int shim_munlockall(void);
 extern int shim_modify_ldt(int func, void *ptr, unsigned long bytecount);
-extern int shim_nanosleep_uint64(uint64_t usec);
+extern int shim_nanosleep_uint64(uint64_t nsec);
 extern int shim_nice(int inc);
 extern time_t shim_time(time_t *tloc);
 extern int shim_gettimeofday(struct timeval *tv, struct timezone *tz);
@@ -3280,17 +2918,19 @@ extern int shim_sched_getattr(pid_t pid, struct shim_sched_attr *attr,
 	unsigned int size, unsigned int flags);
 extern int shim_sched_setattr(pid_t pid, struct shim_sched_attr *attr,
 	unsigned int flags);
+extern int shim_setdomainname(const char *name, size_t len);
 extern long shim_sgetmask(void);
 extern long shim_ssetmask(long newmask);
 extern int shim_stime(const time_t *t);
 extern int shim_sched_yield(void);
+extern int shim_seccomp(unsigned int operation, unsigned int flags, void *args);
 extern int shim_set_mempolicy(int mode, unsigned long *nodemask,
 	unsigned long maxnode);
-extern int shim_seccomp(unsigned int operation, unsigned int flags, void *args);
-extern int shim_statx(int dfd, const char *filename, int flags,
-	unsigned int mask, shim_statx_t *buffer);
+extern int shim_setgroups(int size, const gid_t *list);
 extern int shim_setxattr(const char *path, const char *name, const void *value,
 	size_t size, int flags);
+extern int shim_statx(int dfd, const char *filename, int flags,
+	unsigned int mask, shim_statx_t *buffer);
 extern size_t shim_strlcat(char *dst, const char *src, size_t len);
 extern size_t shim_strlcpy(char *dst, const char *src, size_t len);
 extern int shim_sync_file_range(int fd, shim_off64_t offset,
