@@ -37,8 +37,8 @@
 
 static const stress_help_t help[] = {
 	{ NULL,	"opcode N",	   "start N workers exercising random opcodes" },
-	{ NULL,	"opcode-ops N",	   "stop after N opcode bogo operations" },
 	{ NULL,	"opcode-method M", "set opcode stress method (M = random, inc, mixed, text)" },
+	{ NULL,	"opcode-ops N",	   "stop after N opcode bogo operations" },
 	{ NULL, NULL,		   NULL }
 };
 
@@ -51,8 +51,8 @@ static const stress_help_t help[] = {
 #define SYSCALL_NR	(offsetof(struct seccomp_data, nr))
 
 #define ALLOW_SYSCALL(syscall)					\
-	BPF_JUMP(BPF_JMP+BPF_JEQ+BPF_K, __NR_##syscall, 0, 1), 	\
-	BPF_STMT(BPF_RET+BPF_K, SECCOMP_RET_ALLOW)
+	BPF_JUMP(BPF_JMP + BPF_JEQ + BPF_K, __NR_##syscall, 0, 1), 	\
+	BPF_STMT(BPF_RET + BPF_K, SECCOMP_RET_ALLOW)
 
 #define PAGES		(16)
 #define TRACK_SIGCOUNT	(0)
@@ -103,11 +103,11 @@ static const int sigs[] = {
 #if defined(HAVE_LINUX_SECCOMP_H) &&	\
     defined(SECCOMP_SET_MODE_FILTER)
 static struct sock_filter filter[] = {
-	BPF_STMT(BPF_LD+BPF_W+BPF_ABS, SYSCALL_NR),
+	BPF_STMT(BPF_LD + BPF_W + BPF_ABS, SYSCALL_NR),
 #if defined(__NR_exit_group)
 	ALLOW_SYSCALL(exit_group),
 #endif
-	BPF_STMT(BPF_RET+BPF_K, SECCOMP_RET_TRAP)
+	BPF_STMT(BPF_RET + BPF_K, SECCOMP_RET_TRAP)
 };
 
 static struct sock_fprog prog = {
@@ -281,8 +281,9 @@ static int stress_opcode(const stress_args_t *args)
 	sig_count = (uint64_t *)mmap(NULL, sig_count_size, PROT_READ | PROT_WRITE,
 		MAP_ANONYMOUS | MAP_SHARED, -1, 0);
 	if (sig_count == MAP_FAILED) {
-		pr_fail("%s: mmap failed, errno=%d (%s)\n",
-			args->name, errno, strerror(errno));
+		pr_inf_skip("%s: mmap of %zu bytes failed, errno=%d (%s) "
+			"skipping stressor\n",
+			args->name, sig_count_size, errno, strerror(errno));
 		return EXIT_NO_RESOURCE;
 	}
 #endif
@@ -477,9 +478,10 @@ static const stress_opt_set_func_t opt_set_funcs[] = {
 };
 
 stressor_info_t stress_opcode_info = {
-	.stressor = stress_not_implemented,
+	.stressor = stress_unimplemented,
 	.class = CLASS_CPU | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
-	.help = help
+	.help = help,
+	.unimplemented_reason = "built without linux/seccomp.h, linux/audit.h, linux/filter.h, sys/prctl.h or mprotect()"
 };
 #endif
