@@ -29,8 +29,8 @@
 
 static const stress_help_t help[] = {
 	{ NULL,	"pty N",	"start N workers that exercise pseudoterminals" },
-	{ NULL,	"pty-ops N",	"stop pty workers after N pty bogo operations" },
 	{ NULL,	"pty-max N",	"attempt to open a maximum of N ptys" },
+	{ NULL,	"pty-ops N",	"stop pty workers after N pty bogo operations" },
 	{ NULL,	NULL,          NULL }
 };
 
@@ -83,7 +83,8 @@ static int stress_pty(const stress_args_t *args)
 
 	ptys = calloc(pty_max, sizeof(*ptys));
 	if (!ptys) {
-		pr_inf("%s: allocation of pty array failed: %d (%s)\n",
+		pr_inf_skip("%s: allocation of pty array failed: %d (%s) "
+			"skipping stressor\n",
 			args->name, errno, strerror(errno));
 		return EXIT_NO_RESOURCE;
 	}
@@ -416,9 +417,10 @@ stressor_info_t stress_pty_info = {
 };
 #else
 stressor_info_t stress_pty_info = {
-	.stressor = stress_not_implemented,
+	.stressor = stress_unimplemented,
 	.class = CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
-	.help = help
+	.help = help,
+	.unimplemented_reason = "built without termios.h, termio.h or ptsname()"
 };
 #endif
