@@ -1,5 +1,6 @@
 /*
- * Copyright (C) 2013-2017 Canonical, Ltd.
+ * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,25 +16,31 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
+#include "core-sort.h"
+
+#define MIN_QSORT_SIZE		(1 * KB)
+#define MAX_QSORT_SIZE		(4 * MB)
+#define DEFAULT_QSORT_SIZE	(256 * KB)
 
 static volatile bool do_jmp = true;
 static sigjmp_buf jmp_env;
 
+static const stress_help_t help[] = {
+	{ "Q N", "qsort N",	"start N workers qsorting 32 bit random integers" },
+	{ NULL,	"qsort-ops N",	"stop after N qsort bogo operations" },
+	{ NULL,	"qsort-size N",	"number of 32 bit integers to sort" },
+	{ NULL,	NULL,		NULL }
+};
+
 /*
  *  stress_qsort_handler()
  *	SIGALRM generic handler
  */
-static void MLOCKED stress_qsort_handler(int dummy)
+static void MLOCKED_TEXT stress_qsort_handler(int signum)
 {
-	(void)dummy;
+	(void)signum;
 
 	if (do_jmp) {
 		do_jmp = false;
@@ -45,60 +52,21 @@ static void MLOCKED stress_qsort_handler(int dummy)
  *  stress_set_qsort_size()
  *	set qsort size
  */
-void stress_set_qsort_size(const char *opt)
+static int stress_set_qsort_size(const char *opt)
 {
 	uint64_t qsort_size;
 
-	qsort_size = get_uint64(opt);
-	check_range("qsort-size", qsort_size,
+	qsort_size = stress_get_uint64(opt);
+	stress_check_range("qsort-size", qsort_size,
 		MIN_QSORT_SIZE, MAX_QSORT_SIZE);
-	set_setting("qsort-size", TYPE_ID_UINT64, &qsort_size);
-}
-
-/*
- *  stress_qsort_cmp_1()
- *	qsort comparison - sort on int32 values
- */
-static int stress_qsort_cmp_1(const void *p1, const void *p2)
-{
-	const int32_t *i1 = (const int32_t *)p1;
-	const int32_t *i2 = (const int32_t *)p2;
-
-	if (*i1 > *i2)
-		return 1;
-	else if (*i1 < *i2)
-		return -1;
-	else
-		return 0;
-}
-
-/*
- *  stress_qsort_cmp_1()
- *	qsort comparison - reverse sort on int32 values
- */
-static int stress_qsort_cmp_2(const void *p1, const void *p2)
-{
-	return stress_qsort_cmp_1(p2, p1);
-}
-
-/*
- *  stress_qsort_cmp_1()
- *	qsort comparison - sort on int8 values
- */
-static int stress_qsort_cmp_3(const void *p1, const void *p2)
-{
-	const int8_t *i1 = (const int8_t *)p1;
-	const int8_t *i2 = (const int8_t *)p2;
-
-	/* Force re-ordering on 8 bit value */
-	return *i1 - *i2;
+	return stress_set_setting("qsort-size", TYPE_ID_UINT64, &qsort_size);
 }
 
 /*
  *  stress_qsort()
  *	stress qsort
  */
-int stress_qsort(const args_t *args)
+static int stress_qsort(const stress_args_t *args)
 {
 	uint64_t qsort_size = DEFAULT_QSORT_SIZE;
 	int32_t *data, *ptr;
@@ -106,7 +74,7 @@ int stress_qsort(const args_t *args)
 	struct sigaction old_action;
 	int ret;
 
-	if (!get_setting("qsort-size", &qsort_size)) {
+	if (!stress_get_setting("qsort-size", &qsort_size)) {
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
 			qsort_size = MAX_QSORT_SIZE;
 		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
@@ -114,8 +82,9 @@ int stress_qsort(const args_t *args)
 	}
 	n = (size_t)qsort_size;
 
-	if ((data = calloc(n, sizeof(int32_t))) == NULL) {
-		pr_fail_dbg("calloc");
+	if ((data = calloc(n, sizeof(*data))) == NULL) {
+		pr_inf_skip("%s: calloc failed allocating %zd integers, "
+			"skipping stressor\n", args->name, n);
 		return EXIT_NO_RESOURCE;
 	}
 
@@ -135,14 +104,16 @@ int stress_qsort(const args_t *args)
 
 	/* This is expensive, do it once */
 	for (ptr = data, i = 0; i < n; i++)
-		*ptr++ = mwc32();
+		*ptr++ = (int32_t)stress_mwc32();
+
+	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
 		/* Sort "random" data */
-		qsort(data, n, sizeof(uint32_t), stress_qsort_cmp_1);
+		qsort(data, n, sizeof(*data), stress_sort_cmp_int32);
 		if (g_opt_flags & OPT_FLAGS_VERIFY) {
 			for (ptr = data, i = 0; i < n - 1; i++, ptr++) {
-				if (*ptr > *(ptr+1)) {
+				if (*ptr > *(ptr + 1)) {
 					pr_fail("%s: sort error "
 						"detected, incorrect ordering "
 						"found\n", args->name);
@@ -150,14 +121,14 @@ int stress_qsort(const args_t *args)
 				}
 			}
 		}
-		if (!g_keep_stressing_flag)
+		if (!keep_stressing_flag())
 			break;
 
 		/* Reverse sort */
-		qsort(data, n, sizeof(uint32_t), stress_qsort_cmp_2);
+		qsort(data, n, sizeof(*data), stress_sort_cmp_rev_int32);
 		if (g_opt_flags & OPT_FLAGS_VERIFY) {
 			for (ptr = data, i = 0; i < n - 1; i++, ptr++) {
-				if (*ptr < *(ptr+1)) {
+				if (*ptr < *(ptr + 1)) {
 					pr_fail("%s: reverse sort "
 						"error detected, incorrect "
 						"ordering found\n", args->name);
@@ -165,16 +136,16 @@ int stress_qsort(const args_t *args)
 				}
 			}
 		}
-		if (!g_keep_stressing_flag)
+		if (!keep_stressing_flag())
 			break;
 		/* And re-order by byte compare */
-		qsort((uint8_t *)data, n * 4, sizeof(uint8_t), stress_qsort_cmp_3);
+		qsort((uint8_t *)data, n * 4, sizeof(uint8_t), stress_sort_cmp_int8);
 
 		/* Reverse sort this again */
-		qsort(data, n, sizeof(uint32_t), stress_qsort_cmp_2);
+		qsort(data, n, sizeof(*data), stress_sort_cmp_rev_int32);
 		if (g_opt_flags & OPT_FLAGS_VERIFY) {
 			for (ptr = data, i = 0; i < n - 1; i++, ptr++) {
-				if (*ptr < *(ptr+1)) {
+				if (*ptr < *(ptr + 1)) {
 					pr_fail("%s: reverse sort "
 						"error detected, incorrect "
 						"ordering found\n", args->name);
@@ -182,16 +153,31 @@ int stress_qsort(const args_t *args)
 				}
 			}
 		}
-		if (!g_keep_stressing_flag)
+		if (!keep_stressing_flag())
 			break;
 
 		inc_counter(args);
-	} while (keep_stressing());
+	} while (keep_stressing(args));
 
 	do_jmp = false;
 	(void)stress_sigrestore(args->name, SIGALRM, &old_action);
 tidy:
+	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
+
 	free(data);
 
 	return EXIT_SUCCESS;
 }
+
+static const stress_opt_set_func_t opt_set_funcs[] = {
+	{ OPT_qsort_integers,	stress_set_qsort_size },
+	{ 0,			NULL }
+};
+
+stressor_info_t stress_qsort_info = {
+	.stressor = stress_qsort,
+	.class = CLASS_CPU_CACHE | CLASS_CPU | CLASS_MEMORY,
+	.opt_set_funcs = opt_set_funcs,
+	.verify = VERIFY_OPTIONAL,
+	.help = help
+};
