@@ -23,21 +23,21 @@
 #define STRESS_RANDLIST_MAX_SIZE	(8192)
 #define STRESS_RANDLIST_DEFAULT_SIZE	(64)
 
-#define STRESS_RANDLIST_ALLOC_HEAP	(1)
-#define STRESS_RANDLIST_ALLOC_MMAP	(2)
+#define STRESS_RANDLIST_ALLOC_HEAP	(0)
+#define STRESS_RANDLIST_ALLOC_MMAP	(1)
 
 static const stress_help_t help[] = {
 	{ NULL,	"randlist N",		"start N workers that exercise random ordered list" },
-	{ NULL,	"randlist-ops N",	"stop after N randlist bogo no-op operations" },
 	{ NULL, "randlist-compact",	"reduce mmap and malloc overheads" },
-	{ NULL, "randlist-items",	"number of items in the random ordered list" },
-	{ NULL, "randlist-size",	"size of data in each item in the list" },
+	{ NULL, "randlist-items N",	"number of items in the random ordered list" },
+	{ NULL,	"randlist-ops N",	"stop after N randlist bogo no-op operations" },
+	{ NULL, "randlist-size N",	"size of data in each item in the list" },
 	{ NULL,	NULL,			NULL }
 };
 
 typedef struct stress_randlist_item {
 	struct stress_randlist_item *next;
-	uint8_t alloc_type:2;
+	uint8_t alloc_type:1;
 	uint8_t data[];
 } stress_randlist_item_t;
 
@@ -117,7 +117,7 @@ static void stress_randlist_free_ptrs(
 
 static void stress_randlist_enomem(const stress_args_t *args)
 {
-	pr_inf("%s: cannot allocate the list, skipping stressor\n",
+	pr_inf_skip("%s: cannot allocate the list, skipping stressor\n",
 		args->name);
 }
 
@@ -147,7 +147,7 @@ static int stress_randlist(const stress_args_t *args)
 
 	ptrs = calloc(randlist_items, sizeof(stress_randlist_item_t *));
 	if (!ptrs) {
-		pr_inf("%s: cannot allocate %zd temporary pointers, skipping stressor\n",
+		pr_inf_skip("%s: cannot allocate %zd temporary pointers, skipping stressor\n",
 			args->name, randlist_items);
 		return EXIT_NO_RESOURCE;
 	}
@@ -229,7 +229,7 @@ retry:
 	 */
 	for (i = 0; i < randlist_items; i++) {
 		ptr = ptrs[i];
-		ptr->next = (i == randlist_items - 1) ? NULL : ptrs[i+1];
+		ptr->next = (i == randlist_items - 1) ? NULL : ptrs[i + 1];
 	}
 
 	head = ptrs[0];
