@@ -43,9 +43,7 @@ static int stress_set_rdrand_seed(const char *opt)
     defined(HAVE_ASM_X86_RDRAND) &&	\
     defined(HAVE_ASM_X86_RDSEED)
 	if (stress_cpu_x86_has_rdseed()) {
-		bool rdrand_seed = true;
-
-		return stress_set_setting("rdrand-seed", TYPE_ID_BOOL, &rdrand_seed);
+		return stress_set_setting_true("rdrand-seed", opt);
 	}
 #endif
 	pr_inf("rdrand-seed ignored, cpu does not support feature, defaulting to rdrand\n");
@@ -89,7 +87,7 @@ static int stress_rdrand_supported(const char *name)
 	return 0;
 }
 
-#if defined(__x86_64__) || defined(__x86_64)
+#if defined(STRESS_ARCH_X86_64)
 /*
  *  rdrand64()
  *	read 64 bit random value
@@ -336,8 +334,7 @@ static int stress_rdrand(const stress_args_t *args)
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	if (rdrand_supported) {
-		double time_start, duration, billion_bits;
-		bool lock = false;
+		double time_start, duration, million_bits, rate;
 		register int i;
 		uint64_t c;
 
@@ -348,7 +345,7 @@ static int stress_rdrand(const stress_args_t *args)
 #if defined(HAVE_SEED_CAPABILITY)
 		if (rdrand_seed) {
 			do {
-				for (i = 0; i < 4096; i++) {
+				for (i = 0; i < 64; i++) {
 					SEED64x32()
 					SEED64x32()
 					SEED64x32()
@@ -364,7 +361,7 @@ static int stress_rdrand(const stress_args_t *args)
 #endif
 		{
 			do {
-				for (i = 0; i < 4096; i++) {
+				for (i = 0; i < 64; i++) {
 					RAND64x32()
 					RAND64x32()
 					RAND64x32()
@@ -380,22 +377,10 @@ static int stress_rdrand(const stress_args_t *args)
 
 		duration = stress_time_now() - time_start;
 		c = get_counter(args);
-		billion_bits = ((double)c * 64.0 * 256.0) * ONE_BILLIONTH;
-
-		pr_lock(&lock);
-		pr_dbg_lock(&lock, "%s: %.3f billion random bits read "
-			"(instance %" PRIu32")\n",
-			args->name, billion_bits, args->instance);
-		if (duration > 0.0) {
-			pr_dbg_lock(&lock, "%s: %.3f billion random bits per "
-				"second (instance %" PRIu32")\n",
-				args->name,
-				(double)billion_bits / duration,
-				args->instance);
-		}
-		pr_unlock(&lock);
-		stress_misc_stats_set(args->misc_stats, 0, "billion random bits read", billion_bits);
-		stress_misc_stats_set(args->misc_stats, 1, "billion random bits / sec", billion_bits / duration);
+		million_bits = ((double)c * 64.0 * 256.0) * ONE_MILLIONTH;
+		rate = (duration > 0.0) ? million_bits / duration : 0.0;
+		stress_misc_stats_set(args->misc_stats, 0, "million random bits read", million_bits);
+		stress_misc_stats_set(args->misc_stats, 1, "million random bits per sec", rate);
 	}
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
@@ -419,10 +404,11 @@ static int stress_rdrand_supported(const char *name)
 }
 
 stressor_info_t stress_rdrand_info = {
-	.stressor = stress_not_implemented,
+	.stressor = stress_unimplemented,
 	.supported = stress_rdrand_supported,
 	.opt_set_funcs = opt_set_funcs,
 	.class = CLASS_CPU,
-	.help = help
+	.help = help,
+	.unimplemented_reason = "x86 CPU only, built without rdrand or rdseed opcode support"
 };
 #endif
