@@ -51,9 +51,9 @@ typedef struct {
 
 static const stress_help_t help[] = {
 	{ NULL,	"sctp N",	 "start N workers performing SCTP send/receives " },
-	{ NULL,	"sctp-ops N",	 "stop after N SCTP bogo operations" },
-	{ NULL,	"sctp-if I",	 "use network interface I, e.g. lo, eth0, etc." },
 	{ NULL,	"sctp-domain D", "specify sctp domain, default is ipv4" },
+	{ NULL,	"sctp-if I",	 "use network interface I, e.g. lo, eth0, etc." },
+	{ NULL,	"sctp-ops N",	 "stop after N SCTP bogo operations" },
 	{ NULL,	"sctp-port P",	 "use SCTP ports P to P + number of workers - 1" },
 	{ NULL, "sctp-sched S",	 "specify sctp scheduler" },
 	{ NULL,	NULL, 		 NULL }
@@ -380,9 +380,9 @@ static void stress_sctp_sockopts(const int fd)
  *  stress_sctp_client()
  *	client reader
  */
-static void stress_sctp_client(
+static int stress_sctp_client(
 	const stress_args_t *args,
-	const pid_t ppid,
+	const pid_t mypid,
 	const int sctp_port,
 	const int sctp_domain,
 	const int sctp_sched,
@@ -392,11 +392,9 @@ static void stress_sctp_client(
 
 	(void)sctp_sched;
 
-	(void)setpgid(0, g_pgrp);
 	stress_parent_died_alarm();
 	(void)sched_settings_apply(true);
 
-
 	do {
 		char ALIGN64 buf[SOCKET_BUF];
 		int fd;
@@ -406,28 +404,26 @@ static void stress_sctp_client(
 		struct sctp_event_subscribe events;
 #endif
 retry:
-		if (!keep_stressing_flag()) {
-			(void)kill(getppid(), SIGALRM);
-			_exit(EXIT_FAILURE);
-		}
+		if (!keep_stressing_flag())
+			return EXIT_FAILURE;
 		if ((fd = socket(sctp_domain, SOCK_STREAM, IPPROTO_SCTP)) < 0) {
 			if (errno == EPROTONOSUPPORT) {
 				if (args->instance == 0)
 					pr_inf_skip("%s: SCTP protocol not supported, skipping stressor\n",
 						args->name);
-				(void)kill(getppid(), SIGALRM);
-				_exit(EXIT_NOT_IMPLEMENTED);
+				return EXIT_FAILURE;
 			}
 			pr_fail("%s: socket failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
-			/* failed, kick parent to finish */
-			(void)kill(getppid(), SIGALRM);
-			_exit(EXIT_FAILURE);
+			return EXIT_FAILURE;
 		}
 
-		stress_set_sockaddr_if(args->name, args->instance, ppid,
-			sctp_domain, sctp_port, sctp_if,
-			&addr, &addr_len, NET_ADDR_LOOPBACK);
+		if (stress_set_sockaddr_if(args->name, args->instance, mypid,
+				sctp_domain, sctp_port, sctp_if,
+				&addr, &addr_len, NET_ADDR_LOOPBACK) < 0) {
+			(void)close(fd);
+			return EXIT_FAILURE;
+		}
 		if (connect(fd, addr, addr_len) < 0) {
 			(void)close(fd);
 			(void)shim_usleep(10000);
@@ -436,8 +432,7 @@ retry:
 				/* Give up.. */
 				pr_fail("%s: connect failed, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
-				(void)kill(getppid(), SIGALRM);
-				_exit(EXIT_FAILURE);
+				return EXIT_FAILURE;
 			}
 			goto retry;
 		}
@@ -449,8 +444,7 @@ retry:
 			(void)close(fd);
 			pr_fail("%s: setsockopt failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
-			(void)kill(getppid(), SIGALRM);
-			_exit(EXIT_FAILURE);
+			return EXIT_FAILURE;
 		}
 #endif
 #if defined(HAVE_SCTP_SCHED_TYPE) &&	\
@@ -459,7 +453,7 @@ retry:
 			struct sctp_assoc_value val;
 
 			(void)memset(&val, 0, sizeof(val));
-			val.assoc_value = sctp_sched;
+			val.assoc_value = (uint32_t)sctp_sched;
 			(void)setsockopt(fd, SOL_SCTP, SCTP_STREAM_SCHEDULER, &val, sizeof(val));
 		}
 #endif
@@ -488,8 +482,7 @@ retry:
 #else
 	UNEXPECTED
 #endif
-	/* Inform parent we're all done */
-	(void)kill(getppid(), SIGALRM);
+	return EXIT_SUCCESS;
 }
 
 /*
@@ -499,7 +492,7 @@ retry:
 static int stress_sctp_server(
 	const stress_args_t *args,
 	const pid_t pid,
-	const pid_t ppid,
+	const pid_t mypid,
 	const int sctp_port,
 	const int sctp_domain,
 	const int sctp_sched,
@@ -510,13 +503,10 @@ static int stress_sctp_server(
 	int so_reuseaddr = 1;
 	socklen_t addr_len = 0;
 	struct sockaddr *addr = NULL;
-	uint64_t msgs = 0;
 	int rc = EXIT_SUCCESS;
 
 	(void)sctp_sched;
 
-	(void)setpgid(pid, g_pgrp);
-
 	if (stress_sig_stop_stressing(args->name, SIGALRM)) {
 		rc = EXIT_FAILURE;
 		goto die;
@@ -529,7 +519,7 @@ static int stress_sctp_server(
 			rc = EXIT_NOT_IMPLEMENTED;
 			goto die;
 		}
-		rc = exit_status(errno);
+		rc = stress_exit_status(errno);
 		pr_fail("%s: socket failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		goto die;
@@ -541,11 +531,13 @@ static int stress_sctp_server(
 		rc = EXIT_FAILURE;
 		goto die_close;
 	}
-
-	stress_set_sockaddr_if(args->name, args->instance, ppid,
-		sctp_domain, sctp_port, sctp_if, &addr, &addr_len, NET_ADDR_ANY);
+	if (stress_set_sockaddr_if(args->name, args->instance, mypid,
+		sctp_domain, sctp_port, sctp_if, &addr, &addr_len, NET_ADDR_ANY) < 0) {
+		rc = EXIT_FAILURE;
+		goto die_close;
+	}
 	if (bind(fd, addr, addr_len) < 0) {
-		rc = exit_status(errno);
+		rc = stress_exit_status(errno);
 		pr_fail("%s: bind failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		goto die_close;
@@ -579,7 +571,7 @@ static int stress_sctp_server(
 		struct sctp_assoc_value val;
 
 		(void)memset(&val, 0, sizeof(val));
-		val.assoc_value = sctp_sched;
+		val.assoc_value = (uint32_t)sctp_sched;
 		(void)setsockopt(fd, SOL_SCTP, SCTP_STREAM_SCHEDULER, &val, sizeof(val));
 	}
 #endif
@@ -600,11 +592,10 @@ static int stress_sctp_server(
 				ssize_t ret = sctp_sendmsg(sfd, buf, i,
 						NULL, 0, 0, 0,
 						LOCALTIME_STREAM, 0, 0);
-				if (ret < 0)
+				if (ret < 0) {
 					break;
-				else {
+				} else {
 					inc_counter(args);
-					msgs++;
 				}
 			}
 			stress_sctp_sockopts(sfd);
@@ -646,11 +637,11 @@ static void stress_sctp_sigpipe(int signum)
  */
 static int stress_sctp(const stress_args_t *args)
 {
-	pid_t pid, ppid = getppid();
+	pid_t pid, mypid = getpid();
 	int sctp_port = DEFAULT_SCTP_PORT;
 	int sctp_domain = AF_INET;
 	int sctp_sched = -1;	/* Undefined */
-	int ret;
+	int ret, reserved_port;
 	char *sctp_if = NULL;
 
 	(void)stress_get_setting("sctp-domain", &sctp_domain);
@@ -672,9 +663,17 @@ static int stress_sctp(const stress_args_t *args)
 	if (stress_sighandler(args->name, SIGPIPE, stress_sctp_sigpipe, NULL) < 0)
 		return EXIT_FAILURE;
 
+	sctp_port += args->instance;
+	reserved_port = stress_net_reserve_ports(sctp_port, sctp_port);
+	if (reserved_port < 0) {
+		pr_inf_skip("%s: cannot reserve port %d, skipping stressor\n",
+			args->name, sctp_port);
+		return EXIT_NO_RESOURCE;
+	}
+	sctp_port = reserved_port;
+
 	pr_dbg("%s: process [%" PRIdMAX "] using socket port %d\n",
-		args->name, (intmax_t)args->pid,
-		sctp_port + (int)args->instance);
+		args->name, (intmax_t)args->pid, sctp_port);
 
 	ret = EXIT_FAILURE;
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
@@ -689,10 +688,12 @@ again:
 			args->name, errno, strerror(errno));
 		return EXIT_FAILURE;
 	} else if (pid == 0) {
-		stress_sctp_client(args, ppid, sctp_port, sctp_domain, sctp_sched, sctp_if);
-		_exit(EXIT_SUCCESS);
+		ret = stress_sctp_client(args, mypid, sctp_port, sctp_domain, sctp_sched, sctp_if);
+		/* Inform parent we're all done */
+		(void)kill(getppid(), SIGALRM);
+		_exit(ret);
 	} else {
-		ret = stress_sctp_server(args, pid, ppid, sctp_port, sctp_domain, sctp_sched, sctp_if);
+		ret = stress_sctp_server(args, pid, mypid, sctp_port, sctp_domain, sctp_sched, sctp_if);
 	}
 
 finish:
@@ -700,6 +701,7 @@ finish:
 		pr_dbg("%s: caught %" PRIu64 " SIGPIPE signals\n", args->name, sigpipe_count);
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
+	stress_net_release_ports(sctp_port, sctp_port);
 
 	return ret;
 }
@@ -712,9 +714,10 @@ stressor_info_t stress_sctp_info = {
 };
 #else
 stressor_info_t stress_sctp_info = {
-	.stressor = stress_not_implemented,
+	.stressor = stress_unimplemented,
 	.class = CLASS_NETWORK,
 	.opt_set_funcs = opt_set_funcs,
-	.help = help
+	.help = help,
+	.unimplemented_reason = "built without netinet/sctp.h or libsctp support"
 };
 #endif
