@@ -65,6 +65,7 @@ static int stress_sendfile(const stress_args_t *args)
 	int i = 0, fdin, fdout, ret, bad_fd, rc = EXIT_SUCCESS;
 	size_t sz;
 	int64_t sendfile_size = DEFAULT_SENDFILE_SIZE;
+	double duration = 0.0, bytes = 0.0, rate;
 
 	if (!stress_get_setting("sendfile-size", &sendfile_size)) {
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
@@ -119,8 +120,15 @@ static int stress_sendfile(const stress_args_t *args)
 
 	do {
 		off_t offset = 0;
+		ssize_t nbytes;
+		double t;
 
-		if (sendfile(fdout, fdin, &offset, sz) < 0) {
+		t = stress_time_now();
+		nbytes = sendfile(fdout, fdin, &offset, sz);
+		if (LIKELY(nbytes >= 0)) {
+			duration += stress_time_now() - t;
+			bytes += (double)nbytes;
+		} else {
 			if (errno == ENOSYS) {
 				if (args->instance == 0)
 					pr_inf_skip("%s: skipping stressor, sendfile not implemented\n",
@@ -173,6 +181,9 @@ static int stress_sendfile(const stress_args_t *args)
 		inc_counter(args);
 	} while (keep_stressing(args));
 
+	rate = (duration > 0.0) ? bytes / duration : 0.0;
+	stress_misc_stats_set(args->misc_stats, 0, "MB per sec sent to /dev/null", rate / (double)MB);
+
 close_out:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 	(void)close(fdout);
@@ -196,9 +207,10 @@ stressor_info_t stress_sendfile_info = {
 };
 #else
 stressor_info_t stress_sendfile_info = {
-	.stressor = stress_not_implemented,
+	.stressor = stress_unimplemented,
 	.class = CLASS_PIPE_IO | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
-	.help = help
+	.help = help,
+	.unimplemented_reason = "built without sys/sendfile.h or sendfile() system call support"
 };
 #endif
