@@ -19,6 +19,7 @@
  */
 #include "stress-ng.h"
 #include "core-arch.h"
+#include "core-cache.h"
 #include "core-cpu.h"
 #include "core-nt-store.h"
 
@@ -37,7 +38,7 @@
 static sigjmp_buf jmp_env;
 #if defined(SA_SIGINFO)
 static volatile void *fault_addr;
-static void *expected_addr;
+static volatile void *expected_addr;
 static volatile int signo;
 static volatile int code;
 #endif
@@ -191,7 +192,16 @@ static void stress_sigsegv_readtsc(void)
 {
 	/* SEGV reading tsc when tsc is not allowed */
 	if (prctl(PR_SET_TSC, PR_TSC_SIGSEGV, 0, 0, 0) == 0) {
-		__asm__ __volatile__("rdtsc\n" : : : "%edx", "%eax");
+#if defined(STRESS_ARCH_X86_64)
+		uint32_t lo, hi;
+
+		__asm__ __volatile__("rdtsc" : "=a" (lo), "=d" (hi));
+#endif
+#if defined(STRESS_ARCH_X86_32)
+		uint64_t tsc;
+
+		__asm__ __volatile__("rdtsc" : "=A" (tsc));
+#endif
 	}
 }
 
@@ -269,7 +279,8 @@ static int stress_sigsegv(const stress_args_t *args)
 	ptr = (uint8_t *)mmap(NULL, args->page_size, PROT_READ,
 		MAP_SHARED | MAP_ANONYMOUS, -1, 0);
 	if (ptr == MAP_FAILED) {
-		pr_inf("%s: mmap of shared read only page failed: %d (%s)\n",
+		pr_inf_skip("%s: mmap of shared read only page failed: "
+			"errno = %d (%s), skipping stressor\n",
 			args->name, errno, strerror(errno));
 		return EXIT_NO_RESOURCE;
 	}
@@ -322,7 +333,7 @@ static int stress_sigsegv(const stress_args_t *args)
 #if defined(SA_SIGINFO)
 			if (verify && expected_addr && fault_addr && fault_addr != expected_addr) {
 				pr_fail("%s: expecting fault address %p, got %p instead\n",
-					args->name, (void *)expected_addr, fault_addr);
+					args->name, (volatile void *)expected_addr, fault_addr);
 			}
 			if (verify &&
 			    (signo != -1) &&
@@ -387,6 +398,7 @@ static int stress_sigsegv(const stress_args_t *args)
 			case 6:
 #if defined(SA_SIGINFO)
 				expected_addr = BAD_ADDR;
+				shim_cacheflush((char *)&expected_addr, (int)sizeof(*expected_addr), SHIM_DCACHE);
 #endif
 				stress_sigsegv_vdso();
 				CASE_FALLTHROUGH;
@@ -394,6 +406,7 @@ static int stress_sigsegv(const stress_args_t *args)
 			default:
 #if defined(SA_SIGINFO)
 				expected_addr = ptr;
+				shim_cacheflush((char *)&expected_addr, (int)sizeof(*expected_addr), SHIM_DCACHE);
 #endif
 				*ptr = 0;
 				break;
