@@ -72,7 +72,7 @@ typedef struct {
 	char sysfs_path[PATH_MAX];	/* path to exercise */
 } stress_ctxt_t;
 
-static uint32_t path_sum(const char *path)
+static inline uint32_t path_sum(const char *path)
 {
 	return stress_hash_x17(path);
 }
@@ -298,8 +298,7 @@ static inline bool stress_sys_rw(stress_ctxt_t *ctxt)
 		{
 			int isz;
 
-			ret = ioctl(fd, FIGETBSZ, &isz);
-			(void)ret;
+			VOID_RET(int, ioctl(fd, FIGETBSZ, &isz));
 		}
 #else
 		UNEXPECTED
@@ -308,8 +307,7 @@ static inline bool stress_sys_rw(stress_ctxt_t *ctxt)
 		{
 			int isz;
 
-			ret = ioctl(fd, FIONREAD , &isz);
-			(void)ret;
+			VOID_RET(int, ioctl(fd, FIONREAD , &isz));
 		}
 #else
 		UNEXPECTED
@@ -341,8 +339,7 @@ err:
 			 */
 			if ((fd = open(path, O_WRONLY | O_NONBLOCK)) < 0)
 				goto next;
-			ret = write(fd, buffer, 0);
-			(void)ret;
+			VOID_RET(ssize_t, write(fd, buffer, 0));
 			(void)close(fd);
 
 			if (stress_time_now() - t_start > threshold)
@@ -368,8 +365,7 @@ err:
 				 *  Accessing ROM memory may be slow,
 				 *  so just do one read for now.
 				 */
-				ret = read(fd, buffer, sizeof(buffer));
-				(void)ret;
+				VOID_RET(ssize_t, read(fd, buffer, sizeof(buffer)));
 
 				/* Disable ROM read */
 				ret = write(fd, "0", 1);
@@ -417,17 +413,26 @@ static void *stress_sys_rw_thread(void *ctxt_ptr)
 	return &nowt;
 }
 
+static const char *sys_skip_paths[] = {
+	"/sys/class/zram-control/hot_add",	/* reading this will add a new zram dev */
+	"/sys/kernel/debug",			/* don't read debug interfaces */
+};
+
 /*
  *  stress_sys_skip()
  *	skip paths that are known to cause issues
  */
 static bool stress_sys_skip(const char *path)
 {
-	/*
-	 *  Skip over debug interfaces
-	 */
-	if (!strncmp(path, "/sys/kernel/debug", 17))
-		return true;
+	size_t i;
+
+	for (i = 0; i < SIZEOF_ARRAY(sys_skip_paths); i++) {
+		const char *skip_path = sys_skip_paths[i];
+		const size_t len = strlen(skip_path);
+
+		if (!strncmp(path, skip_path, len))
+			return true;
+	}
 	/*
 	 *  Can OOPS on Azure when reading
 	 *  "/sys/devices/LNXSYSTM:00/LNXSYBUS:00/PNP0A03:00/device:07/" \
@@ -633,19 +638,16 @@ static int stress_sysfs(const stress_args_t *args)
 				     PROT_READ | PROT_WRITE,
 				     MAP_SHARED | MAP_ANONYMOUS, -1, 0);
 	if (ctxt == MAP_FAILED) {
-		pr_inf("%s: cannot mmap shared context region\n", args->name);
+		pr_inf_skip("%s: cannot mmap shared context region, skipping stressor\n", args->name);
 		return EXIT_NO_RESOURCE;
 	}
 
 	n = scandir("/sys", &dlist, NULL, alphasort);
-	if (n <= 0) {
-		if (args->instance == 0)
-			pr_inf_skip("%s: no /sys entries found, skipping stressor\n", args->name);
-		stress_dirent_list_free(dlist, n);
-		(void)munmap((void *)ctxt, sizeof(*ctxt));
-		return EXIT_NO_RESOURCE;
-	}
+	if (n <= 0)
+		goto exit_no_sysfs_entries;
 	n = stress_dirent_list_prune(dlist, n);
+	if (n <= 0)
+		goto exit_no_sysfs_entries;
 
 	os_release = 0;
 #if defined(HAVE_UNAME) &&	\
@@ -732,8 +734,7 @@ again:
 						args->name, errno, strerror(errno));
 				/* Ring ring, time to die */
 				(void)kill(pid, SIGALRM);
-				ret = shim_waitpid(pid, &status, 0);
-				(void)ret;
+				VOID_RET(int, shim_waitpid(pid, &status, 0));
 			} else {
 				if (stress_sysfs_bad_signal(status)) {
 					pr_inf("%s: killed by %s exercising '%s'\n",
@@ -783,8 +784,7 @@ again:
 				pr_dbg("%s: failed to lock spin lock for sysfs_path\n", args->name);
 			} else {
 				shim_strlcpy(ctxt->sysfs_path, "", sizeof(ctxt->sysfs_path));
-				ret = shim_pthread_spin_unlock(&lock);
-				(void)ret;
+				VOID_RET(int, shim_pthread_spin_unlock(&lock));
 			}
 
 			/* Forcefully kill threads */
@@ -812,10 +812,17 @@ finish:
 		(void)close(ctxt->kmsgfd);
 	(void)shim_pthread_spin_destroy(&lock);
 
+exit_free:
 	stress_dirent_list_free(dlist, n);
 	(void)munmap((void *)ctxt, sizeof(*ctxt));
 
 	return rc;
+
+exit_no_sysfs_entries:
+	if (args->instance == 0)
+		pr_inf_skip("%s: no /sys entries found, skipping stressor\n", args->name);
+	rc = EXIT_NO_RESOURCE;
+	goto exit_free;
 }
 
 stressor_info_t stress_sysfs_info = {
@@ -826,8 +833,9 @@ stressor_info_t stress_sysfs_info = {
 };
 #else
 stressor_info_t stress_sysfs_info = {
-	.stressor = stress_not_implemented,
+	.stressor = stress_unimplemented,
 	.class = CLASS_OS,
-	.help = help
+	.help = help,
+	.unimplemented_reason = "not Linux or built without pthread support"
 };
 #endif
