@@ -44,9 +44,9 @@ UNEXPECTED
 
 static const stress_help_t help[] = {
 	{ NULL,	"timerfd N",	  "start N workers producing timerfd events" },
-	{ NULL,	"timerfd-ops N",  "stop after N timerfd bogo events" },
 	{ NULL, "timerfd-fds N", "number of timerfd file descriptors to open" },
 	{ NULL,	"timerfd-freq F", "run timer(s) at F Hz, range 1 to 1000000000" },
+	{ NULL,	"timerfd-ops N",  "stop after N timerfd bogo events" },
 	{ NULL,	"timerfd-rand",	  "enable random timerfd frequency" },
 	{ NULL,	NULL,		  NULL }
 };
@@ -58,6 +58,8 @@ static const stress_help_t help[] = {
 #elif defined(HAVE_SELECT)
 #define USE_SELECT		(1)
 #define TIMER_FDS_MAX		(FD_SETSIZE)
+#else
+#define TIMER_FDS_MAX		(INT_MAX)
 #endif
 #define TIMER_FDS_DEFAULT	STRESS_MINIMUM(1024, TIMER_FDS_MAX)
 
@@ -428,9 +430,10 @@ stressor_info_t stress_timerfd_info = {
 };
 #else
 stressor_info_t stress_timerfd_info = {
-	.stressor = stress_not_implemented,
+	.stressor = stress_unimplemented,
 	.class = CLASS_INTERRUPT | CLASS_OS,
 	.opt_set_funcs = opt_set_funcs,
-	.help = help
+	.help = help,
+	.unimplemented_reason = "built without sys/timerfd.h, timerfd_create(), timerfd_settime(), timerfd_setime, select() or poll()"
 };
 #endif
