@@ -71,20 +71,21 @@ static int stress_xattr(const stress_args_t *args)
 #if defined(XATTR_SIZE_MAX)
 	large_tmp = calloc(XATTR_SIZE_MAX + 2, sizeof(*large_tmp));
 	if (!large_tmp) {
-		pr_inf("%s: failed to allocate large xattr buffer\n", args->name);
+		pr_inf_skip("%s: failed to allocate large xattr buffer, skipping stressor\n",
+			args->name);
 		return EXIT_NO_RESOURCE;
 	}
 #endif
 
 	ret = stress_temp_dir_mk_args(args);
 	if (ret < 0) {
-		rc = exit_status(-ret);
+		rc = stress_exit_status(-ret);
 		goto out_free;
 	}
 
 	(void)stress_temp_filename_args(args, filename, sizeof(filename), stress_mwc32());
 	if ((fd = open(filename, O_CREAT | O_RDWR, S_IRUSR | S_IWUSR)) < 0) {
-		rc = exit_status(errno);
+		rc = stress_exit_status(errno);
 		pr_fail("%s: open %s failed, errno=%d (%s)\n",
 			args->name, filename, errno, strerror(errno));
 		goto out;
@@ -467,29 +468,23 @@ static int stress_xattr(const stress_args_t *args)
 		/*
 		 *  Exercise invalid filename, ENOENT
 		 */
-		ret = shim_removexattr("", "user.var_1234");
-		(void)ret;
-		ret = shim_lremovexattr("", "user.var_1234");
-		(void)ret;
+		VOID_RET(int, shim_removexattr("", "user.var_1234"));
+		VOID_RET(int, shim_lremovexattr("", "user.var_1234"));
 
 #if defined(XATTR_SIZE_MAX)
 		/*
 		 *  Exercise long attribute, ERANGE
 		 */
 		(void)memset(large_tmp, 'X', XATTR_SIZE_MAX + 1);
-		ret = shim_removexattr(filename, large_tmp);
-		(void)ret;
-		ret = shim_lremovexattr(filename, large_tmp);
-		(void)ret;
-		ret = shim_fremovexattr(fd, large_tmp);
-		(void)ret;
+		VOID_RET(int, shim_removexattr(filename, large_tmp));
+		VOID_RET(int, shim_lremovexattr(filename, large_tmp));
+		VOID_RET(int, shim_fremovexattr(fd, large_tmp));
 #endif
 
 		/*
 		 *  Exercise bad/invalid fd
 		 */
-		ret = shim_fremovexattr(bad_fd, "user.var_bad");
-		(void)ret;
+		VOID_RET(int, shim_fremovexattr(bad_fd, "user.var_bad"));
 
 #if defined(HAVE_LLISTXATTR)
 		sret = shim_llistxattr(filename, NULL, 0);
@@ -527,8 +522,9 @@ stressor_info_t stress_xattr_info = {
 };
 #else
 stressor_info_t stress_xattr_info = {
-	.stressor = stress_not_implemented,
+	.stressor = stress_unimplemented,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
-	.help = help
+	.help = help,
+	.unimplemented_reason = "built without sys/xattr.h or attr/xattr.h and xattr family of system calls"
 };
 #endif
