@@ -1,5 +1,6 @@
 /*
- * Copyright (C) 2013-2017 Canonical, Ltd.
+ * Copyright (C) 2013-2021 Canonical, Ltd.
+ * Copyright (C)      2022 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -15,36 +16,37 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * This code is a complete clean re-write of the stress tool by
- * Colin Ian King <colin.king@canonical.com> and attempts to be
- * backwardly compatible with the stress tool by Amos Waterland
- * <apw@rossby.metr.ou.edu> but has more stress tests and more
- * functionality.
- *
  */
 #include "stress-ng.h"
 
-#if defined(_POSIX_PRIORITY_SCHEDULING) && !defined(__minix__)
+static const stress_help_t help[] = {
+	{ "y N", "yield N",	"start N workers doing sched_yield() calls" },
+	{ NULL,	 "yield-ops N",	"stop after N bogo yield operations" },
+	{ NULL,	 NULL,		NULL }
+};
+
+#if defined(_POSIX_PRIORITY_SCHEDULING) &&	\
+    !defined(__minix__)
 
 /*
  *  stress on sched_yield()
  *	stress system by sched_yield
  */
-int stress_yield(const args_t *args)
+static int stress_yield(const stress_args_t *args)
 {
 	uint64_t *counters;
 	uint64_t max_ops_per_yielder;
-	size_t counters_sz, yielders_sz;
+	size_t counters_sz;
 	int32_t cpus = stress_get_processors_configured();
-	const size_t instances = args->num_instances;
-	size_t yielders = 2;
-#if defined(__linux__) && NEED_GLIBC(2,3,0)
+	const uint32_t instances = args->num_instances;
+	uint32_t yielders = 2;
+#if defined(HAVE_SCHED_GETAFFINITY)
 	cpu_set_t mask;
 #endif
 	pid_t *pids;
 	size_t i;
 
-#if defined(__linux__) && NEED_GLIBC(2,3,0)
+#if defined(HAVE_SCHED_GETAFFINITY)
 	/*
 	 *  If the process is limited to a subset of cores
 	 *  then make sure we do not create too many yielders
@@ -54,11 +56,13 @@ int stress_yield(const args_t *args)
 			PRId32 " yielder%s (instance %" PRIu32 ")\n",
 			args->name, cpus, (cpus == 1) ? "" : "s", args->instance);
 	} else {
-		if (CPU_COUNT(&mask) < cpus)
-			cpus = CPU_COUNT(&mask);
-		pr_inf("%s: limiting to %" PRId32 " yielder%s (instance %"
+		if (CPU_COUNT(&mask) < (int)cpus)
+			cpus = (int32_t)CPU_COUNT(&mask);
+		pr_inf("%s: limiting to %" PRId32 " child yielder%s (instance %"
 			PRIu32 ")\n", args->name, cpus, (cpus == 1) ? "" : "s", args->instance);
 	}
+#else
+	UNEXPECTED
 #endif
 
 	/*
@@ -71,26 +75,27 @@ int stress_yield(const args_t *args)
 		if (yielders < 1)
 			yielders = 1;
 		if (!args->instance) {
-			int32_t residual = cpus - (yielders * instances);
+			/* residual may be -ve, ensure it is signed */
+			int32_t residual = cpus - (int32_t)(yielders * instances);
+
 			if (residual > 0)
 				yielders += residual;
 		}
 	}
 
 	max_ops_per_yielder = args->max_ops / yielders;
-	yielders_sz = yielders * sizeof(pid_t);
-	pids = calloc(yielders, sizeof(pid_t));
+	pids = calloc(yielders, sizeof(*pids));
 	if (!pids) {
-		pr_err("%s: calloc failed\n", args->name);
+		pr_inf_skip("%s: calloc failed allocating %" PRIu32
+			" pids, skipping stressor\n", args->name, yielders);
 		return EXIT_NO_RESOURCE;
 	}
-	(void)memset(pids, 0, yielders_sz);
 
-	counters_sz = yielders * sizeof(uint64_t);
+	counters_sz = yielders * sizeof(*counters);
 	counters = (uint64_t *)mmap(NULL, counters_sz, PROT_READ | PROT_WRITE,
 		MAP_SHARED | MAP_ANONYMOUS, -1, 0);
 	if (counters == MAP_FAILED) {
-		int rc = exit_status(errno);
+		int rc = stress_exit_status(errno);
 
 		pr_err("%s: mmap failed: errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
@@ -99,43 +104,54 @@ int stress_yield(const args_t *args)
 	}
 	(void)memset(counters, 0, counters_sz);
 
-	for (i = 0; g_keep_stressing_flag && (i < yielders); i++) {
+	stress_set_proc_state(args->name, STRESS_STATE_RUN);
+
+	for (i = 0; keep_stressing_flag() && (i < yielders); i++) {
 		pids[i] = fork();
 		if (pids[i] < 0) {
 			pr_dbg("%s: fork failed (instance %" PRIu32
 				", yielder %zd): errno=%d (%s)\n",
 				args->name, args->instance, i, errno, strerror(errno));
 		} else if (pids[i] == 0) {
-			(void)setpgid(0, g_pgrp);
 			stress_parent_died_alarm();
+			(void)sched_settings_apply(true);
 
 			do {
 				int ret;
 
 				ret = shim_sched_yield();
 				if ((ret < 0) && (g_opt_flags & OPT_FLAGS_VERIFY))
-					pr_fail_err("sched_yield");
+					pr_fail("%s: sched_yield failed, errno=%d (%s)\n",
+						args->name, errno, strerror(errno));
 				counters[i]++;
-			} while (g_keep_stressing_flag && (!max_ops_per_yielder || *args->counter < max_ops_per_yielder));
+			} while (keep_stressing_flag() && (!max_ops_per_yielder || (counters[i] < max_ops_per_yielder)));
 			_exit(EXIT_SUCCESS);
 		}
 	}
 
 	do {
-		*args->counter = 0;
-		(void)pause();
+		set_counter(args, 0);
+#if defined(__FreeBSD__)
+		VOID_RET(int, shim_sched_yield());
+#else
+		VOID_RET(int, shim_usleep(100000));
+#endif
+
 		for (i = 0; i < yielders; i++)
-			*args->counter += counters[i];
-	} while (keep_stressing());
+			add_counter(args, counters[i]);
+	} while (keep_stressing(args));
 
 	/* Parent, wait for children */
+	set_counter(args, 0);
+
+	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 	for (i = 0; i < yielders; i++) {
 		if (pids[i] > 0) {
 			int status;
 
 			(void)kill(pids[i], SIGKILL);
-			(void)waitpid(pids[i], &status, 0);
-			*args->counter += counters[i];
+			(void)shim_waitpid(pids[i], &status, 0);
+			add_counter(args, counters[i]);
 		}
 	}
 	(void)munmap((void *)counters, counters_sz);
@@ -143,9 +159,18 @@ int stress_yield(const args_t *args)
 
 	return EXIT_SUCCESS;
 }
+
+stressor_info_t stress_yield_info = {
+	.stressor = stress_yield,
+	.class = CLASS_SCHEDULER | CLASS_OS,
+	.verify = VERIFY_OPTIONAL,
+	.help = help
+};
 #else
-int stress_yield(const args_t *args)
-{
-	return stress_not_implemented(args);
-}
+stressor_info_t stress_yield_info = {
+	.stressor = stress_unimplemented,
+	.class = CLASS_SCHEDULER | CLASS_OS,
+	.help = help,
+	.unimplemented_reason = "built without scheduling support"
+};
 #endif
